/*megaide_adapter.h*/
/*
 * Linux MegaRAID IDEal Software RAID Stack
 * Copyright (c) 2001 LSI Logic Corporation, All Rights Reserved.
 *-----------------------------------------------------------------------------
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *-----------------------------------------------------------------------------
 */

#ifndef __MEGAIDE_ADAPTER_H__
#define __MEGAIDE_ADAPTER_H__

/*Macros Definition*/
#define DELL_SUPPORT 1
#define HYPERDISK_MAJOR_VERSION 0x05
#define HYPERDISK_MINOR_VERSION 0x06
#define HYPERDISK_BUILD_VERSION 0x07
#define MEGAIDE_VERSION \
         "driver version 05.06g (Date: Nov 24, 2003; 11:19 EST)"

#define M_IDE_CRLFSTR                   "\n"
#define M_IDE_HOST_INITIATOR_ID         0xFF
#define M_IDE_MAX_SGLIST                32        /*128K transfer*/
#define M_IDE_MAX_CMD_PER_LUN           16
#define M_IDE_MAX_CHANNELS              1
#define M_IDE_MAX_TARGET_PER_CHAN       4
#define M_IDE_MAX_LOGICAL_DRIVES        8

#define M_IDE_MAX_CMD_CAN_QUEUE         \
         (M_IDE_MAX_CMD_PER_LUN * M_IDE_MAX_LOGICAL_DRIVES)

#define M_IDE_TOTAL_COMMANDS \
         ((M_IDE_MAX_CMD_PER_LUN * M_IDE_MAX_LOGICAL_DRIVES) + \
         10/*for gate keeper*/)

#define M_IDE_BLOCK_CMD                 0x08000000L
#define M_IDE_REQUEST_IRCD_UPDATE       0x04000000L
#define M_IDE_REQUEST_ERRORREC          0x02000000L
#define M_IDE_REQUEST_LOGERROR          0x01000000L

#define M_IDE_FOR_CHANNEL2              0x00000002L
#define M_IDE_FOR_CHANNEL1              0x00000001L

#define MAX_RAID1_DRIVES                2
#define MAX_THRESHOLD_COMMANDS          5


/*IDE RAID Signature Related definitions*/
#define IDE_RAID_SIGNATURE              "$XIDE$"
#define IDE_RAID_SIGNATURE_LENGTH       7
/*new IRCD*/
#define NEW_IDE_RAID_SIGNATURE          "$_IDE$"

#ifndef PCI_VENDOR_ID_DELL
 #define PCI_VENDOR_ID_DELL             0x1028
#endif

#ifndef PCI_VENDOR_ID_CMD
 #define PCI_VENDOR_ID_CMD              0x1095
#endif

#ifndef PCI_VENDOR_ID_SERVERWORKS
 #define PCI_VENDOR_ID_SERVERWORKS      0x1166
#endif

#ifndef PCI_DEVICE_ID_CMD_648
 #define PCI_DEVICE_ID_CMD_648          0x0648
#endif

#ifndef PCI_DEVICE_ID_CMD_649
 #define PCI_DEVICE_ID_CMD_649          0x0649
#endif

#ifndef PCI_DEVICE_ID_CMD_680
 #define PCI_DEVICE_ID_CMD_680          0x0680
#endif

//Device change as requested by Dell
#ifndef PCI_DEVICE_ID_DELL_680
 #define PCI_DEVICE_ID_DELL_680          0x1028
#endif

#ifndef PCI_DEVICE_ID_YELLOWSTONE
 #define PCI_DEVICE_ID_YELLOWSTONE      0x1680
#endif

#ifndef PCI_DEVICE_ID_DELL_CERC_ATA_AS
 #define PCI_DEVICE_ID_DELL_CERC_ATA_AS 0x013B
#endif

#ifndef PCI_DEVICE_ID_CMD_3112
 #define PCI_DEVICE_ID_CMD_3112         0x3112
#endif

#ifndef PCI_DEVICE_ID_SERVERWORKS_CSB5IDE
 #define PCI_DEVICE_ID_SERVERWORKS_CSB5IDE       0x0212
#endif

#ifndef PCI_DEVICE_ID_SERVERWORKS_CSB6IDE
 #define PCI_DEVICE_ID_SERVERWORKS_CSB6IDE       0x0213
#endif

#ifndef PCI_DEVICE_ID_LSI_524
 #define PCI_DEVICE_ID_LSI_524          0x0524
#endif

#ifndef PCI_VENDOR_ID_LSI_LOGIC
 #define PCI_VENDOR_ID_LSI_LOGIC        0x1000
#endif

#ifndef PCI_VENDOR_ID_COMPAQ
 #define PCI_VENDOR_ID_COMPAQ           0x0E11
#endif

#ifndef PCI_DEVICE_ID_COMPAQ_CSB6
 #define PCI_DEVICE_ID_COMPAQ_CSB6      0x00E8
#endif

#ifndef PCI_DEVICE_ID_IBM_CSB6
 #define PCI_DEVICE_ID_IBM_CSB6         0x0213
#endif

#ifndef PCI_VENDOR_ID_INTEL
 #define PCI_VENDOR_ID_INTEL            0x8086
#endif

#ifndef PCI_DEVICE_ID_INTEL_ICH5R
 #define PCI_DEVICE_ID_INTEL_ICH5R      0x24DF
#endif

#ifndef PCI_DEVICE_ID_INTEL_HR
 #define PCI_DEVICE_ID_INTEL_HR         0x25b0
#endif

#define  AR_TRUE      1
#define  AR_FALSE     0

#define  AR_SUCCESSS  1
#define  AR_ERROR     0

/*Maximum supported Drives and channels and controllers*/
#define MAX_CONTROLLERS              2
#define MAX_CHANNELS_PER_CONTROLLER  2
#define MAX_DRIVES_PER_CHANNEL       2
#define MAX_DRIVES_PER_CONTROLLER \
         (MAX_CHANNELS_PER_CONTROLLER * MAX_DRIVES_PER_CHANNEL)

/*Bytes in an IDE disk sector.*/
#define IDE_SECTOR_SIZE                 512

/* Memory page size.*/
#ifndef PAGE_SIZE
 #define PAGE_SIZE                      0x1000
#endif

#define MAX_STRIPES_PER_ROW             MAX_DRIVES_PER_CONTROLLER

/* Minimum stripe size supported.*/
#define MIN_STRIPE_SIZE_IN_SECTORS      32

/* Memory page size in sectors.*/
#define PAGE_SIZE_IN_SECTORS            (PAGE_SIZE/IDE_SECTOR_SIZE)


/*Maximum number of SGL entries.*/
#define MAX_SGL_ENTRIES_PER_PHYSICAL_DRIVE ( \
                (((MAX_SECTORS_PER_IDE_TRANSFER) / \
                (PAGE_SIZE_IN_SECTORS)) + 1 \
                + 1)   /* used for alignment adjustments */ \
                 * 2)  /* guarantees we have enough entries in 4KB region */


/* SGL physical boundary.*/
#define SGL_HW_BOUNDARY                 (4*1024)

/* Memory region boundary.*/
#define REGION_HW_BOUNDARY              (64*1024)

/* Maximum number of sectors accepted by IDE devices in a transfer.*/
#define MAX_SECTORS_PER_IDE_TRANSFER    256

/*
 * Maximum number of PDDs that can be derived from an SRB.
 * (For logical drives, SRBs are split into one or more PDDs per
 * physical drive in the set).
 * This number is used to request the allocation of the proper
 * SRB extension size.
 */

#define MAX_PDDS_PER_SRB                MAX_STRIPES_PER_ROW

/*End-of-List marker. Used for array index fields */
#define        END_OF_LIST_FLAG         0x80000000

#define PRIAMRY_CHANNEL_INTERRUPT       0x04
#define CHAN2_INTERRUPT        0x08

/*Flag Definitions*/
#define  QUEUE_AT_TAIL           0
#define  QUEUE_AT_FRONT          1

#define  SRB_STATUS_SUCCESS      0
#define  SRB_STATUS_PENDING      1
#define  SRB_STATUS_ERROR        2

/**
 * list_for_each_safe	-	iterate over a list safe against removal of list entry
 * @pos:	the &struct list_head to use as a loop counter.
 * @n:		another &struct list_head to use as temporary storage
 * @head:	the head for your list.
 */
#ifndef list_for_each_safe
 #define list_for_each_safe(pos, n, head) \
	for (pos = (head)->next, n = pos->next; pos != (head); \
		pos = n, n = pos->next)
#endif 

#ifndef MODULE_LICENSE
 #define MODULE_LICENSE(a)
#endif
		
/*
 * Inquiry buffer structure. This is the data returned from the target
 * after it receives an inquiry.
 *
 * This structure may be extended by the number of bytes specified
 * in the field AdditionalLength. The defined size constant only
 * includes fields through ProductRevisionLevel.
 *
 * The NT SCSI drivers are only interested in the first 36 bytes of data.
 */

#define INQUIRYDATABUFFERSIZE 36

typedef struct _INQUIRYDATA {
  UCHAR   DeviceType : 5;
  UCHAR   DeviceTypeQualifier : 3;
  UCHAR   DeviceTypeModifier : 7;
  UCHAR   RemovableMedia : 1;
  UCHAR   Versions;
  UCHAR   ResponseDataFormat;
  UCHAR   AdditionalLength;
  UCHAR   Reserved[2];
  UCHAR   SoftReset : 1;
  UCHAR   CommandQueue : 1;
  UCHAR   Reserved2 : 1;
  UCHAR   LinkedCommands : 1;
  UCHAR   Synchronous : 1;
  UCHAR   Wide16Bit : 1;
  UCHAR   Wide32Bit : 1;
  UCHAR   RelativeAddressing : 1;
  UCHAR   VendorId[8];
  UCHAR   ProductId[16];
  UCHAR   ProductRevisionLevel[4];
  UCHAR   VendorSpecific[20];
  UCHAR   Reserved3[40];
} INQUIRYDATA, *PINQUIRYDATA;

/*Read Capacity Data - returned in Big Endian format*/
typedef struct _READ_CAPACITY_DATA {
  ULONG  LogicalBlockAddress;
  ULONG  BytesPerBlock;
} READ_CAPACITY_DATA, *PREAD_CAPACITY_DATA;


#ifdef  ENABLE_ERROR_INJECTOR
 #define DOUBLE_ERROR        0
 #define WRITE_OTHER_ERROR   1
 #define READ_ERROR_WRITE    2
 #define MAX_STATE           6
 #define MAX_PATTERN         10
 #define CLEAR_ERROR         0x8000

typedef struct  _ErrorInjector{
  ULONG  MedErrorStartSector;
  ULONG  MedErrorEndSector;
  ULONG  MedErrorFrequnecy;
  ULONG  MedErrorRemainder;
  BOOLEAN  InJectDoubleError;
  ULONG  DoubleErrorSector;
}PDErrorInjectorInfo;

typedef struct _ErrorPatternData {
  ULONG sector;
  BOOLEAN isWrite;
  ULONG pattern[MAX_STATE];
} ErrorPatternData;
#endif

#endif /*__MEGAIDE_ADAPTER_H__*/

/*vi set ts=8:*/



