/*megaide_ata.h*/
/*
 * Linux MegaRAID IDEal Software RAID Stack
 * Copyright (c) 2001 LSI Logic Corporation, All Rights Reserved.
 *-----------------------------------------------------------------------------
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *-----------------------------------------------------------------------------
 */

#ifndef __MEGAIDE_ATA_H__
#define __MEGAIDE_ATA_H__

/*Device selection constants used for programming
 *the device/head register.
 */
#define IDE_LBA_MODE  0xE0
#define IDE_CHS_MODE  0xA0

/*IDE status definitions*/
#define IDE_STATUS_NULL    0x00
#define IDE_STATUS_ERROR   0x01
#define IDE_STATUS_INDEX   0x02
#define IDE_STATUS_CORRECTED_ERROR   0x04
#define IDE_STATUS_DRQ     0x08
#define IDE_STATUS_DSC     0x10
#define IDE_STATUS_DRDY    0x40
#define IDE_STATUS_IDLE    0x50
#define IDE_STATUS_BUSY    0x80

/*IDE command definitions*/
#define IDE_COMMAND_ATAPI_RESET         0x08
#define IDE_COMMAND_RECALIBRATE         0x10
#define IDE_COMMAND_READ                0x20
#define IDE_COMMAND_READ_SECTORS_EXT    0x24
#define IDE_COMMAND_READ_DMA_EXT        0x25
#define IDE_COMMAND_READ_DMA_QUEUED_EXT 0x26
#define IDE_COMMAND_READ_NATIVE_MAX_ADDRESS_EXT 0x27
#define IDE_COMMAND_READ_MULTIPLE_EXT   0x29
#define IDE_COMMAND_WRITE_SECTORS_EXT   0x34
#define IDE_COMMAND_WRITE_DMA_EXT       0x35
#define IDE_COMMAND_WRITE_DMA_QUEUED_EXT 0x36
#define IDE_COMMAND_SET_MAX_ADDRESS_EXT 0x37
#define IDE_COMMAND_WRITE_MULTIPLE_EXT  0x39
#define IDE_COMMAND_WRITE               0x30
#define IDE_COMMAND_VERIFY              0x40
#define IDE_COMMAND_READ_VERIFY_SECTORS_EXT 0x42
#define IDE_COMMAND_SEEK                0x70
#define IDE_COMMAND_SET_DRIVE_PARAMETERS 0x91
#define IDE_COMMAND_ATAPI_PACKET        0xA0
#define IDE_COMMAND_ATAPI_IDENTIFY      0xA1
#define IDE_COMMAND_EXECUTE_SMART       0xB0
#define IDE_COMMAND_READ_MULTIPLE       0xC4
#define IDE_COMMAND_WRITE_MULTIPLE      0xC5
#define IDE_COMMAND_SET_MULTIPLE        0xC6
#define IDE_COMMAND_READ_DMA            0xC8
#define IDE_COMMAND_WRITE_DMA           0xCA
#define IDE_COMMAND_GET_MEDIA_STATUS    0xDA
#define IDE_COMMAND_FLUSH_CACHE         0xE7
#define IDE_COMMAND_FLUSH_CACHE_EXT     0xEA
#define IDE_COMMAND_IDENTIFY            0xEC
#define IDE_COMMAND_MEDIA_EJECT         0xED
#define IDE_COMMAND_SET_FEATURES        0xEF
#define IDE_COMMAND_ENABLE_MEDIA_STATUS 0xEF /*A set feature command*/

#define MAX_TIME_OUT_ERROR_COUNT        5

/*Device Extension Device Flags*/

/*Indicates that some device is present.*/
#define DFLAGS_DEVICE_PRESENT        (1 << 0)

/*Indicates whether Atapi commands can be used.*/
#define DFLAGS_ATAPI_DEVICE          (1 << 1)

/*Indicates whether this is a tape device.*/
#define DFLAGS_TAPE_DEVICE           (1 << 2)

/*Indicates whether device interrupts as DRQ*/
#define DFLAGS_INT_DRQ               (1 << 3)

/*is set after receiving Atapi Packet Command.*/
/*Indicates that the drive has the 'removable'*/
#define DFLAGS_REMOVABLE_DRIVE       (1 << 4)

/*bit set in identify data (offset 128).*/
/*Media status notification enabled*/
#define DFLAGS_MEDIA_STATUS_ENABLED  (1 << 5)

/*Indicates whether device can use DMA*/
#define DFLAGS_USE_DMA               (1 << 9)

/*support LBA addressing*/
#define DFLAGS_LBA                   (1 << 10)

/*Indicates that the init path for multi-lun*/
#define DFLAGS_MULTI_LUN_INITED      (1 << 11)

/*has already been done.*/
/*Device support media status notification*/
#define DFLAGS_MSN_SUPPORT           (1 << 12)

/*bootup default enables auto eject*/
#define DFLAGS_AUTO_EJECT_ZIP        (1 << 13)

/*Indicates that unit is WD-Mode(not SFF-Mode).*/
#define DFLAGS_WD_MODE               (1 << 14)

/*Indicates that unit uses*/
#define DFLAGS_LS120_FORMAT          (1 << 15)

/*ATAPI_LS120_FORMAT_UNIT to format.*/
/*Indicates whether device can use UDMA*/
#define DFLAGS_USE_UDMA              (1 << 16)

/* this handles the case of power removal */
#define DFLAGS_UPDATE_IRCD           (1 << 17)

/*Indicates atapi 2.5 changer present.*/
#define DFLAGS_ATAPI_CHANGER         (1 << 29)

/*Indicates multi-platter device, not*/
#define DFLAGS_SANYO_ATAPI_CHANGER   (1 << 30)

/*conforming to the 2.5 spec Indicates that the init path
 *for changers has already been done.*/
#define DFLAGS_CHANGER_INITED        (1 << 31)
#define DFLAGS_USES_EITHER_DMA       0x10200

/*Types of interrupts expected.*/
#define IDE_SEEK_INTERRUPT       (1 << 0)
#define IDE_PIO_INTERRUPT        (1 << 1)
#define IDE_DMA_INTERRUPT        (1 << 2)
#define ANY_DMA_INTERRUPT        (IDE_DMA_INTERRUPT)

/*DMA transfer control */
#define READ_TRANSFER        0x08
#define WRITE_TRANSFER       0x00
#define STOP_TRANSFER        0x00
#define START_TRANSFER       0x01

/*UDMA Timing values for Intel PIIX4.
 *To be programmed in the UDMA Timing Register.
 */
#define UDMA_MODE0_TIMING 0
#define UDMA_MODE1_TIMING 1
#define UDMA_MODE2_TIMING 2
#define UDMA_MODE3_TIMING 3
#define UDMA_MODE4_TIMING 4

/*DMA modes to be used as index into the IO_TIMING_TABLE.*/
#define DMA_MODE0 2
#define DMA_MODE1 3
#define DMA_MODE2 4
#define DMA_MODE3 4       /*Not sure what to fill with*/
#define DMA_MODE4 4       /*Not sure what to fill with*/

/*Advanced PIO modes to be used as index into the IO_TIMING_TABLE*/
#define PIO_MODE0 0
#define PIO_MODE2 2
#define PIO_MODE3 3
#define PIO_MODE4 4

/*PIO_MODE0 timing*/
#define PIO_MODE0_TIMING 900

/*IDENTIFY Advanced PIO Modes*/
#define IDENTIFY_PIO_MODE3 (1 << 0)
#define IDENTIFY_PIO_MODE4 (1 << 1)

/*IDENTIFY Multi-word DMA modes*/
#define IDENTIFY_DMA_MODE0 (1 << 0)
#define IDENTIFY_DMA_MODE1 (1 << 1)
#define IDENTIFY_DMA_MODE2 (1 << 2)
#define IDENTIFY_DMA_MODE3 (1 << 3)
#define IDENTIFY_DMA_MODE4 (1 << 4)


/*IDENTIFY UDMA modes*/
#define IDENTIFY_UDMA_MODE0 (1 << 0)
#define IDENTIFY_UDMA_MODE1 (1 << 1)
#define IDENTIFY_UDMA_MODE2 (1 << 2)
#define IDENTIFY_UDMA_MODE3 (1 << 3)
#define IDENTIFY_UDMA_MODE4 (1 << 4)
#define IDENTIFY_UDMA_MODE5 (1 << 5)
#define IDENTIFY_UDMA_MODE6 (1 << 6)

/*IDENTIFY DMA timing cycle modes */
#define IDENTIFY_DMA_MODE0_TIMING 0x00
#define IDENTIFY_DMA_MODE1_TIMING 0x01
#define IDENTIFY_DMA_MODE2_TIMING 0x02
#define IDENTIFY_DMA_MODE3_TIMING 0x03
#define IDENTIFY_DMA_MODE4_TIMING 0x04

/*Value of bit 1 of TranslationFieldsValid in the IDENTIFY_DATA structure.
 *PIO mode 3+ or DMA modes supported*/

#define IDENTIFY_FAST_TRANSFERS_SUPPORTED 2

/*IDENTIFY capability bit definitions*/
#define IDENTIFY_CAPABILITIES_DMA_SUPPORTED 0x0100
#define IDENTIFY_CAPABILITIES_LBA_SUPPORTED 0x0200

/* Set Features register definitions*/
#define FEATURE_ENABLE_WRITE_CACHE          0x02
#define FEATURE_SET_TRANSFER_MODE           0x03
#define FEATURE_KEEP_CONFIGURATION_ON_RESET 0x66
#define FEATURE_ENABLE_READ_CACHE           0xAA
#define FEATURE_DISABLE_WRITE_CACHE         0x82
#define FEATURE_DISABLE_READ_CACHE          0x55



/*Interrupt Status Bits to be read from either 71h of PCI
 *Config Space or In (BM Base Register + 1) for Primary Channel
 */
#define CHAN1_INTERRUPT                 0x04
#define CHAN2_INTERRUPT                 0x08
#define ANY_CHANNEL_INTERRUPT           (CHAN1_INTERRUPT | CHAN2_INTERRUPT)
#define CHANNEL_INTERRUPT_SERVERWORKS   0x04
#define PRIMARY_CHAN_INTERRUPT_680      0x08
#define SECONDARY_CHAN_INTERRUPT_680    0x08

#define SIL_680_BASE_REG_5_DEPTH        0xFF
#define SIL_3112_BASE_REG_5_DEPTH       0x1FF

/*SStatus Values for Detecting the power Mangement mode*/
#define PM_MODE_SSTATUS_NODEV     0x00 // No Device present value in SStatus.
#define PM_MODE_SSTATUS_ACTIVE    0x01 // Link in ACTIVE-No PM mode.
#define PM_MODE_SSTATUS_PARTIAL   0x02 // Partial Mode PM value in SStatus.
#define PM_MODE_SSTATUS_SLUMBER   0x06 // Slumber mode PM value in SStatus.


#define GET_SECTOR_6BYTE_CMND(cmnd) (ULONG)cmnd[4]

#define GET_START_SECTOR_6BYTE_CMND(cmnd) ((ULONG) cmnd[1] << 16) | \
                                        ((ULONG) cmnd[2] << 8) | \
                                        (ULONG) cmnd[3]

#define GET_SECTOR_10BYTE_CMND(cmnd) (ULONG) cmnd[8] | \
                                      ((ULONG) cmnd[7] << 8)

#define GET_START_SECTOR_10BYTE_CMND(cmnd)  ((ULONG) cmnd[2] << 24) |\
                                        ((ULONG) cmnd[3] << 16) |\
                                        ((ULONG) cmnd[4] << 8) |\
                                        (ULONG) cmnd[5];

/*Register definitions for Sil 680*/
#define SUBSYS_ID_SUBSYS_VEN_ID_REG680      0x2C
#define EXPANSION_ROM_BASE_ADDR_REG680      0x30
#define CAPABILITIES_POINTER_REG680         0x34
#define MAX_MIN_LAT_INT_PIN_INT_LINE_REG680 0x3C
#define CONFIGURATION_REG680             0x40
#define SOFTWARE_DATA_REG680             0x44
#define POWER_MGMNT_CAPBLTY_REG680       0x60
#define POWER_MGMNT_CNTRL_STAT_REG680    0x64
#define PCI_BUS_MASTER_IDEO_REG680       0x70
#define PRD_TABLE_ADDR_IDE0_REG680       0x74
#define PCI_BUS_MASTER_IDE1_REG680       0x78
#define PRD_TABLE_ADDR_IDE1_REG680       0x7C
#define DATA_TRFR_MODE_IDE0_REG680       0x80
#define DATA_TRFR_MODE_IDE1_REG680       0x84
#define SYS_CONF_STAT_CMD_REG680         0x88
#define SYS_SOFTWARE_DATA_REG680         0x8C
#define FLASH_MEM_ADDR_CMD_STAT_REG680   0x90
#define FLASH_MEM_DATA_REG680            0x94
#define EEPROM_MEM_ADDR_CMD_STAT_REG680  0x98
#define EEPROM_MEM_DATA_REG680           0x9C
#define IDE0_TFT_CONF_STAT_REG680        0xA0
#define IDE0_PIO_TIMING_REG680           0xA4
#define IDE0_DMA_TIMING_REG680           0xA8
#define IDE0_UDMA_TIMING_REG680          0xAC
#define IDE1_TFT_CONF_STAT_REG680        0xB0
#define IDE1_PIO_TIMING_REG680           0xB4
#define IDE1_DMA_TIMING_REG680           0xB8
#define IDE1_UDMA_TIMING_REG680          0xBC

// athena Specific
#define ATHENA_PRIMARY_BM_ENABLE_OFFSET    0x10
#define ATHENA_SECONDARY_BM_ENABLE_OFFSET  0x18

/*IDE controller speed definition.*/
typedef enum {
  Udma33,
  Udma66,
  Udma100,
  Udma133 /*UDMA6*/
} CONTROLLER_SPEED, *PCONTROLLER_SPEED;

/*IDE register definition*/
typedef struct _IDE_REGISTERS_1 {
  USHORT  Data;
  UCHAR  SectorCount;
  UCHAR  SectorNumber;
  UCHAR  CylinderLow;
  UCHAR  CylinderHigh;
  UCHAR  DriveSelect;
  UCHAR  Command;
} IDE_REGISTERS_1, *PIDE_REGISTERS_1;

typedef struct _IDE_REGISTERS_2 {
  UCHAR  ucReserved1;
  UCHAR  ucReserved2;
  UCHAR  AlternateStatus;  /* This will itself behave as
                          * "DriveAddress" when it is
                          * used as out port*/
  UCHAR  ucReserved3;
} IDE_REGISTERS_2, *PIDE_REGISTERS_2;

typedef struct _IDE_REGISTERS_3 {
  ULONG  Data;
  UCHAR  Others[4];
} IIDE_REGISTERS_3, *PIDE_REGISTERS_3;

/* PIIX4 BMICX - Bus Master IDE Command Register.
 * IDE Controller I/O Space offset:
 * Primary Channel Base + 00h, Secondary Channel Base + 08h
 * Default value: 00h
 * Attribute: R/W
 */
typedef union _BM_COMMAND_REGISTER {
  UCHAR  AsUchar;
  struct {
     UCHAR  StartStopBm:1;       /* SSMB*/
     UCHAR  Reserved:2;
     UCHAR  BmReadWriteControl:1;/* RWCON - Indicates the direction of a DMA*/
     UCHAR  Reserved2:4;
  }f;
} BM_COMMAND_REGISTER, *PBM_COMMAND_REGISTER;

/* PIIX4 BMISX - Bus Master IDE Status Register.
 * IDE Controller I/O Space offset: Primary Channel Base + 02h,
 * Secondary Channel Base + 0Ah
 * Default value: 00h
 * Attribute: R/W Clear
 */
typedef union _BM_STATUS_REGISTER {
  UCHAR  AsUchar;
  struct {
    UCHAR  BmActive:1;       /* BMIDEA - Read-Only*/
    UCHAR  DmaError:1;       /* (Read/Write-Clear)*/
    UCHAR  InterruptStatus:1;/* IDEINTS - Read/Write-Clear*/
    UCHAR  Reserved:2;
    UCHAR  Drive0DmaCapable:1;/* DMA0CAP - R/W - Software controlled.*/
    UCHAR  Drive1DmaCapable:1;/* DMA1CAP - R/W - Software controlled.*/
    UCHAR  Reserved2:1;
  }f;
} BM_STATUS_REGISTER, *PBM_STATUS_REGISTER;

/*Bus Master Registers*/
typedef struct _BM_REGISTERS {
  BM_COMMAND_REGISTER  Command;
  UCHAR  Reserved;
  BM_STATUS_REGISTER  Status;
  UCHAR  Reserved2;
  ULONG  SglAddress;
} BM_REGISTERS, *PBM_REGISTERS;

#define BM_STATUS_ERROR       0x02
#define BM_STATUS_INTERRUPT   0x04

/*Register definition for sil 680 chipset*/
/*Bus master Status 2 Register*/
typedef union _BM_STATUS2_REGISTER {
  UCHAR   chStatusReg;
  struct {
     UCHAR  BmActive:1;
     UCHAR  DmaError:1;
     UCHAR  InterruptStatus:1;
     UCHAR  BMBufferEmpty:1;
     UCHAR  BMWatchDog:1;
     UCHAR  Drive0DmaCapable:1;
     UCHAR  Drive1DmaCapable:1;
     UCHAR  BMSimplex:1;
  }stat;
} BM_STATUS2_REGISTER, *PBM_STATUS2_REGISTER;

/*Second PCI Bus Master Register for Channel 2*/
typedef struct _BM2_REGISTERS_IDE1 {
  BM_COMMAND_REGISTER  Command;
  UCHAR  reserved1;
  BM_STATUS2_REGISTER  Status2;
  UCHAR  reserved2;
} BM2_REGISTERS_IDE1, *PBM2_REGISTERS_IDE1;

/*Second PCI Bus Master Register for Channel 1*/
typedef struct _BM2_REGISTERS_IDE0 {
  BM_COMMAND_REGISTER  Command;
  UCHAR  SoftwareData;
  BM_STATUS2_REGISTER  Status2;
  BM_STATUS_REGISTER  Status1;
} BM2_REGISTERS_IDE0, *PBM2_REGISTERS_IDE0;

/*Bus master Byte Count*/
typedef union _BM_BYTE_COUNT_REGISTER {
  ULONG  ulByteCount;
  struct {
     USHORT  BtCntLow;
     USHORT  BtCntHigh:15;
     USHORT  EndOfTable:1;
  }btCnt;
}BM_BYTE_COUNT_REGISTER,*PBM_BYTE_COUNT_REGISTER;

/*FIFO Valide Byte Count, Wr request, Rd Request*/
typedef union _FIFO_VALID_BYTE_COUNT {
  ULONG  ulByteCount;
  struct {
     UCHAR   RdReqCtrl:7;
     UCHAR   Reserved1:1;
     UCHAR   WrReqCtrl:7;
     UCHAR   Reserved2:1;
     USHORT  validByteCnt:9;
     USHORT  Reserved3:7;
  }validBtCnt;
}FIFO_VALID_BYTE_COUNT,*PFIFO_VALID_BYTE_COUNT;

/*System Configuration and System Command registers*/
typedef union _SYSTEM_CONFIG_STAT_CMD {
  ULONG  ulconfStatCmd;
  struct {
     UCHAR   PBMModuleRst:1;  /*Byte 0*/
     UCHAR   ARBModuleRst:1;
     UCHAR   Reserved1:2;
     UCHAR   FF1ModuleRst:1;
     UCHAR   FF0ModuleRst:1;
     UCHAR   IDE1ModuleRst:1;
     UCHAR   IDE0ModuleRst:1;
     UCHAR   Reserved2;       /*Byte 1*/
     UCHAR   BA5_Enable:1;    /*Byte 2*/
     UCHAR   Reserved3:3;
     UCHAR   IDEClkSelect:2;
     UCHAR   IDE0IntBlock:1;
     UCHAR   IDE1IntBlock:1;
     UCHAR   Reserved4;       /*Byte 3*/
  }Cmd;
}SYSTEM_CONFIG_STAT_CMD,*PSYSTEM_CONFIG_STAT_CMD;

/*FLASH related register*/
typedef union _FLASH_MEM {
  ULONG  ulFlash[2];
  struct {
     ULONG  MemAddr:19;        /*Byte 0,1,2*/
     ULONG  Reserved1:5;
     ULONG  MemAccessType:1;   /*Byte 3*/
     ULONG  MemAccessStart:1;
     ULONG  Reserved2:1;
     ULONG  MemInitDone:1;
     ULONG  Reserved3:4;
     UCHAR  MemoryData;         /*Byte 0*/
     ULONG  Reserved4:24;       /*Byte 1,2,3*/
  }Reg;
}FLASH_MEM,*PFLASH_MEM;

/*This structure contains the EEPROM related register It contains EEPROM
 *Memory address-Command and Status and memory data registers
 */
typedef union _EEPROM_MEM {
  ULONG  ulEeprom[2];
  struct {
     USHORT  MemAddr:16;
     UCHAR  Reserved1:8;
     UCHAR  MemAccessType:1;
     UCHAR  MemAccessStart:1;
     UCHAR  Reserved2:1;
     UCHAR  MemInitDone:1;
     UCHAR  Memerror:1;
     ULONG  Reserved3:3;
     UCHAR  MemoryData;
     ULONG  Reserved4:24;
  }EepromReg;
}EEPROM_MEM,*PEEPROM_MEM;

/*FIFO related register of a channel*/
typedef struct _FIFO_REG {
  ULONG  Port;
  ULONG  Reserved1;
  UCHAR  Bt0RdPtr;
  UCHAR  Bt0WrPtr;
  UCHAR  Bt1RdPtr;
  UCHAR  Bt1WrPtr;
  UCHAR  Bt2RdPtr;
  UCHAR  Bt2WrPtr;
  UCHAR  Bt3RdPtr;
  UCHAR  Bt3WrPtr;
}FIFO_REG,*PFIFO_REG;

/*Task File Timing registers*/
typedef union _TF_READ_AHEAD_REG{
  ULONG  ulTfReadAhead[8];
  struct {
     UCHAR  Data;
     UCHAR  Features;
     UCHAR  SecCnt;
     UCHAR  StartSecNum;
     UCHAR  CylLow;
     UCHAR  CylHigh;
     UCHAR  DeviceHead;
     UCHAR  CmdStat;
     UCHAR  Reserved1;
     UCHAR  tfDeviceCntrl;
     UCHAR  Reserved2;
     UCHAR  Reserved3;
     ULONG  ReadAheaddata;
     UCHAR  Data2;
     UCHAR  Features2;
     UCHAR  SecCnt2;
     UCHAR  StartSecNum2;
     UCHAR  CylLow2;
     UCHAR  CylHigh2;
     UCHAR  DeviceHead2;
     UCHAR  CmdStat2;
     ULONG  Reserved4;
     ULONG  VirtDMAReadAheadBt;
  }reg;
}TF_READ_AHEAD_REG, *PTF_READ_AHEAD_REG;

/*Task File Timing, Config Status, Command Status registers*/
typedef union _TF_TIMING_STAT{
  ULONG  TfTimeStat;
  struct {
     UCHAR  Cable80:1;
     UCHAR  BufferedCmd:1;
     UCHAR  ChanRst:1;
     UCHAR  ChanTriState:1;
     UCHAR  reserved1:4;
     UCHAR  reserved2:1;
     UCHAR  IORDYMontr:1;
     UCHAR  VirtDmaAddr:1;
     UCHAR  VirtDmaInt:1;
     UCHAR  IntStat:1;
     UCHAR  WatchDogTimeOut:1;
     UCHAR  WatchDogEna:1;
     UCHAR  WatchDogIntEna:1;
     UCHAR  reserved3:1;
     USHORT  RecCnt:6;
     USHORT  ActiveCnt:6;
     USHORT  AddrSetupCnt:4;
  }reg;
}TF_TIMING_STAT, *PTF_TIMING_STAT;

/*Device0 and Device 1 PIO Timing registers*/
typedef union _PIO_TIMING{
  ULONG  PioTiming;
  struct {
    USHORT  Dev0RecCnt:6;
    USHORT  Dev0ActCnt:6;
    USHORT  Dev0AddsSetUpCnt:4;
    USHORT  Dev1RecCnt:6;
    USHORT  Dev1ActCnt:6;
    USHORT  Dev1AddsSetUpCnt:4;
 }reg;
} PIO_TIMING, *PPIO_TIMING;

/*Device0 and Device 1 DMA Timing register*/
typedef union _DMA_TIMING{
  ULONG  DmaTiming;
  struct {
    USHORT  Dev0RecCnt:6;
    USHORT  Dev0ActCnt:6;
    USHORT  Dev0AddsSetUpCnt:4;
    USHORT  Dev1RecCnt:6;
    USHORT  Dev1ActCnt:6;
    USHORT  Dev1AddsSetUpCnt:4;
 }reg;
} DMA_TIMING, *PDMA_TIMING;

/*Device0 and Device 1 UDMA Timing register*/
typedef union _UDMA_TIMING{
  ULONG  UdmaTiming;
  struct {
    USHORT  Dev0CylTimeCnt:6;
    USHORT  reserved1:3;
    USHORT  Dev0HstRobeDelay:3;
    USHORT  Dev0DstRobeDelay:2;
    USHORT  Dev0DataIpDelay:2;
    USHORT  Dev1CylTimeCnt:6;
    USHORT  reserved2:3;
    USHORT  Dev1HstRobeDelay:3;
    USHORT  Dev1DstRobeDelay:2;
    USHORT  Dev1DataIpDelay:2;
 }reg;
} UDMA_TIMING, *PUDMA_TIMING;


/*TF, PIO, DMA and UDMA timing registers Together*/
typedef struct _TIMING_REGS{
 TF_TIMING_STAT tfStat;
 PIO_TIMING  pio;
 DMA_TIMING  dma;
 UDMA_TIMING  udma;
}TIMING_REGS, *PTIMING_REGS;

/*Channel Test Register*/
typedef union _TEST_REG{
    ULONG  ulTestReg;
 struct{
    USHORT  DataField;
    UCHAR  SubModSel;
    UCHAR  NibbleSel:4;
    UCHAR  ModSel:4;
 }reg;
}TEST_REG, *PTEST_REG;

/*Transfer Mode Register*/
typedef union _TRANSFER_MODE_REG{
    ULONG  ulTranMode;
 struct{
    UCHAR  Dev0TranMode:2;
    UCHAR  reserved1:2;
    UCHAR  Dev1TranMode:2;
    UCHAR  reserved2:2;
     ULONG  reserved3:24;
 }reg;
}TRANSFER_MODE_REG, *PTRANSFER_MODE_REG;

/*all channel specific registers together, easy programming*/
typedef struct _CHAN_REGS{
 TF_READ_AHEAD_REG tfRdAhead;
 TIMING_REGS Timings;
 TEST_REG    Test;
 TRANSFER_MODE_REG TransMode;
   ULONG  reserved1;
   ULONG  reserved2;
}CHAN_REGS, *PCHAN_REGS;

/* Sil 680 Internal Register Space of Base Adress 5
 * Defines the addressing of Various Control funtions with in the Sil 680
 */
typedef struct _CONTROL_FUNTION_REG_SIL680 {
  BM_REGISTERS  firstBm0;            /*0x00-0x07*/
  BM_REGISTERS  firstBm1;            /*0x08-0x0F*/
  BM2_REGISTERS_IDE0 secondBm0;      /*0x10-0x13*/
  ULONG  reserved1;                 /*0x14-0x17*/
  BM2_REGISTERS_IDE1 secondBm1;      /*0x18-0x1B*/
  ULONG  reserved2;                 /*0x1C-0x1F*/
  ULONG  prdAdrIde0;                /*0x20-0x23*/
  BM_BYTE_COUNT_REGISTER bmBtCnt0;   /*0x24-0x27*/
  ULONG  prdAdrIde1;                /*0x28-0x2B*/
  BM_BYTE_COUNT_REGISTER bmBtCnt1;   /*0x2C-0x2F*/
  ULONG  reserved3;                 /*0x30-0x33*/
  ULONG  reserved4;                 /*0x34-0x37*/
  ULONG  reserved5;                 /*0x38-0x3B*/
  ULONG  reserved6;                 /*0x3C-0x3F*/
  FIFO_VALID_BYTE_COUNT validBtCnt0; /*0x40-0x43*/
  FIFO_VALID_BYTE_COUNT validBtCnt1; /*0x44-0x47*/
  SYSTEM_CONFIG_STAT_CMD sysConfCmd; /*0x48-0x4B*/
  ULONG  sysSoftwareData;           /*0x4C-0x4F*/
  FLASH_MEM  flash;                  /*0x50-0x57*/
  EEPROM_MEM  eeprom;                /*0x58-0x5F*/
  FIFO_REG  Fifo0;                   /*0x60-0x6F*/
  FIFO_REG  Fifo1;                   /*0x70-0x7F*/
  CHAN_REGS  Chan0;                  /*0x80-0xBF*/
  CHAN_REGS  Chan1;                  /*0xC0-0xFF*/
} CONTROL_FUNTION_REG_SIL680, *PCONTROL_FUNTION_REG_SIL680;

/*IDENTIFY data*/
#pragma pack (1)
typedef struct _IDENTIFY_DATA {
  USHORT  GeneralConfiguration;     /*00 00*/
  USHORT  NumberOfCylinders;        /*02  1*/
  USHORT  Reserved1;                /*04  2*/
  USHORT  NumberOfHeads;            /*06  3*/
  USHORT  UnformattedBytesPerTrack; /*08  4*/
  USHORT  UnformattedBytesPerSector;/*0A  5*/
  USHORT  SectorsPerTrack;          /*0C  6*/
  USHORT  VendorUnique1[3];         /*0E  7-9*/
  UCHAR   SerialNumber[20];         /*14  10-19*/
  USHORT  BufferType;               /*28  20*/
  USHORT  BufferSectorSize;         /*2A  21*/
  USHORT  NumberOfEccBytes;         /*2C  22*/
  UCHAR   FirmwareRevision[8];      /*2E  23-26*/
  UCHAR   ModelNumber[40];          /*36  27-46*/
  UCHAR   MaximumBlockTransfer;     /*5E  47*/
  UCHAR   VendorUnique2;            /*5F*/
  USHORT  DoubleWordIo;             /*60  48*/
  USHORT  Capabilities;             /*62  49*/
  USHORT  Reserved2;                /*64  50*/
  UCHAR   VendorUnique3;            /*66  51*/
  UCHAR   PioCycleTimingMode;       /*67*/
  UCHAR   VendorUnique4;            /*68  52*/
  UCHAR   DmaCycleTimingMode;       /*69*/
 /*first 3 bits will carry Translation Fields Valid*/
  USHORT  TranslationFieldsValid;   /*6A  53*/
  USHORT  NumberOfCurrentCylinders; /*6C  54*/
  USHORT  NumberOfCurrentHeads;     /*6E  55*/
  USHORT  CurrentSectorsPerTrack;   /*70  56*/
  ULONG   CurrentSectorCapacity;    /*72  57-58*/
  USHORT  CurrentMultiSectorSetting;/*    59*/
  ULONG   UserAddressableSectors;   /*    60-61*/
  UCHAR   SingleWordDmaActive;
  UCHAR   SingleWordDmaSupport;     /*    62*/
  UCHAR   MultiWordDmaActive;
  UCHAR   MultiWordDmaSupport;      /*    63*/
  UCHAR   Reserved4;
  UCHAR   AdvancedPioModes;         /*    64*/
  USHORT  MinimumMwXferCycleTime;   /*    65*/
  USHORT  RecommendedMwXferCycleTime;      /*    66*/
  USHORT  MinimumPioCycleTime;      /*    67*/
  USHORT  MinimumPioCycleTimeIordy; /*    68*/
  USHORT  Reserved5[11];            /*    69-79*/
  USHORT  MajorRevision;            /*    80*/
  USHORT  MinorRevision;            /*    81*/
  USHORT  CmdSupported1;            /*    82*/
  USHORT  CmdSupported2;            /*    83*/
  USHORT  FtrSupported;             /*    84*/
  USHORT  CmdEnabled1;              /*    85*/
  USHORT  CmdEnabled2;              /*    86*/
  USHORT  FtrEnabled;               /*    87*/
  UCHAR   UltraDmaSupport;
  UCHAR   UltraDmaActive;           /*    88*/
  USHORT  Reserved7[11];            /*89-100*/
  ULONG   AddressableSectors48bit[2];/*100-104*/
  USHORT  Reserved8[23];            /*104-125*/
  USHORT  LastLun;                  /*    126*/
 /*127 Last bit will represent the StatusNotification...*/
 /*remaining are reserved*/
  UCHAR   MediaStatusNotification;
 /*Last two bits will represent WriteProtect...*/
 /*remaining are reserved*/
  USHORT  DeviceWriteProtect;
  USHORT  Reserved11[128];          /* 128-255*/
} IDENTIFY_DATA, *PIDENTIFY_DATA;

typedef struct _reg{
 UCHAR  type;
 IDE_REGISTERS_1 taskfile;
 IDE_REGISTERS_2 alternate;
 BM_REGISTERS bm0;
 BM_REGISTERS bm1;
} REGS;

#pragma pack ()

#endif /* __MEGAIDE_ATA_H__*/
/*vi set ts=8:*/
