/*megaide_devext.h*/
/*
 * Linux MegaRAID IDEal Software RAID Stack
 * Copyright (c) 2001 LSI Logic Corporation, All Rights Reserved.
 *-----------------------------------------------------------------------------
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *-----------------------------------------------------------------------------
 */

#ifndef __MEGAIDE_DEVEXT_H__
#define __MEGAIDE_DEVEXT_H__

#define      PROCBUFSIZE   4096

#ifdef BP_SUPPORT
/*Initialize Backplane Structure*/
typedef
void
(*BP_EVENT_CALLBACK_ROUTINE)(unsigned long status);

typedef
ULONG
(*BP_ENTRY_POINT)(unsigned int cmd, unsigned long arg);

typedef struct _INIT_BACKPLANE
{
   BP_EVENT_CALLBACK_ROUTINE  callback;
   BP_ENTRY_POINT  bp_control;
   USHORT  MajorVersion;
   USHORT  MinorVersion;
   ULONG  Capability;
}
__attribute__((packed))
INIT_BACKPLANE, *PINIT_BACKPLANE;

typedef struct  _BACKPLANE_DATA{
   UCHAR  bpRegistered;
   UCHAR  NotifyController;
   BP_ENTRY_POINT  pIdeBackplaneControl;
   BP_EVENT_CALLBACK_ROUTINE  pCallBackFunc;
   INIT_BACKPLANE  init_bp;
   BP_STATUS_INFO  bpControlStatus;
   UCHAR  StartEvent_Index;
   UCHAR  EndEvent_Index;
   ADD_EVENT  EventLogBuffer[256];
} BACKPLANE_DATA, *PBACKPLANE_DATA;
#endif

typedef enum _IRCD_TYPE
{
   NO_IRCD=0,
   IRCD_2X = 1,
   IRCD_3X = 2
}IRCD_TYPE;

typedef enum _CONTROLLER_TYPE
{
   CMD_648 = 0,
   CMD_649 = 1,
   CMD_680 = 2,
   RCC_SERVER_WORKS_CSB5 = 3,
   SIL_3112 = 4,
   RCC_SERVER_WORKS_CSB6 = 5,
   INTEL_ICH5R = 6
}CONTROLLER_TYPE;

/*Note... we have to change this for each controller added*/
#define NUM_SUPPORTED_CONTROLLERS         7

/*SGL definition for DMA operations*/
typedef struct _PHYSICAL_SGL_ENTRY {
   PVOID  Address;
   ULONG  Length:31;
   ULONG  EndOfListFlag:1;
} PHYSICAL_SGL_ENTRY, *PPHYSICAL_SGL_ENTRY;

/*SGL definition for PIO operations.*/
typedef struct _LOGICAL_SGL_ENTRY {
   PVOID  Address;
   ULONG  Length:31;
   ULONG  EndOfListFlag:1;
} LOGICAL_SGL_ENTRY, *PLOGICAL_SGL_ENTRY;

/*SGL definition for PIO operations.*/
typedef struct _PRINTABLE_SGL_ENTRY {
   PVOID  Address;
   ULONG  Length;
} PRINTABLE_SGL_ENTRY, *PPRINTABLE_SGL_ENTRY;

/*Scatter/Gather list definition.*/
typedef union _SGL_ENTRY {
   PHYSICAL_SGL_ENTRY   Physical;
   LOGICAL_SGL_ENTRY   Logical;
   PRINTABLE_SGL_ENTRY   Printable;
} SGL_ENTRY, *PSGL_ENTRY;

typedef struct _COMMAND_SPLIT_STATUS
{
   union
   {
      struct
      {
         UCHAR  Drive0:1;
         UCHAR  Drive1:1;
         UCHAR  Drive2:1;
         UCHAR  Drive3:1;
         UCHAR  Reserved:4;
      }Bits;
      UCHAR Drives;
   }u;
}COMMAND_SPLIT_STATUS, *PCOMMAND_SPLIT_STATUS;

/* Not using any kernal defines in closed source.. so
 * creating a structure as in kernal and using it
 */
struct list_head_local {
   struct list_head_local *next, *prev;
};

typedef struct _FAST_IDE_SCB_EXTENSION
{
   COMMAND_SPLIT_STATUS   CmdBuildStatus;
   ULONG  NumberOfPendingCmd;
   UCHAR  Reserved[2];
   UCHAR  CmdStatus;
   /* scsi cmd stuff copied here*/
   unsigned int  target;
   unsigned int  lun;
   unsigned int  channel;
   unsigned char  scsi_cdb[16];
   unsigned char  cdb_len;
   void  *request_buffer;
   unsigned short  sglem;
   void  *pstScsiCmd;
   struct list_head_local  list;
#ifdef PERFORMANCE_NUMBER
   unsigned int time;
#endif
   UCHAR  PhysicalDrvCmd[1];
}
__attribute__ ((packed))
FAST_IDE_SCB_EXTENSION, *PFAST_IDE_SCB_EXTENSION;


/*Logical drive descriptor*/
typedef struct _LOGICAL_DRIVE_INFO{
   MP_RAID_LEVEL   RaidLevel;
   UCHAR  LogicalDriveNumber;
   MP_LOGICAL_DRIVE_STATUS   Status;
   ULONG  Sectors;      /* Total number of available sectors.*/
   ULONG  StripeSize;      /* In 512-byte units.*/
   UCHAR  StripesPerRow;
   UCHAR  PhysicalDriveCount;
   UCHAR  PhysicalDriveTid[MAX_DRIVES_PER_CONTROLLER];
   UCHAR  SlotPhysicalDriveTid[MAX_DRIVES_PER_CONTROLLER];
   ULONG  StartBlock;   /*start block on phy drive below*/
   BOOLEAN  IsBkOperationStarted;
   BOOLEAN IsAbortReceived;
   ULONG  EachArmLength;
   ULONG  RebuildStartBlock;
   ULONG  CheckConStartBlock;
   ULONG  CompletedBlocks;/*number of completed blocks within the log drive*/
   ULONG  ulInconsistencyCount;
   ULONG  ulErrorCountDuringBackgroundTask;
   UCHAR  ucLastBackgroundTaskOperation;
   MP_ACTIVITY_STATUS  ucLastBackgroundTaskStatus;
   void  *GateKeeper;
   FAST_IDE_SCB_EXTENSION  *InternalScbExtension;
   UCHAR  ucControllerId;
   struct _ARRAY_DRIVE_INFO   *pArrayInfo;
   struct _LOGICAL_DRIVE_INFO   *NextLogDrive;
   struct _LOGICAL_DRIVE_INFO   *PrevLogDrive;
}
__attribute__ ((packed))
LOGICAL_DRIVE_INFO, *PLOGICAL_DRIVE_INFO;


/*Array drive descriptor*/
typedef struct _ARRAY_DRIVE_INFO{
   UCHAR  ucArrayDriveNumber;
   UCHAR  ucLogicalDriveCount;
   LOGICAL_DRIVE_INFO  *HeadLogDrive;
   LOGICAL_DRIVE_INFO  *TailLogDrive;
}
__attribute__ ((packed))
ARRAY_DRIVE_INFO, *PARRAY_DRIVE_INFO;

/* Physical drive descriptor.*/
typedef struct _PHYSICAL_DRIVE_INFO {
   /*Fixed Physical Drive Info*/
   UCHAR  PhysicalDriveId;/*0,1,2,3 for four physcial Drive*/
   UCHAR  LogicalDriveId;   /* these are needed for old IRCD*/
   UCHAR  LogicalIndex;      /* these are needed for old IRCD*/
   ULONG  BiosExposedSectors;
   UCHAR  ucMirrorDriveId;
   ARRAY_DRIVE_INFO  *pArrayInfo;       /* points to the logical data area of this drive*/
   UCHAR  ucConfigurationWasSwitched; /* we switched the Config in when reading IRCD*/
   UCHAR  error_log_needs_reading;
   USHORT  ErrorFound; /* count of total errors*/
/* Dell is wanting us to program down the drives if we get
 * 3 sequentail errors on a physical drive
 * this is the counter for this*/
   USHORT  SequentialErrorCount;
   void  *ErrorLogSector;
   SGL_ENTRY  DiskErrorLogSgl;
   ULONG  ErrorLogSectorIndex;
   USHORT  ErrorReported;
   ULONG  IrcdSectorIndex;
   ULONG  EnquirySectorIndex;
   ULONG  DeviceFlags;
   BOOLEAN  bEnableRwCache;
   BOOLEAN  bEnableReadAheadCache;
   BOOLEAN  ucIsPowerConnected;
   MP_TRANSFER_MODE TransferMode;
   ULONG  StartBlock;
   ULONG  NumberOfBlocks;
   ULONG  NumberOfBlocks48bit;
   MP_PHYSICAL_DRIVE_STATUS Status;
   BOOLEAN  bIsSpareDrive;
   IDENTIFY_DATA IdentifyData;
   ULONG  MaximumBlockXfer;
   UCHAR  RetryCount;
   ULONG  LastBlockRead;
   ULONG  BadErrorSector;   /* location of last error found on drive*/
   UCHAR  errorByte; /*type of error found on drive.*/
   BOOLEAN  IsFrozenQueue;
   ULONG  CurrentQueueLength;
   void  *HoldQueue; /*Head Pointer*/
   void  *TailQueue; /*Tail pointer*/
   UCHAR  ucControllerId;
   BOOLEAN  bErrorLogUpdatePending;
   ULONG last_check_time;
}
__attribute__ ((packed))
PHYSICAL_DRIVE_INFO, *PPHYSICAL_DRIVE_INFO;

typedef struct _DMA_INFO {
   u_long   physical_address;
   u_long   physical_address_len;
}
__attribute__ ((packed))
DMA_INFO, *PDMA_INFO;

/*Channel descriptor*/
typedef struct _CHANNEL{
   UCHAR  LastDriveFed;
   UCHAR  SwitchDrive;
   SGL_ENTRY  *pSglEntry;
   SGL_ENTRY  *SglBaseVirtualAddress;
   ULONG  SglBasePhysicalAddress;
   ULONG  MaxSglEntries;
   IDE_REGISTERS_1  *BaseIoAddress1;
   IDE_REGISTERS_2  *BaseIoAddress2;
   BM_REGISTERS *BaseBmAddress;
   void  *ActiveQueue;
   UCHAR  ExpectingInterrupt;
   UCHAR  ucActiveCmd;
   UCHAR  ulActiveTargetId;
   UCHAR  SglElemCnt;
   u_long   dma_sgl_count;
   DMA_INFO  dma_info[MAX_SGL_ENTRIES_PER_PHYSICAL_DRIVE];
   SGL_ENTRY  DiskConfigurationSgl;
   UCHAR  *DiskConfiguration;
   SGL_ENTRY  DiskEnquirySgl;
   UCHAR  *DiskEnquiry;
   PASS_THRU_DATA  passthru_data;
   UCHAR  ChannelBusy;
   UCHAR  ucCurrentPMMode;/*Serial ATA power Management*/
   unsigned long time_jiffies;
}
__attribute__ ((packed))
CHANNEL, *PCHANNEL;

#ifdef DEBUG_IO_ACCESS
#define IO_ACCESS_READ_PCI		1
#define IO_ACCESS_WRITE_PCI		2
#define IO_ACCESS_READ_IO		3
#define IO_ACCESS_WRITE_IO		4
#define MAX_IO_ACCESS_LOG_ENTRIES	512
typedef struct{
    UCHAR	io_access;
    ULONG	io_register;
    ULONG	io_value;
}IO_ACCESS_LOG;
#endif

/*Device extension*/
typedef struct _HW_DEVICE_EXTENSION
{
   UCHAR  ucControllerId;
   ULONG  ulVendorId; /*PCI PNP IDs.*/
   ULONG  ulDeviceId;
   USHORT  usSubsysVendorId;
   USHORT  usSubsysDeviceId;
   ULONG  ulUniqueId;
   ULONG  ulProdId;
   USHORT  usSlot;
   UCHAR  ucPCIBus;
   UCHAR  ucPCIDev;
   UCHAR  ucPCIFun;
   ULONG  ulIntLine; /*Base register locations.*/
   CONTROLLER_SPEED ControllerSpeed; /*Controller speed (UDMA/33, UDMA66)*/
   ULONG  MaximumTransferLength;
   ULONG  ulLogicalDriveCount;
   ULONG  ulLastChannelFed;
   ULONG  ulArrayDriveCount;
   ARRAY_DRIVE_INFO ArrayDrive[MAX_DRIVES_PER_CONTROLLER];
   LOGICAL_DRIVE_INFO LogicalDrive[MAX_LOGICAL_DRIVES];
   UCHAR  BootLogDrvID;
   UCHAR  BootLogDrvMapping[MAX_LOGICAL_DRIVES];
   PHYSICAL_DRIVE_INFO PhysicalDrive[MAX_DRIVES_PER_CONTROLLER];
   CHANNEL   Channel[MAX_CHANNELS_PER_CONTROLLER];
   ULONG  baseAddress5;
   UCHAR  *virtBaseAddress5;
   void  *ActiveGateKeeper;
   BOOLEAN  IsUsed;
   UCHAR  *GateKeeperBuffer;
   ULONG  GateKeeperPhysicalBuffer;
   ULONG  GateKeeperBufferLength;
   UCHAR  ConfigurationVirtualBuffer[IDE_SECTOR_SIZE];
   UCHAR  EnquiryVirtualBuffer[ IDE_SECTOR_SIZE ];
   ULONG  NumberOfPendingCmd;
   ULONG  NumberOfPendingScb;
   UCHAR  BusyTarget;
#ifdef   BP_SUPPORT
   BP_BEEP   bBuzzerEnable;
   BACKPLANE_DATA BackPlane;
#endif
   COMMAND_SPLIT_STATUS drive_inserted;
   COMMAND_SPLIT_STATUS drive_removed;
   UCHAR  detect_try_count;
   void  *GateKeeper;
   FAST_IDE_SCB_EXTENSION *GateScbExtension;
   MP_IDE_VERSION stFirmwareVersion;
   MP_IDE_VERSION stIRCDVersion;
   IRCD_TYPE ucIRCDType;
   UCHAR   ucIRCDMajVer;
   UCHAR   ucIRCDMinVer;
   USHORT  uwCodeSize;
   CONTROLLER_TYPE ucCardType;
   ULONG  uiFlag;
   UCHAR  ucErrorRecovery;
   UCHAR  aucChannelList[MAX_CHANNELS_PER_CONTROLLER];
   char  achProcBuf[PROCBUFSIZE];
   int   iProcIdx;
   void *adapter;
   MP_RAID_OPTIONS_EX Options;
   BP_OPTIONS bpOptions;
   ULONG rebuild_rate_temp;
#ifdef DEBUG_IO_ACCESS
   UCHAR debug_io_head;
   IO_ACCESS_LOG debug_io[MAX_IO_ACCESS_LOG_ENTRIES];
#endif

#ifdef PERFORMANCE_NUMBER
   unsigned int pcmd_2_isr[2][30];
   unsigned int io_2_pcmd[2][30];
   unsigned int scbduration[30];
   unsigned int int_count[2];
   unsigned int io_count;
#endif

   int  ICH5R_thread_pid;
   int  ICH5R_thread_flag;

#ifdef  ENABLE_ERROR_INJECTOR
   ULONG  ErrorAfterCounter;
   PDErrorInjectorInfo  pdErrorInjectorInfo[MAX_DRIVES_PER_CONTROLLER];
   ULONG  ErrorCounter[10];
   ErrorPatternData ErrorPattern[MAX_PATTERN];
#endif
}
__attribute__ ((packed))
HW_DEVICE_EXTENSION, *PHW_DEVICE_EXTENSION;
#endif /*__MEGAIDE_DEVEXT_H__ */
/*vi set ts=8:*/


