/*megaide_macros.h*/
/*
 * Linux MegaRAID IDEal Software RAID Stack
 * Copyright (c) 2001 LSI Logic Corporation, All Rights Reserved.
 *-----------------------------------------------------------------------------
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *-----------------------------------------------------------------------------
 */

#ifndef __MACROS_H__
#define __MACROS_H__

/*Macros for port communication This is a Linux dependent code*/
#define STM_PIO(mode)            ((UCHAR) ((1 << 3) | (mode)))
#define STM_MULTIWORD_DMA(mode)  ((UCHAR) ((1 << 5) | (mode)))
#define STM_UDMA(mode)           ((UCHAR) ((1 << 6) | (mode)))

#define ABS_DIFF(pos1, pos2) \
          (ULONG)((pos1 > pos2) ? (pos1-pos2) : (pos2 - pos1))

/*Beautification macros*/

/*Get minimum value of two*/
#define MEGA_FIND_MIN(a,b) ((a) <= (b) ? (a) : (b))

#define BIG_ENDIAN_ULONG(Ulong) \
         (((UCHAR *)&Ulong)[0] << 24) | \
         (((UCHAR *)&Ulong)[1] << 16) | \
         (((UCHAR *)&Ulong)[2] << 8) | \
         ((UCHAR *)&Ulong)[3]

#define BIG_ENDIAN_USHORT(Ushort) \
         (((UCHAR *)&Ushort)[0] << 8) | \
         (((UCHAR *)&Ushort)[1] >> 8)

#define GET_DEVICE_ID(pdev) \
         pdev->ulDeviceId

#define IS_48_BIT_ENABLED(pdev, tid) \
         ((pdev->PhysicalDrive[tid].IdentifyData.CmdSupported2 & \
           (1<<10))!=0)

#define IS_48_BIT_ADDESSABLE(pdev, tid) \
         (IS_48_BIT_ENABLED(pdev,tid) && \
          (pdev->PhysicalDrive[tid].NumberOfBlocks > 0xFFFFFFF))

#define USES_DMA(tid) \
         ((pdev->PhysicalDrive[tid].DeviceFlags & \
           DFLAGS_USES_EITHER_DMA) != 0)

#define SELECT_DEVICE(BaseIoAddress, tid) \
         ScsiPortWritePortUchar((unsigned)&BaseIoAddress->DriveSelect, \
                                (UCHAR)((((tid) & 1) << 4) | IDE_CHS_MODE)); \
         ScsiPortStallExecution(60)

#define SELECT_CHS_DEVICE(BaseIoAddress, tid, HeadNumber) \
         ScsiPortWritePortUchar((unsigned)&BaseIoAddress->DriveSelect, \
                                (UCHAR)((((tid) & 1) << 4) | IDE_CHS_MODE | \
                                       (HeadNumber & 15))); \
         ScsiPortStallExecution(60)

#define SELECT_LBA_DEVICE(BaseIoAddress, tid, Lba) \
         ScsiPortWritePortUchar((unsigned)&BaseIoAddress->DriveSelect, \
                                (UCHAR)((((tid) & 1) << 4) | IDE_LBA_MODE | \
                                (Lba & 0x0f000000) >> 24)); \
         ScsiPortStallExecution(60)

#define SELECT_LBA_DEVICE_48BIT(BaseIoAddress, tid) \
         ScsiPortWritePortUchar((unsigned)&BaseIoAddress->DriveSelect, \
                                (UCHAR)((((tid) & 1) << 4) | IDE_LBA_MODE)); \
         ScsiPortStallExecution(60)

#define GET_STATUS(BaseIoAddress1, Status) \
         Status = ScsiPortReadPortUchar((unsigned)&BaseIoAddress1->Command);

#define GET_BASE_STATUS(BaseIoAddress1, Status) \
         ScsiPortStallExecution(1); \
         Status = ScsiPortReadPortUchar((unsigned)&BaseIoAddress1->Command);

#define WRITE_COMMAND(BaseIoAddress1, Command) \
         ScsiPortWritePortUchar((unsigned)&BaseIoAddress1->Command, Command);

#define AR_READ_BUFFER(BaseIoAddress, Buffer, Count) \
         ScsiPortReadPortBufferUshort((unsigned)(&BaseIoAddress->Data), \
                                      Buffer, Count);

#define AR_WRITE_BUFFER(BaseIoAddress, Buffer, Count) \
         ScsiPortWritePortBufferUshort((unsigned)(&BaseIoAddress->Data), \
                                       Buffer, Count);

#define AR_READ_BUFFER2(BaseIoAddress, Buffer, Count) \
         ScsiPortReadPortBufferUlong((unsigned)(&BaseIoAddress->Data), \
                                     Buffer, Count);

#define AR_WRITE_BUFFER2(BaseIoAddress, Buffer, Count) \
         ScsiPortWritePortBufferUlong((unsigned)(&BaseIoAddress->Data), \
                                      Buffer, Count);

/*WAIT_IN_BUSY waits for up to 1 s.*/
#define WAIT_ON_BUSY(BaseIoAddress, Status) \
{ \
    ULONG i; \
    for (i = 0; i < 200000; i++) { \
        GET_STATUS(BaseIoAddress, Status); \
        if (Status & (IDE_STATUS_BUSY/*|IDE_STATUS_DRQ*/)) { \
            ScsiPortStallExecution(5); \
            continue; \
        } else { \
            break; \
        } \
    } \
}

#define WAIT_ON_ALTERNATE_STATUS_BUSY(base2, Status) \
{ \
    ULONG i; \
    for (i = 0; i < 200000; i++) { \
      Status = ScsiPortReadPortUchar((unsigned)&(base2->AlternateStatus)); \
      if (Status & (IDE_STATUS_BUSY/*|IDE_STATUS_DRQ*/)) { \
        ScsiPortStallExecution(5); \
        continue; \
      } else { \
        break; \
      } \
    } \
}

#define SET_ENQUIRY_FLAG(pdev) \
{ \
    ULONG i; \
    UCHAR Ch1=0, Ch2=0; \
    if((pdev->PhysicalDrive[0].Status == PDS_Online) || \
       (pdev->PhysicalDrive[1].Status == PDS_Online)) { \
        Ch1=1; \
    } \
    if((pdev->PhysicalDrive[0].Status == PDS_Online) || \
       (pdev->PhysicalDrive[1].Status == PDS_Online)) { \
        Ch2=1; \
    } \
    if(Ch1 && !Ch2) { \
        pdev->bChangedEnqValid = FIRST_CHANNEL; \
    } else if(!Ch1 && Ch2) { \
        pdev->bChangedEnqValid = SECOND_CHANNEL; \
    } else if(Ch1 && Ch2) { \
        pdev->bChangedEnqValid = BOTH_CHANNELS; \
    } \
}

#define WAIT_ON_ALTERNATE_DRIVE_READY(base2, Status) \
{ \
    ULONG i; \
    for (i = 0; i < 20000; i++) { \
      Status = ScsiPortReadPortUchar((unsigned)&(base2->AlternateStatus)); \
      if (Status != 0x50) { \
        ScsiPortStallExecution(50); \
        continue; \
      } else { \
        break; \
      } \
    } \
}

#define WAIT_ON_BASE_BUSY(BaseIoAddress, Status) \
{ \
    ULONG i; \
    for (i = 0; i < 200000; i++) { \
        GET_STATUS(BaseIoAddress, Status); \
        if (Status & IDE_STATUS_BUSY) { \
            ScsiPortStallExecution(5); \
            continue; \
        } else { \
            break; \
        } \
    } \
}

#define WAIT_FOR_DRQ(BaseIoAddress, Status) \
{ \
    ULONG i; \
    for (i = 0; i < 200000; i++) { \
        GET_STATUS(BaseIoAddress, Status); \
        if (Status & IDE_STATUS_BUSY) { \
            ScsiPortStallExecution(100); \
        } else if (Status & IDE_STATUS_DRQ) { \
            break; \
        } else { \
            ScsiPortStallExecution(5); \
        } \
    } \
}

#define WAIT_FOR_ALTERNATE_DRQ(base2, Status) \
{ \
    ULONG i; \
    for (i = 0; i < 200000; i++) { \
      Status = ScsiPortReadPortUchar((unsigned)&(base2->AlternateStatus)); \
      if (Status & IDE_STATUS_BUSY) { \
        ScsiPortStallExecution(100); \
      } else if (Status & IDE_STATUS_DRQ) { \
        break; \
      } else { \
        ScsiPortStallExecution(5); \
      } \
    } \
}


#define WAIT_SHORT_FOR_DRQ(BaseIoAddress, Status) \
{ \
    ULONG i; \
    for (i = 0; i < 40; i++) { \
        GET_STATUS(BaseIoAddress, Status); \
        if (Status & IDE_STATUS_BUSY) { \
            ScsiPortStallExecution(100); \
        } else if (Status & IDE_STATUS_DRQ) { \
            break; \
        } else { \
            ScsiPortStallExecution(5); \
        } \
    } \
}

#define ATAPI_SOFT_RESET(BaseIoAddress, DeviceNumber) \
{ \
    UCHAR status; \
    ScsiPortWritePortUchar((unsigned)&BaseIoAddress->DriveSelect, \
                           (UCHAR)(((DeviceNumber & 0x1) << 4) | 0xA0)); \
    ScsiPortStallExecution(500); \
    ScsiPortWritePortUchar((unsigned)&BaseIoAddress->Command, \
                           IDE_COMMAND_ATAPI_RESET); \
    ScsiPortStallExecution(50000); \
    ScsiPortWritePortUchar((unsigned)&BaseIoAddress->DriveSelect, \
                           (UCHAR)((DeviceNumber << 4) | 0xA0)); \
    WAIT_ON_BUSY(BaseIoAddress, status); \
    ScsiPortStallExecution(500); \
}

#define IDE_DC_RESET_CONTROLLER 0x04
#define IDE_DC_REENABLE_CONTROLLER 0x00
#define IDE_HARD_RESET(base1, base2, tid, result) \
{ \
    UCHAR status; \
    ULONG i; \
    unsigned)&base2->AlternateStatus,IDE_DC_RESET_CONTROLLER); \
    ScsiPortStallExecution(1000); \
    ScsiPortStallExecution(1000); \
    ScsiPortStallExecution(1000); \
    ScsiPortWritePortUchar((unsigned)&base2->AlternateStatus, \
                           IDE_DC_REENABLE_CONTROLLER); \
    ScsiPortStallExecution(1000); \
    ScsiPortStallExecution(1000); \
    ScsiPortStallExecution(1000); \
    ScsiPortStallExecution(1000); \
    ScsiPortStallExecution(1000); \
    SELECT_DEVICE(base1, tid); \
    for (i = 0; i < 1000; i++) { \
        status = ScsiPortReadPortUchar((unsigned)&base1->Command); \
        if (status != IDE_STATUS_IDLE && status != 0x0) { \
            ScsiPortStallExecution(1000); \
        } else { \
            break; \
        } \
    } \
    if (i == 1000) { \
        result = FALSE; \
    } \
    result = TRUE; \
}

#define IS_RDP(op_code) \
         ((op_code == SCSIOP_ERASE)|| \
          (op_code == SCSIOP_LOAD_UNLOAD)|| \
          (op_code == SCSIOP_LOCATE)|| \
          (op_code == SCSIOP_REWIND) || \
          (op_code == SCSIOP_SPACE)|| \
          (op_code == SCSIOP_SEEK)|| \
          (op_code == SCSIOP_WRITE_FILEMARKS))

#define SET_BM_COMMAND_REGISTER(Base, Field, Value) \
{ \
    BM_COMMAND_REGISTER buffer; \
    buffer.AsUchar = \
        ScsiPortReadPortUchar((unsigned)&(Base->Command.AsUchar)); \
    buffer.Field = Value; \
    ScsiPortWritePortUchar((unsigned)&(Base->Command.AsUchar), \
                           buffer.AsUchar); \
}

#define SET_BM_STATUS_REGISTER(Base, Field, Value) \
{ \
    BM_STATUS_REGISTER buffer; \
    buffer.AsUchar = \
        ScsiPortReadPortUchar((unsigned)&(Base->Status.AsUchar)); \
    buffer.Field = Value; \
    ScsiPortWritePortUchar((unsigned)&(Base->Status.AsUchar), \
                           buffer.AsUchar); \
}

#define CLEAR_BM_INT(Base, status) \
         status = ScsiPortReadPortUchar((unsigned)&(Base->Status.AsUchar)); \
         status |= 0x06; \
         ScsiPortWritePortUchar((unsigned)&(Base->Status.AsUchar), status)

#define DRIVE_PRESENT(pdev,tid) \
         ((tid < MAX_DRIVES_PER_CONTROLLER) && \
          (pdev->PhysicalDrive[tid].DeviceFlags & DFLAGS_DEVICE_PRESENT))

#define ENABLE_IRCD_UPDATE_ON_DRIVE(pdev, tid) \
         (pdev->PhysicalDrive[tid].DeviceFlags |= DFLAGS_UPDATE_IRCD)

#define DISABLE_IRCD_UPDATE_ON_DRIVE(pdev, tid) \
         (pdev->PhysicalDrive[tid].DeviceFlags &= ~ DFLAGS_UPDATE_IRCD)

#define IRCD_UPDATE_ON_DRIVE(pdev, tid) \
         (pdev->PhysicalDrive[tid].DeviceFlags & DFLAGS_UPDATE_IRCD)

#define IS_IDE_DRIVE(pdev, tid) \
         (pdev->PhysicalDrive[tid].DeviceFlags & DFLAGS_DEVICE_PRESENT)

#define IS_ATAPI_DRIVE(pdev, tid) \
         ((pdev->PhysicalDrive[tid].DeviceFlags & DFLAGS_DEVICE_PRESENT) && \
          (pdev->PhysicalDrive[tid].DeviceFlags & DFLAGS_ATAPI_DEVICE))

#define FEED_ALL_CHANNELS(pdev) \
         { \
           StartChannelIo(pdev, 0); \
           StartChannelIo(pdev, 1); \
         }

#define CLEAR_CHANNEL(pdev, chan) \
         pdev->Channel[chan].ActiveQueue = NULL; \
         pdev->Channel[chan].ExpectingInterrupt = 0;

#define CLEAR_AND_POST_COMMAND(pdev, chan) \
         MarkChannelFree(pdev, chan);

#define ACTIVE_COMMAND_PRESENT(pdev, chan) \
         (pdev->Channel[chan].ActiveQueue)

#define GET_ALTERNATE_STATUS_REGISTER_VALUE(base2) \
         ScsiPortReadPortUchar((unsigned)&(base2->AlternateStatus))

/*Read the Drive Head Register*/
#define GET_DRIVE_HEAD_REGISTER_VALUE(base1) \
         ScsiPortReadPortUchar((unsigned)&(base1->DriveSelect))
/*Read the error Register*/
#define GET_ERROR_REGISTER_VALUE(base1) \
         ScsiPortReadPortUchar((unsigned)&(base1->SectorCount))

/*IDE Error Sense Macro*/
#define  SET_IDE_ERROR_SENSE(S,D,E) \
          ((0x00000000) | ((S) << 16) | ((D) << 8) | (E))

#define IS_CHANNEL_BUSY(pdev, chan) \
         (pdev->Channel[chan].pstActiveCommand)

#define DRIVE_HAS_COMMANDS(pdev, tid) \
         pdev->PhysicalDrive[tid].stScsiCmdQueue.pHead

#define  DEBUG_INQUIRY_LEVEL    (0x1 << 31)

#define CHANNEL_IS_BUSY(pdev, chan) \
         (pdev->Channel[chan].ActiveQueue)

#define IS_DRIVE_READY(pdev, tid) \
         ((tid < MAX_DRIVES_PER_CONTROLLER) && \
          (pdev->PhysicalDrive[tid].Status < PDS_Failed))

#define IS_DRIVE_ONLINE(pdev, tid) \
         ((tid < MAX_DRIVES_PER_CONTROLLER) && \
          (pdev->PhysicalDrive[tid].Status == PDS_Online))

#define IS_DRIVE_REBUILDING(pdev, tid) \
         ((tid < MAX_DRIVES_PER_CONTROLLER) && \
          (pdev->PhysicalDrive[tid].Status == PDS_Rebuilding))

#define IS_DRIVE_FAILED(pdev, tid) \
         ((tid == INVALID_DRIVE_ID) || \
          (pdev->PhysicalDrive[tid].Status == PDS_Failed))


/*Absolute sure that there is a logical drive present for this id*/
#define GET_LD_INFO(pdev,ld) \
         (((ULONG)ld < pdev->ulLogicalDriveCount) ? \
          &pdev->LogicalDrive[pdev->BootLogDrvMapping[ld]] : NULL)

#define SET_IDENTIFY_STAMP(Ld, PdIndex) \
         ((((USHORT)Ld<<8) & 0xff00)|(PdIndex & 0xff))

#define UPDATE_IDENTIFY_STAMP(Buffer, IdentityStamp) \
         Buffer[0x1f0] = (UCHAR)(IdentityStamp & 0xff); \
         Buffer[0x1f1] = (UCHAR)(IdentityStamp >> 8)


#define IS_OTHER_ERROR(errVal) \
         ((!(errVal & (ICRC_ERR|ECC_ERR))) && (errVal & ABRT_ERR))

/*Dell is wanting us to program down the drives if we get
 * 3 sequentail errors on a physical drive
 *here is the macros for handling this
 */
#define CLEAR_SEQ_ERROR_FLAG(pdev, tid) \
         pdev->PhysicalDrive[tid].SequentialErrorCount = 0;

#define INC_SEQ_ERROR_FLAG(pdev, tid) \
         pdev->PhysicalDrive[tid].SequentialErrorCount ++; \
         if (pdev->PhysicalDrive[tid].SequentialErrorCount >= 3) { \
           SetTransferMode (pdev, tid, \
                            (pdev->PhysicalDrive[tid].TransferMode - 1)) ; \
           CLEAR_SEQ_ERROR_FLAG(pdev, tid); \
         }

#define COPY_BACK(pdev, offset, iCount) \
  if (offset > pdev->iProcIdx) { \
    *eof = TRUE; \
        pdev->iProcIdx = 0; \
        pdev->achProcBuf[0] = 0; \
        return 0; } \
 if ((iCount + offset) > pdev->iProcIdx) { \
      iCount = pdev->iProcIdx - offset; \
      *eof = TRUE; } \
      memcpy(page, &pdev->achProcBuf[offset], iCount); \
      pdev->iProcIdx = 0; \
      pdev->achProcBuf[0] = 0;

#define  POLL_FOR_INTERRUPT(pdev, intVal) \
{ \
    ULONG i; \
    ScsiPortStallExecution(1000); \
    for (i = 0; i < 5000; i++) /* for a whole second delay*/ \
    { \
      ucStatus = ScsiPortReadPortUchar( \
      (unsigned)&(pdev->Channel[0].BaseBmAddress->Reserved)); \
      if (ucStatus & intVal) { \
        break; \
      } else { \
        ScsiPortStallExecution(1000); \
        ScsiPortStallExecution(1000); \
        continue; \
      } \
    } \
}

#define  POLL_FOR_DMA_INTERRUPT(baseBm) \
{ \
    ULONG i; \
    ScsiPortStallExecution(1000); \
    for (i = 0; i < 5000; i++) /* for a whole second delay*/ \
    { \
      ucStatus = ScsiPortReadPortUchar((unsigned)&(baseBm->Status.AsUchar)); \
      if (((BM_STATUS_REGISTER) ucStatus) .f.InterruptStatus) { \
        /*WAIT_ON_ALTERNATE_STATUS_BUSY(BaseIoAddress2, ucStatus); */ \
        break; \
      } else { \
        ScsiPortStallExecution(1000); \
        ScsiPortStallExecution(1000); \
        continue; \
      } \
    } \
}

#define  POLL_FOR_INTERRUPT_68X(pdev, tid) \
{ \
    ULONG i; \
    ScsiPortStallExecution(1000); \
    for (i = 0; i < 5000; i++) /* for a whole second delay*/ \
    { \
        if (!(tid >> 1)) /*secondary*/ { \
            pci_read_config_byte_Rlsi(pdev, (IDE0_TFT_CONF_STAT_REG680 + 1),\
                                      &ucStatus); \
            if (ucStatus & PRIMARY_CHAN_INTERRUPT_680) { \
                break; \
            } else { \
                ScsiPortStallExecution(1000); \
                ScsiPortStallExecution(1000); \
                continue; \
            } \
        } else/*Primary channel*/ { \
            pci_read_config_byte_Rlsi(pdev, (IDE1_TFT_CONF_STAT_REG680 + 1), \
                                      &ucStatus); \
            if (ucStatus & SECONDARY_CHAN_INTERRUPT_680) { \
                break; \
            } else { \
                ScsiPortStallExecution(1000); \
                ScsiPortStallExecution(1000); \
                continue; \
            } \
        } \
    } \
}

#define  POLL_FOR_INTERRUPT_CSB5(pdev, tid) \
{ \
    ULONG i; \
    ScsiPortStallExecution(1000); \
    for (i = 0; i < 5000; i++) /* for a whole second delay*/ \
    { \
        if (!(tid >> 1)) /*secondary*/ { \
            ucStatus = ScsiPortReadPortUchar((unsigned)(baseBm+0x0A)); \
            if (ucStatus & CHANNEL_INTERRUPT_SERVERWORKS) { \
                break; \
            } else { \
                ScsiPortStallExecution(1000); \
                ScsiPortStallExecution(1000); \
                continue; \
            } \
        } else/*Primary channel*/ { \
            ucStatus = ScsiPortReadPortUchar((unsigned)(baseBm+ 0x02)); \
            if (ucStatus & CHANNEL_INTERRUPT_SERVERWORKS) { \
                break; \
            } else { \
                ScsiPortStallExecution(1000); \
                ScsiPortStallExecution(1000); \
                continue; \
            } \
        } \
    } \
}

#endif /*__MACROS_H__*/
/*vi set ts=8:*/

