/*megaide_ata.c*/
/*
 * Linux MegaRAID IDEal Software RAID Stack
 * Copyright (c) 2001 LSI Logic Corporation, All Rights Reserved.
 *-----------------------------------------------------------------------------
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *-----------------------------------------------------------------------------
 */


void
ClearPCIInterupt(HW_DEVICE_EXTENSION *pdev, UCHAR ucChannel)
{
	UCHAR ucInterruptValue = 0;
	if ((GET_DEVICE_ID(pdev) == PCI_DEVICE_ID_CMD_648) ||
		(GET_DEVICE_ID(pdev) == PCI_DEVICE_ID_CMD_649))
	{
		ucInterruptValue = (ucChannel ? CHAN2_INTERRUPT:CHAN1_INTERRUPT) | 0x01;
		ScsiPortWritePortUchar((unsigned)&(pdev->Channel[0].BaseBmAddress->Reserved), ucInterruptValue);
	}
}

BOOLEAN
FoundInterrupt(HW_DEVICE_EXTENSION *pdev, UCHAR *pChan)
{
	BOOLEAN retVal;
	UCHAR card = pdev->ucCardType;

	retVal = gaMideFunc[card].pfnFoundInterrupt(pdev, pChan);

#ifdef BP_SUPPORT
	if (!retVal) {
		idebp_check_dock_status(pdev);
	}
#endif
	if (!retVal) {
		retVal = mide_handle_athena_pm_hp(pdev);
	}
	return retVal;
}/*Found Interrupt() */

/*for all chipset support*/
BOOLEAN
EnableInterrupts(HW_DEVICE_EXTENSION *pdev)
{
	IDE_REGISTERS_1 *base1;
	IDE_REGISTERS_2 *base2;
	BM_REGISTERS   *BMReg = NULL;
	ULONG ulDrive, chan;
	UCHAR status;
	UCHAR tid;

	/*clear drive interrupts*/
	for (chan = 0; chan < MAX_CHANNELS_PER_CONTROLLER; chan++)
	{
		base1 = (IDE_REGISTERS_1 *) pdev->Channel[chan].BaseIoAddress1;
		base2 = (IDE_REGISTERS_2 *) pdev->Channel[chan].BaseIoAddress2;

		for (ulDrive = 0; ulDrive < MAX_DRIVES_PER_CHANNEL; ulDrive++)
		{
			tid = (chan << 1) + ulDrive;
			if (!IS_IDE_DRIVE(pdev, tid)) {
				continue;
			}

			SELECT_DEVICE(base1, ulDrive);
			GET_BASE_STATUS(base1, status);
		}
	}

	/*clear drive interrupt*/
	for (chan = 0; chan < MAX_CHANNELS_PER_CONTROLLER; chan++)
	{
		BMReg = pdev->Channel[chan].BaseBmAddress;
		CLEAR_BM_INT(BMReg, status);
	}

	BMReg = pdev->Channel[0].BaseBmAddress;

	/*
	 * Enable the Interrupt notification so that further interrupts are got.
	 * This is done because, there is no interrupt handler at the time
	 * before the actual registration of the Int. handler..
	 */

	return gaMideFunc[pdev->ucCardType].pfnEnableInterrupts(pdev);
}/*Enable Interrupts() */

/*This function dissables the controller interupt*/
BOOLEAN
DisableInterrupts(HW_DEVICE_EXTENSION *pdev)
{
	return gaMideFunc[pdev->ucCardType].pfnDisableInterrupts(pdev);
}/*Disable Interrupts*/

BOOLEAN
Is80PinCable(HW_DEVICE_EXTENSION *pdev, UCHAR chan)
{
	return gaMideFunc[pdev->ucCardType].pfnIs80PinCable(pdev, chan);
}

BOOLEAN
IsPostCommandToDriveSuccess(HW_DEVICE_EXTENSION *pdev,
                    ULONG tid,
                    ULONG ulStartBlock,
                    ULONG ulBlockCount,
                    UCHAR ucCmd,
                    ULONG ulSglPhysicalAddress)
{
	return gaMideFunc[pdev->ucCardType].pfnIsPostCmd(pdev, tid,
                     ulStartBlock, ulBlockCount, ucCmd, ulSglPhysicalAddress);

}

/**
 ** CMD648 and 649 Chipset Related Functions
 **/

BOOLEAN
Found64xInterrupt(HW_DEVICE_EXTENSION *pdev, UCHAR *pChan)
{
	BM_REGISTERS *baseBm;
	UCHAR status;
	ULONG chan;
	ULONG ulIntChannelValue = 0;

	/*
	 *	We are going to keep as a Bit Map where if one is set in
	 *  	one particular channel number place that means interrupt
	 *	has been raised from that channel.. if no bit was set
	 *	that means it is from a different card other than our PCI
	 *	Card
	 */

	/*
	 *	This interrupt status will show the
	 *	interrupt on two channels...(ie., per PCI Space
	 */
	for (chan = 0; chan<MAX_CHANNELS_PER_CONTROLLER; chan+=2) {
		baseBm = pdev->Channel[chan].BaseBmAddress;

		status = ScsiPortReadPortUchar((unsigned)&(baseBm->Reserved));

		if (status & CHAN1_INTERRUPT) {
			ulIntChannelValue |= (1 << chan);
		}

		if (status & CHAN2_INTERRUPT) {
			ulIntChannelValue |= (1 << (chan + 1));
		}
	}

	*pChan = ulIntChannelValue;

	if (ulIntChannelValue) {
		return TRUE;
	} else {
		return FALSE;
	}

}/*Found 64x Interrupt() */

static unsigned char auc64xUdmaModeValues[2][9] = {
	{0x0, 0x0, 0x0, 0x31, 0x21, 0x11, 0x25, 0x15, 0x05},   /*Master*/
	{0x0, 0x0, 0x0, 0xc2, 0x82, 0x42, 0x8a, 0x4a, 0x0a}    /*Slave*/
	};
	/* MDMA0, MDMA1, MDMA2, UDMA0, UMDA1, UDMA2, UDMA3, UDMA4, UDMA5*/

BOOLEAN
ProgramDriveFor64xChip(HW_DEVICE_EXTENSION *pdev,
			UCHAR tid, UCHAR TransferMode)
{
	UCHAR ucOrMask, ucAndMask, ucValue, devNo;
	BM_REGISTERS *bmBase;

	if (TransferMode > MP_UdmaMode5) {
		return FALSE;
	}

	devNo = (tid & 0x01);
	ucOrMask = auc64xUdmaModeValues[devNo][TransferMode - MP_DmaMode0];

	if (devNo) {
		ucAndMask = 0x35; 	/*if slave do not touch master bits*/
	} else {
		ucAndMask = 0xca; 	/*if master do not touch slave bits*/
	}

	bmBase = pdev->Channel[tid >> 1].BaseBmAddress;
	ucValue = ScsiPortReadPortUchar((unsigned)((UCHAR *) bmBase + 3));
	ucValue &= ucAndMask;
	ucValue |= ucOrMask;
	ScsiPortWritePortUchar((unsigned)((UCHAR *) bmBase + 3), ucValue);

	return(TRUE);

} /* end ProgramDriveFor64xChip()*/

BOOLEAN
Enable64xInterrupts(HW_DEVICE_EXTENSION *pdev)
{
	BM_REGISTERS   *BMReg = NULL;
	UCHAR opcimcr;

	BMReg = pdev->Channel[0].BaseBmAddress;

	opcimcr = ScsiPortReadPortUchar((unsigned)((UCHAR *) BMReg + 1));
	opcimcr &= 0xCF;
	opcimcr |= 0x01;
	ScsiPortWritePortUchar((unsigned)((UCHAR *) BMReg + 1), opcimcr);

	return AR_TRUE;
}/*Enable 64xInterrupts() */


BOOLEAN
Disable64xInterrupts(HW_DEVICE_EXTENSION *pdev)
{
	BM_REGISTERS   *BMReg = NULL;
	UCHAR opcimcr;

	BMReg = pdev->Channel[0].BaseBmAddress;

	/*
	 * Enable Interrupt notification so that further interrupts are got.
	 * This is done because, there is no interrupt handler at the time
	 * before the actual registration of the Int. handler..
	 */
	opcimcr = ScsiPortReadPortUchar((unsigned)((UCHAR *) BMReg + 1));
	opcimcr |= 0x30;
	ScsiPortWritePortUchar((unsigned)((UCHAR *) BMReg + 1), opcimcr);

	return AR_TRUE;
}/*Disable 64x Interrupts*/

BOOLEAN
Is64x80PinCable(HW_DEVICE_EXTENSION *pdev, UCHAR chan)
{
	UCHAR ucIs80PinCable;
	pci_read_config_byte_Rlsi(pdev, 0x79, &ucIs80PinCable);
	if (chan) {   /*secondary channel*/
		ucIs80PinCable = (ucIs80PinCable & 0x02) >> 1;
	} else {   /*primary channel*/
		ucIs80PinCable = (ucIs80PinCable & 0x01);
	}
	return(ucIs80PinCable);
}

BOOLEAN
IsPostCommandToDriveSuccess64x(HW_DEVICE_EXTENSION *pdev,
                    ULONG tid,
                    ULONG ulStartBlock,
                    ULONG ulBlockCount,
                    UCHAR ucCmd,
                    ULONG ulSglPhysicalAddress)
{
	CHANNEL *pChannel;
	IDE_REGISTERS_1 *base1;
	IDE_REGISTERS_2 *base2;
	BM_REGISTERS *baseBm;
	UCHAR ucStatus = 0;
	UCHAR intVal = ((tid >> 1) ? CHAN2_INTERRUPT:CHAN1_INTERRUPT)|0x1;

	pChannel = &(pdev->Channel[(tid >> 1)]);
	base1 = (IDE_REGISTERS_1 *) pChannel->BaseIoAddress1;
	base2 = (IDE_REGISTERS_2 *) pChannel->BaseIoAddress2;
	baseBm = pChannel->BaseBmAddress;


	PostCommandToDrive(pdev, tid, ulStartBlock,
	                   ulBlockCount, ucCmd, ulSglPhysicalAddress);
	POLL_FOR_INTERRUPT(pdev, intVal);
	POLL_FOR_DMA_INTERRUPT(baseBm)

	ScsiPortWritePortUchar((unsigned)&(baseBm->Command.AsUchar),
	                       STOP_TRANSFER);

	GET_BASE_STATUS(base1, ucStatus);

	/*Clear interrupt*/
	ClearPCIInterupt(pdev, tid >> 1);

	/*if no error continue;*/
	if (ucStatus & IDE_STATUS_BUSY) {
		return FALSE;
	}

	/*if no error continue*/
	if (ucStatus & IDE_STATUS_ERROR) {
		return FALSE;
	}

	return TRUE;
}

BOOLEAN
Found68xInterrupt(HW_DEVICE_EXTENSION *pdev, UCHAR *pChan)
{
unsigned baseBm;
UCHAR status;
ULONG chan = 0;
ULONG ulIntChannelValue = 0;

	baseBm = (unsigned)&(pdev->Channel[0].BaseBmAddress->Command.AsUchar);

	if(pdev->Channel[0].ExpectingInterrupt){
		status = ScsiPortReadPortUchar((baseBm + 0x02));
		if ((status & 0x4) == 0x4) {
			ulIntChannelValue |= (1 << chan);
		}
	}

	if(pdev->Channel[1].ExpectingInterrupt){
		status = ScsiPortReadPortUchar((baseBm + 0x0A));
		if ((status & 0x4) == 0x4) {
			ulIntChannelValue |= (1 << (chan + 1));
		}
	}

	*pChan = ulIntChannelValue;

	if (ulIntChannelValue) {
		return TRUE;
	} else {
		return FALSE;
	}
}/*Found 68x Interrupt() */

/*Refer Sil0680 Data Sheet for explanation of these values*/
static UCHAR auc68xUdmaMode[7] = {0x0F,0x0B,0x07,0x05,0x03,0x02,0x01 };
static USHORT auc68xDmaMode[3] = { 0x2208,0x10C2,0x10C1};
static USHORT auc68xPIOMode[5] = { 0x328A,0x2283,0x1104,0x10C3,0x10C1};
static USHORT auc68xTFTiming[5]= { 0x328A,0x2283,0x1281,0x10C3,0x10C1};


BOOLEAN
ProgramDriveFor68xChip(HW_DEVICE_EXTENSION *pdev, UCHAR tid, UCHAR ucMode)
{
	u_char ucReg = 0;
	u_char tf_mode, pio_mode, mdma_mode, udma_mode;
	u_char ucReg80 = 0, ucReg84 = 0;
	MP_DRIVE_ID drive_id = (MP_DRIVE_ID) tid;

	/* program IDE CLK Select to 133 MHz*/
	pci_read_config_byte_Rlsi(pdev,SYS_CONF_STAT_CMD_REG680+2, &ucReg);
	ucReg = (ucMode >= MP_UdmaMode0) ?(ucReg | 0x10) :(ucReg & 0xEF);
	pci_write_config_byte_Rlsi(pdev,SYS_CONF_STAT_CMD_REG680+2, ucReg);

	/*PIO2 = MDMA0 PIO3 = MDMA1 PIO4 = MDMA2(and UDMAx)*/
	/*program TF Timing*/

	tf_mode = (ucMode < MP_UdmaMode0) ?
		(ucMode - MP_DmaMode0):(MP_DmaMode2 - MP_DmaMode0);
	tf_mode += 2;

	/*program PIO Timing*/
	pio_mode = (ucMode < MP_UdmaMode0) ?
		(ucMode - MP_DmaMode0) :(MP_DmaMode2 - MP_DmaMode0);
	pio_mode += 2;

	/*program mdma timing*/
	mdma_mode = (ucMode < MP_UdmaMode0) ?
		(ucMode - MP_DmaMode0) :(MP_DmaMode2 - MP_DmaMode0);

	/*program dma timing*/
	udma_mode = (ucMode >=  MP_UdmaMode0) ?
		(ucMode - MP_UdmaMode0) : 6;

	/*set TF Timing*/
	if ((tid > 1) == 0) {
		pci_write_config_byte_Rlsi(pdev, IDE0_TFT_CONF_STAT_REG680+2,
		                           auc68xTFTiming[tf_mode]);
	} else {
		pci_write_config_byte_Rlsi(pdev, IDE1_TFT_CONF_STAT_REG680+2,
		                           auc68xTFTiming[tf_mode]);
	}

	/* get Transfer Mode registers values*/
	/* bits 1:0 - Device0 { 2=mdma, 3=udma }*/
	/* bits 5:4 - Device1 {2=mdma, 3=udma }*/
	pci_read_config_byte_Rlsi(pdev, DATA_TRFR_MODE_IDE0_REG680, &ucReg80);
	pci_read_config_byte_Rlsi(pdev, DATA_TRFR_MODE_IDE1_REG680, &ucReg84);

	ucReg80 &= ((tid & 1) ? 0xCF : 0xFC);
	ucReg84 &= ((tid & 1) ? 0xCF : 0xFC);

	if (ucMode >= MP_UdmaMode0) {
		ucReg80 |= ((tid & 1) ? 0x30 : 0x03);
		ucReg84 |= ((tid & 1) ? 0x30 : 0x03);
	} else if (ucMode >= MP_DmaMode0) {
		ucReg80 |= ((tid & 1) ? 0x20 : 0x02);
		ucReg84 |= ((tid & 1) ? 0x20 : 0x02);
	}

	switch (drive_id)
	{
	 case PrimaryMaster :
		/* set PIO Timing*/
		pci_write_config_word_Rlsi(pdev, IDE0_PIO_TIMING_REG680,
		                           auc68xPIOMode[pio_mode]);

		/* set mdma timing*/
		pci_write_config_word_Rlsi(pdev, IDE0_DMA_TIMING_REG680,
		                           auc68xDmaMode[mdma_mode]);

		/* set udma timing*/
		pci_write_config_byte_Rlsi(pdev, IDE0_UDMA_TIMING_REG680,
		                           auc68xUdmaMode[udma_mode]);

		/* set tran mode*/
		pci_write_config_byte_Rlsi(pdev, DATA_TRFR_MODE_IDE0_REG680,
			ucReg80);

		break;


	 case PrimarySlave :
		/* set PIO Timing*/
		pci_write_config_word_Rlsi(pdev, IDE0_PIO_TIMING_REG680+2,
		                           auc68xPIOMode[pio_mode]);

		/* set mdma timing*/
		pci_write_config_word_Rlsi(pdev, IDE0_DMA_TIMING_REG680+2,
		                           auc68xDmaMode[mdma_mode]);

		/* set udma timing*/
		pci_write_config_byte_Rlsi(pdev, IDE0_UDMA_TIMING_REG680+2,
		                           auc68xUdmaMode[udma_mode]);

		/* set tran mode*/
		pci_write_config_byte_Rlsi(pdev, DATA_TRFR_MODE_IDE0_REG680,
		                           ucReg80);

		break;


	 case SecondaryMaster :
	 	/* set PIO Timing*/
		pci_write_config_word_Rlsi(pdev, IDE1_PIO_TIMING_REG680,
		                           auc68xPIOMode[pio_mode]);
		/* set mdma timing*/
		pci_write_config_word_Rlsi(pdev, IDE1_DMA_TIMING_REG680,
		                           auc68xDmaMode[mdma_mode]);
		/* set udma timing*/
		pci_write_config_byte_Rlsi(pdev, IDE1_UDMA_TIMING_REG680,
		                           auc68xUdmaMode[udma_mode]);
		/* set tran mode*/
		pci_write_config_byte_Rlsi(pdev, DATA_TRFR_MODE_IDE1_REG680,
		                           ucReg84);
		break;

	 case SecondarySlave :
		/* set PIO Timing*/
		pci_write_config_word_Rlsi(pdev, IDE1_PIO_TIMING_REG680+2,
		                           auc68xPIOMode[pio_mode]);
		/* set mdma timing*/
		pci_write_config_word_Rlsi(pdev, IDE1_DMA_TIMING_REG680+2,
		                           auc68xDmaMode[mdma_mode]);
		/* set udma timing*/
		pci_write_config_byte_Rlsi(pdev, IDE1_UDMA_TIMING_REG680+2,
		                           auc68xUdmaMode[udma_mode]);
		/* set tran mode*/
		pci_write_config_byte_Rlsi(pdev, DATA_TRFR_MODE_IDE1_REG680,
		                           ucReg84);
		break;

	 default :
		break;
	}
	return(TRUE);
} /* end ProgramDriveFor68xChip()*/

BOOLEAN
Enable68xInterrupts(HW_DEVICE_EXTENSION *pdev)
{
	UCHAR opcimcr;

	pci_read_config_byte_Rlsi(pdev, SYS_CONF_STAT_CMD_REG680+2, &opcimcr);
	opcimcr &= 0x3F; /*0011 111*/
	pci_write_config_byte_Rlsi(pdev, SYS_CONF_STAT_CMD_REG680+2, opcimcr);

	return AR_TRUE;
}/*Enable 68xInterrupts() */

BOOLEAN
Disable68xInterrupts(HW_DEVICE_EXTENSION *pdev)
{
	UCHAR opcimcr;

	pci_read_config_byte_Rlsi(pdev, SYS_CONF_STAT_CMD_REG680+2, &opcimcr);
	opcimcr |= 0xC0; /*1100 0000*/
	pci_write_config_byte_Rlsi(pdev, SYS_CONF_STAT_CMD_REG680+2, opcimcr);

	return AR_TRUE;
}/*Disable 68xInterrupts*/

BOOLEAN
Is68x80PinCable(HW_DEVICE_EXTENSION *pdev, UCHAR chan)
{
	UCHAR ucIs80PinCable;

	if (chan) {   /*secondary channel*/
		pci_read_config_byte_Rlsi(pdev, 0xB0, &ucIs80PinCable);
	} else {   /*primary channel*/
		pci_read_config_byte_Rlsi(pdev, 0xA0, &ucIs80PinCable);
	}
	return(ucIs80PinCable & 0x01);
}

BOOLEAN
IsPostCommandToDriveSuccess68x(HW_DEVICE_EXTENSION *pdev,
                    ULONG tid,
                    ULONG ulStartBlock,
                    ULONG ulBlockCount,
                    UCHAR ucCmd,
                    ULONG ulSglPhysicalAddress)
{
	CHANNEL *pChannel;
	IDE_REGISTERS_1 *base1;
	IDE_REGISTERS_2 *base2;
	BM_REGISTERS *baseBm;
	UCHAR ucStatus = 0;

	pChannel = &(pdev->Channel[(tid >> 1)]);
	base1 = (IDE_REGISTERS_1 *) pChannel->BaseIoAddress1;
	base2 = (IDE_REGISTERS_2 *) pChannel->BaseIoAddress2;
	baseBm = pChannel->BaseBmAddress;

	PostCommandToDrive(pdev, tid, ulStartBlock, ulBlockCount,
	                   ucCmd, ulSglPhysicalAddress);

	POLL_FOR_INTERRUPT_68X(pdev, tid);
	POLL_FOR_DMA_INTERRUPT(baseBm)

	ScsiPortWritePortUchar((unsigned)&(baseBm->Command.AsUchar),
	                       STOP_TRANSFER);

	GET_BASE_STATUS(base1, ucStatus);

	/* Clear interrupt*/
	ClearPCIInterupt(pdev, tid >> 1);

	if (ucStatus & IDE_STATUS_BUSY) {
		printk_Rlsi("megaide: 680 intr busy bit set tid=%ld\n", tid);
		return FALSE;
	}

	/*if no error continue;*/
	if (ucStatus & IDE_STATUS_ERROR){
		UCHAR err;

		printk_Rlsi("megaide: 680 intr err bit set tid=%ld\n", tid);
		err = ScsiPortReadPortUchar((unsigned)((UCHAR *) base1 + 1));
		printk_Rlsi("megaide: error byte = 0x%x\n", err);
		return FALSE;
	}

	return TRUE;
}

/**
 ** CSB5/ CSB6  Chipset Related Functions
 **/

BOOLEAN
FoundCSB5Interrupt(HW_DEVICE_EXTENSION *pdev, UCHAR *pChan)
{
unsigned baseBm;
UCHAR status;
ULONG chan = 0;
ULONG ulIntChannelValue = 0;

	baseBm = (unsigned)&(pdev->Channel[0].BaseBmAddress->Command.AsUchar);

	if(pdev->Channel[0].ExpectingInterrupt){
		status = ScsiPortReadPortUchar((baseBm + 0x02));
		if ((status & 0x4) == 0x4) {
			ulIntChannelValue |= (1 << chan);
		}
	}

	if(pdev->Channel[1].ExpectingInterrupt){
		status = ScsiPortReadPortUchar((baseBm + 0x0A));
		if ((status & 0x4) == 0x4) {
			ulIntChannelValue |= (1 << (chan + 1));
		}
	}

	*pChan = ulIntChannelValue;

	if (ulIntChannelValue) {
		return TRUE;
	} else {
		return FALSE;
	}
}/*Found CSB5 Interrupt() */

/* Please refer CSB5 Data Sheet Ver 2.0a for explanation of these values*/
static USHORT aucCSB5PIOTimingValues[5] = { 0x50,  0x47, 0x34, 0x22, 0x20};
static USHORT aucCSB5DmaTimingMode[3] = { 0x77, 0x21, 0x20 };
static USHORT aucCSB5UdmaMode0[7] = {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x05};
static USHORT aucCSB5UdmaMode1[7] = {0x00, 0x10, 0x20, 0x30, 0x40, 0x50, 0x50};

BOOLEAN
ProgramDriveForCSB5Chip(HW_DEVICE_EXTENSION *pdev, UCHAR tid, UCHAR ucMode)
{
	UCHAR   ucReg = 0;
	u_char  pio_mode, mdma_mode, udma_mode;
	MP_DRIVE_ID drive_id = (MP_DRIVE_ID) tid;

	/* PIO2 = MDMA0
	 * PIO3 = MDMA1
	 * PIO4 = MDMA2(and UDMAx)
	 */

	/* program PIO Timing*/
	pio_mode = (ucMode < MP_UdmaMode0) ?
		(ucMode - MP_DmaMode0) :(MP_DmaMode2 - MP_DmaMode0);
	pio_mode += 2;

	/*program mdma timing*/
	mdma_mode = (ucMode < MP_UdmaMode0) ?
		(ucMode - MP_DmaMode0) :(MP_DmaMode2 - MP_DmaMode0);

	/*program dma timing*/
	udma_mode = (ucMode >=  MP_UdmaMode0) ?
		(ucMode - MP_UdmaMode0) : 6;

	switch (drive_id)
	{
		case PrimaryMaster :
		{
		  /*PIO Timing Registers*/
		  pci_write_config_byte_Rlsi(pdev, 0x41,
		                             aucCSB5PIOTimingValues [pio_mode]);
		  /*DMA Timing Mode Registers*/
		  pci_write_config_byte_Rlsi(pdev, 0x45,
		                             aucCSB5DmaTimingMode[mdma_mode]);
		  /*PIO Mode Register*/
		  pci_read_config_byte_Rlsi(pdev, 0x4A, &ucReg);
		  ucReg |= 0x04;
		  pci_write_config_byte_Rlsi(pdev, 0x4A, ucReg);
		  /*UDMA Control Register*/
		  pci_read_config_byte_Rlsi(pdev, 0x54, &ucReg);
		  ucReg |= 0x01;
		  pci_write_config_byte_Rlsi(pdev, 0x54, ucReg);
		  /*ATA Mode Register*/
		  pci_read_config_byte_Rlsi(pdev, 0x56, &ucReg);
		  ucReg |= aucCSB5UdmaMode0[udma_mode];
		  pci_write_config_byte_Rlsi(pdev, 0x56, ucReg);
		  break;
		}

		case PrimarySlave :
		{

		  pci_write_config_byte_Rlsi(pdev, 0x40,
		                             aucCSB5PIOTimingValues [pio_mode]);
		  pci_write_config_byte_Rlsi(pdev, 0x44,
		                             aucCSB5DmaTimingMode[mdma_mode]);
		  pci_read_config_byte_Rlsi(pdev, 0x4A, &ucReg);

		  ucReg |= 0x40;
		  pci_write_config_byte_Rlsi(pdev, 0x4A, ucReg);
		  pci_read_config_byte_Rlsi(pdev, 0x54, &ucReg);

		  ucReg |= 0x02;
		  pci_write_config_byte_Rlsi(pdev, 0x54, ucReg);
		  pci_read_config_byte_Rlsi(pdev, 0x56, &ucReg);

		  ucReg |= aucCSB5UdmaMode1[udma_mode];
		  pci_write_config_byte_Rlsi(pdev, 0x56, ucReg);
		  break;
		}

		case SecondaryMaster :
		{
		  pci_write_config_byte_Rlsi(pdev, 0x43,
		  aucCSB5PIOTimingValues [pio_mode]);
		  pci_write_config_byte_Rlsi(pdev, 0x47,
		                             aucCSB5DmaTimingMode[mdma_mode]);
		  pci_read_config_byte_Rlsi(pdev, 0x4B, &ucReg);

		  ucReg |= 0x04;
		  pci_write_config_byte_Rlsi(pdev, 0x4B, ucReg);
		  pci_read_config_byte_Rlsi(pdev, 0x54, &ucReg);

		  ucReg |= 0x04;
		  pci_write_config_byte_Rlsi(pdev, 0x54, ucReg);
		  pci_read_config_byte_Rlsi(pdev, 0x57, &ucReg);

		  ucReg |= aucCSB5UdmaMode0[udma_mode];
		  pci_write_config_byte_Rlsi(pdev, 0x57, ucReg);
		  break;
		}

		case SecondarySlave :
		{
		  pci_write_config_byte_Rlsi(pdev, 0x42,
		                             aucCSB5PIOTimingValues [pio_mode]);
		  pci_write_config_byte_Rlsi(pdev, 0x46,
		                             aucCSB5DmaTimingMode[mdma_mode]);
		  pci_read_config_byte_Rlsi(pdev, 0x4B, &ucReg);

		  ucReg |= 0x40;
		  pci_write_config_byte_Rlsi(pdev, 0x4B, ucReg);
		  pci_read_config_byte_Rlsi(pdev, 0x54, &ucReg);

		  ucReg |= 0x08;
		  pci_write_config_byte_Rlsi(pdev, 0x54, ucReg);
		  pci_read_config_byte_Rlsi(pdev, 0x57, &ucReg);

		  ucReg |= aucCSB5UdmaMode1[udma_mode];
		  pci_write_config_byte_Rlsi(pdev, 0x57, ucReg);
		  break;
		}

		default :
		{
		  AR_DEBUG(AR_DEBUG_WARNING, (KERN_DEBUG
		           "megaide: wrong ID ProgramDriveForCSB5Chip\n"));
		  break;
		}
	}/*switch (drive_id)*/

	return TRUE;
}

BOOLEAN
EnableCSB5Interrupts(HW_DEVICE_EXTENSION *pdev)
{
	ScsiPortWritePortUchar(0xC00, 0x02);
	ScsiPortWritePortUchar(0xC01, pdev->ulIntLine);
	ScsiPortWritePortUchar(0xC00, 0x04);
	ScsiPortWritePortUchar(0xC01, pdev->ulIntLine);
	return AR_TRUE;
}/*Enable CSB5Interrupts() */

BOOLEAN
DisableCSB5Interrupts(HW_DEVICE_EXTENSION *pdev)
{
	ScsiPortWritePortUchar(0xC00, 0x02);
	ScsiPortWritePortUchar(0xC01, 0x00);
	ScsiPortWritePortUchar(0xC00, 0x04);
	ScsiPortWritePortUchar(0xC01, 0x00);
	return AR_TRUE;
}/*Disable CSB5Interrupts*/

BOOLEAN
IsCSB580PinCable(HW_DEVICE_EXTENSION *pdev, UCHAR chan)
{
	/*always expect a 80 pin cable*/
	return 0x01;
}

BOOLEAN
IsPostCommandToDriveSuccessCSB5(HW_DEVICE_EXTENSION *pdev,
                    ULONG tid,
                    ULONG ulStartBlock,
                    ULONG ulBlockCount,
                    UCHAR ucCmd,
                    ULONG ulSglPhysicalAddress)
{
	CHANNEL *pChannel;
	IDE_REGISTERS_1 *base1;
	IDE_REGISTERS_2 *base2;
	BM_REGISTERS *baseBm;
	UCHAR ucStatus = 0;

	pChannel = &(pdev->Channel[(tid >> 1)]);
	base1 = (IDE_REGISTERS_1 *) pChannel->BaseIoAddress1;
	base2 = (IDE_REGISTERS_2 *) pChannel->BaseIoAddress2;
	baseBm = pChannel->BaseBmAddress;

	PostCommandToDrive(pdev, tid, ulStartBlock, ulBlockCount,
	                   ucCmd, ulSglPhysicalAddress);

	POLL_FOR_INTERRUPT_CSB5(pdev, tid);
	POLL_FOR_DMA_INTERRUPT(baseBm)

	ScsiPortWritePortUchar((unsigned)&(baseBm->Command.AsUchar),
	                       STOP_TRANSFER);

	GET_BASE_STATUS(base1, ucStatus);


	if (ucStatus & IDE_STATUS_BUSY) {
		return FALSE;
	}

	/* if no error continue;*/
	if (ucStatus & IDE_STATUS_ERROR) {
		return FALSE;
	}

	return TRUE;
}

BOOLEAN
FoundCSB6Interrupt(HW_DEVICE_EXTENSION *pdev, UCHAR *pChan)
{
UCHAR status;
ULONG chan = 0;
ULONG ulIntChannelValue = 0;
PBM_REGISTERS baseBm;
PIDE_REGISTERS_2 baseIoAddress2;

	for (chan=0; chan<2; chan++)
	{
		baseBm = pdev->Channel[chan].BaseBmAddress;
		baseIoAddress2 = pdev->Channel[chan].BaseIoAddress2;

		if(pdev->Channel[chan].ExpectingInterrupt == IDE_DMA_INTERRUPT){
			status = ScsiPortReadPortUchar((unsigned)&(baseBm->Status));
			if ((status & BM_STATUS_INTERRUPT) || (status & BM_STATUS_ERROR)) {
				ulIntChannelValue |= (1 << chan);
				continue;
			}
		}

		if (pdev->Channel[chan].ExpectingInterrupt == IDE_PIO_INTERRUPT){
			status = ScsiPortReadPortUchar((unsigned)&(baseBm->Status));
			if (!(status & IDE_STATUS_BUSY)) {
				ulIntChannelValue |= (1 << chan);
				continue;
			}
		}
	}

	*pChan = ulIntChannelValue;

	if (ulIntChannelValue) {
		return TRUE;
	} else {
		return FALSE;
	}
}/*Found CSB6 Interrupt() */


BOOLEAN
EnableCSB6Interrupts(HW_DEVICE_EXTENSION *pdev)
{
	return AR_TRUE;
}/*Enable CSB6Interrupts() */

BOOLEAN
DisableCSB6Interrupts(HW_DEVICE_EXTENSION *pdev)
{
	return AR_TRUE;
}/*Disable CSB6Interrupts*/

BOOLEAN
IsCSB680PinCable(HW_DEVICE_EXTENSION *pdev, UCHAR chan)
{
	/*always expect a 80 pin cable*/
	return 0x01;
}

BOOLEAN
ProgramDriveForCSB6Chip(HW_DEVICE_EXTENSION *pdev, UCHAR tid, UCHAR ucMode)
{
	return TRUE;
}

/**
 ** Silicon Image Athena Related Functions
 **/

BOOLEAN
EnableAthenaInterrupts(HW_DEVICE_EXTENSION *pdev)
{
	UCHAR opcimcr;

	pci_read_config_byte_Rlsi(pdev, SYS_CONF_STAT_CMD_REG680+2, &opcimcr);
	opcimcr &= 0x3F; /*0011 111*/
	pci_write_config_byte_Rlsi(pdev, SYS_CONF_STAT_CMD_REG680+2, opcimcr);

	AthenaEnablePMInterrupts(pdev);
	AthenaEnableHPInterrupt(pdev);
	return AR_TRUE;
}/*EnableAthenaInterrupts() */

BOOLEAN
DisableAthenaInterrupts(HW_DEVICE_EXTENSION *pdev)
{
	UCHAR opcimcr;

	pci_read_config_byte_Rlsi(pdev, SYS_CONF_STAT_CMD_REG680+2, &opcimcr);
	opcimcr |= 0xC0; /*1100 0000*/
	pci_write_config_byte_Rlsi(pdev, SYS_CONF_STAT_CMD_REG680+2, opcimcr);

	AthenaDisablePMInterrupts(pdev);
	AthenaDisableHPInterrupt(pdev);
	return AR_TRUE;

}/*DisableAthenaInterrupts*/

/**
 * FoundICH5RInterrupt: Found ICH5R interrupt
 * This function checks whether any ICH5R channel raised any interrupt.
 * This check is done only if Driver expecting an interrupt from that
 * channel.
 *
 * Function parameters
 * @pdev	:	ICH5R device extension structure pointer
 * @p_int_return_val:	Interrupt detect status
 *		return 0x01 if interrupt detect in primary channel
 *		return 0x02 if interrupt detect in Secondary channel
 *		return 0x03 if interrupt detect in both channel
 * @return;
 *	TRUE if interrupt detected
 *	FALSE if no interrupt detected
 **/

BOOLEAN
FoundICH5RInterrupt(HW_DEVICE_EXTENSION *pdev, UCHAR *p_int_return_val)
{
	unsigned	baseBm;

	*p_int_return_val = 0;
	baseBm = (unsigned) &(pdev->Channel[0].BaseBmAddress->Command.AsUchar);

	if(pdev->Channel[0].ExpectingInterrupt) {
		if ((ScsiPortReadPortUchar((baseBm + 0x02)) & 0x4) == 0x4) {
			*p_int_return_val |= 0x01; /* found chan 1 interrupt */
		}
	}

	if(pdev->Channel[1].ExpectingInterrupt){
		if ((ScsiPortReadPortUchar((baseBm + 0x0A)) & 0x4) == 0x4) {
			*p_int_return_val |= 0x02; /* found chan 2 interrupt */
		}
	}
	
	if (!(*p_int_return_val))
		AR_DEBUG(AR_DEBUG_ERRORS, (KERN_DEBUG
		           "megaide: spurios interrupt in ICH5R\n"));
			   
	return ((*p_int_return_val) ? TRUE: FALSE);
}

/**
 * ProgramDriveForICH5RChip: Program drives connected to ICH5R controller
 * This function supposed to program a drive connected to thecontroller.
 * It is used if bios is not programming the drives and in case of newly.
 *
 * Function parameters
 * @pdev	:	ICH5R device extension structure pointer
 * @tid		:	Target ID
 *		00h for primary master
 *		02h for secondary master
 * @ucMode	: Trasnsfer mode. it is defined
 *		MP_TRANSFER_MODE definition gives the values and meaning
 * @return;
 *	TRUE if succeed
 *	FALSE if failed
 **/

BOOLEAN
ProgramDriveForICH5RChip(HW_DEVICE_EXTENSION *pdev, UCHAR tid, UCHAR ucMode)
{
	return AR_TRUE;
}

/**
 * EnableICH5RInterrupts: Enable ICH5R Interrupt
 * This function enables the ICH5R interrupts by programming pci config space.
 *
 * Function parameters
 * @pdev	:	ICH5R device extension structure pointer
 *
 * @return;
 *	TRUE if succeed
 *	FALSE if failed
 **/

BOOLEAN
EnableICH5RInterrupts(HW_DEVICE_EXTENSION *pdev)
{
	UCHAR	reg_val;

	/* reading interrupt control register */
	pci_read_config_byte_Rlsi(pdev, 0x05, &reg_val);

	/* Enabling interrupt by writing 0 to 3rd bit */
	reg_val ^= 0x4;

	/* writing interrupt control register */
	pci_write_config_byte_Rlsi(pdev, 0x05, reg_val);

	ScsiPortStallExecution(1000);

	return AR_TRUE;
}

/**
 * DisableICH5RInterrupts: Disable ICH5R Interrupt
 * This function Disables the ICH5R interrupts by programming pci config space.
 *
 * Function parameters
 * @pdev	:	ICH5R device extension structure pointer
 *
 * @return;
 *	TRUE if succeed
 *	FALSE if failed
 **/

BOOLEAN
DisableICH5RInterrupts(HW_DEVICE_EXTENSION *pdev)
{
	UCHAR	reg_val;

	/* reading interrupt control register */
	pci_read_config_byte_Rlsi(pdev, 0x05, &reg_val);

	/* disabling interrupt by writing 1 to 3rd bit */
	reg_val |= 0x4;

	/* writing interrupt control register */
	pci_write_config_byte_Rlsi(pdev, 0x05, reg_val);

	ScsiPortStallExecution(1000);

	return AR_TRUE;
}

/**
 * IsICH5R80PinCable: Is This device using 80 pin cable?
 * This function will always return TRUE because this return value is used
 * for other ATA specific programming.
 *
 * Function parameters
 * @pdev	:	ICH5R device extension structure pointer
 * @ucChannelId	:	Channel ID 00h-Primary, 01h-secondary
 * @return;
 *	TRUE always
 **/
BOOLEAN
IsICH5R80PinCable(HW_DEVICE_EXTENSION *pdev, UCHAR ucChannelId)
{
	return AR_TRUE;
}

/**
 * IsPostCommandToDriveSuccessICH5R: Post a Read Or Write Command to ICH5R Drive
 * This function is used to issue a read/write command to the drives
 * connected to ICH5R controller. This function is used to write/read
 * Errorlog, IRCD, Enquiry etc
 *
 * Function parameters
 * @pdev	:	ICH5R device extension structure pointer
 * @tid		:	Target ID 00h - primary master, 02h - secondary master
 * @ulStartBlock:	Start Logica Block address
 * @ulBlockCount:	Number of Blocks
 * @ucCmd	:	ATA read/write command READ_6/WRITE_6
 * @ulSglPhysicalAddress: Physical address of the SGList used for the command
 * @return;
 *	TRUE always
 **/
BOOLEAN
IsPostCommandToDriveSuccessICH5R(HW_DEVICE_EXTENSION *pdev, ULONG tid,
			ULONG ulStartBlock, ULONG ulBlockCount,
			UCHAR ucCmd, ULONG ulSglPhysicalAddress)
{
	CHANNEL *pChannel;
	IDE_REGISTERS_1 *base1;
	IDE_REGISTERS_2 *base2;
	BM_REGISTERS *baseBm;
	UCHAR ucStatus = 0;
	UCHAR ucStatus2;
	
	pChannel = &(pdev->Channel[(tid >> 1)]);
	base1 = (IDE_REGISTERS_1 *) pChannel->BaseIoAddress1;
	base2 = (IDE_REGISTERS_2 *) pChannel->BaseIoAddress2;
	baseBm = pChannel->BaseBmAddress;

	PostCommandToDrive(pdev, tid, ulStartBlock, ulBlockCount,
	                   ucCmd, ulSglPhysicalAddress);

	POLL_FOR_DMA_INTERRUPT(baseBm)

	ScsiPortWritePortUchar((unsigned)&(baseBm->Command.AsUchar),
	                       STOP_TRANSFER);

	GET_BASE_STATUS(base1, ucStatus);

	CLEAR_BM_INT(baseBm, ucStatus2);
	
	if (ucStatus & IDE_STATUS_BUSY) {
		printk_Rlsi("megaide: ICH5R intr busy bit set tid=%ld\n", tid);
		return FALSE;
	}

	/*if no error continue;*/
	if (ucStatus & IDE_STATUS_ERROR){
		UCHAR err;
		printk_Rlsi("megaide: ICH5R intr err bit set tid=%ld\n", tid);
		err = ScsiPortReadPortUchar((unsigned)((UCHAR *) base1 + 1));
		printk_Rlsi("megaide: error byte = 0x%x\n", err);
		return FALSE;
	}

	return TRUE;
}

/**
 * ICH5R_reset_all_channels: reset both ICH5R channels
 * This function enable and dissable both channels in ICH5R to make sure 
 * Port control status register is updated with drive presence status. This 
 * is done by dissabling and enabling both channels.
 *
 * Function parameters
 * @pdev	:	ICH5R device extension structure pointer
 * @return	:	void
 **/
void
ICH5R_reset_all_channels(HW_DEVICE_EXTENSION *pdev)
{
	UCHAR	status;
	int	i;

	/* reading port control and status register */
	pci_read_config_byte_Rlsi(pdev, 0x92, &status);

	/* disabling both channels */
	status ^= 0x03;
	pci_write_config_byte_Rlsi(pdev, 0x92, status);
	
	/* as per intel 400 to 500 milli second delay is required between
	 * disabling and enabling events of ports
	 */
	for (i = 0; i < 500; i++) {
		ScsiPortStallExecution(1000);
	}

	/* enabling both channels */
	status |= 0x03;
	pci_write_config_byte_Rlsi(pdev, 0x92, status);
	
	for (i = 0; i < 500; i++) {
		ScsiPortStallExecution(1000);
	}
}

/*vi set ts=8:*/

