/*megaide_char.h*/
/*
 * Linux MegaRAID IDEal Software RAID Stack
 * Copyright (c) 2001 LSI Logic Corporation, All Rights Reserved.
 *-----------------------------------------------------------------------------
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *-----------------------------------------------------------------------------
 */


static int giMajorNumber;


static struct file_operations megaidedev_fops = {
        ioctl:    megaidedev_ioctl,
        open:     megaidedev_open,
        release:  megaidedev_close,
};

void
megaidedev_attach(void)
{
	giMajorNumber = register_chrdev(0, "megaidedev", &megaidedev_fops);
}

void
megaidedev_detach(void)
{
	unregister_chrdev(giMajorNumber, "megaidedev");
}

static int
megaidedev_open(struct inode *inode, struct file *filep)
{
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
	return 0;/* success */
}

static int
megaidedev_close(struct inode *inode, struct file *filep)
{
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
	return 0;/* success */
}

static int
megaidedev_ioctl(struct inode *inode, struct file *filep,
                 unsigned int cmd, unsigned long arg)
{
	int chk = 0;
	int iRetVal = -EFAULT;
	char_megaide_ioctl ioctl;
	HW_DEVICE_EXTENSION *pdev = NULL;
	adapter_t *adapter;

	chk = access_ok(VERIFY_READ, arg, sizeof(char_megaide_ioctl));
	if (chk != 1) {
 		AR_DEBUG(AR_DBG_NIT,("megaide: VERIFY_READ failed"));
		return -EFAULT;
	}

	if (copy_from_user(&ioctl,(char *) arg, sizeof(char_megaide_ioctl))) {
		AR_DEBUG(AR_DBG_NIT,("megaide: copy char_megaide_ioctl failed"));
		return -EFAULT;
	}

	/*data area check for requested data size*/
	chk = access_ok(VERIFY_WRITE, arg, (CHAR_HEADER_SIZE + ioctl.outlen));
	if (chk != 1) {
		AR_DEBUG(AR_DBG_NIT,("megaide: VERIFY_WRITE failed"));
		return -EFAULT;
	}

	/*incoming data area check*/
	chk = access_ok(VERIFY_READ, arg,(CHAR_HEADER_SIZE + ioctl.inlen));
	if (chk != 1) {
 		AR_DEBUG(AR_DBG_NIT,("megaide: VERIFY_READ failed"));
		return -EFAULT;
	}

	if (memcmp_Rlsi(ioctl.acSignature, "MEGANIT", 7) != 0) {
		AR_DEBUG(AR_DBG_NIT,("megaide: wrong signature\n"));
		return -EINVAL;
	}

	if ((ioctl.iAdapNumber > giNumberOfCtrlrs) || (!ioctl.iAdapNumber)) {
		AR_DEBUG(AR_DBG_NIT,("megaide: Wrong adapter number\n"));
		return -EINVAL;
	}

	/*one ioctl only allowed at a time */
	adapter = gastCardInfo[ioctl.iAdapNumber-1];
	pdev = &adapter->DeviceExtension;

	down(&adapter->nit_lock);
	iRetVal = mide_handle_ioctl(cmd, arg, ioctl.inlen,
	                            ioctl.outlen, ioctl.iAdapNumber);
	up(&adapter->nit_lock);

	return iRetVal;
}

static void
mide_char_ioctl_over(Scsi_Cmnd * scmd)
{
	HW_DEVICE_EXTENSION *pdev = NULL;
	adapter_t *adapter;
	AR_DEBUG(AR_DBG_NIT,(" megaide: mide_char_ioctl_over\n"));

	pdev = (HW_DEVICE_EXTENSION *) scmd->host;
	adapter = (adapter_t *) pdev->adapter;
	wake_up_interruptible(&adapter->wait_queue);
	scmd->state=MIDE_CHAR_IOCTL_FINISHED;

}/*mide_char_ioctl_over()*/

int
mide_handle_ioctl(unsigned int cmd, unsigned long arg,
                  u_long inlen, u_long outlen, int adp_no)
{
	int i;
	int iRetVal = -EFAULT;
	int data_size = 0;
	Scsi_Cmnd *scmd;
	unsigned long flags=0;
	pchar_megaide_ioctl p_ioctl;
	HW_DEVICE_EXTENSION *pdev = NULL;
	FAST_IDE_SCB_EXTENSION *scb;
	adapter_t *adapter;

	if (cmd == MEGAIDE_DEV_IOCTL) {

		data_size = (inlen > outlen) ? inlen : outlen;
		adapter = gastCardInfo[adp_no-1];
		pdev = &adapter->DeviceExtension;
		if (pdev==NULL) {
		 AR_DEBUG(AR_DBG_NIT, ("megaide: dev is Null,adp=%d\n", adp_no));
		 return -EINVAL;
		}

		/*memory allocated at initialization*/
		scmd = (Scsi_Cmnd *) adapter->nit_scsi_cmd;
		if (scmd == NULL) {
		 AR_DEBUG (AR_DBG_NIT, ("megaide: scmd mem alloc Failed!!\n"));
		 return -ENOMEM;
		}
		memset_Rlsi(scmd, 0, sizeof(Scsi_Cmnd));

		scmd->host_scribble = adapter->host_scribble;
		scb = (FAST_IDE_SCB_EXTENSION *) scmd->host_scribble;
		if (scb == NULL) {
		 printk_Rlsi(KERN_WARNING "megaide: host malloc failed\n");
		 return -ENOMEM;
		}

		memset_Rlsi(scb, 0, get_scb_extension_size());

		p_ioctl = (pchar_megaide_ioctl) adapter->ioctl_data;
		if (p_ioctl == NULL) {
		 AR_DEBUG(AR_DBG_NIT, ("megaide: IOCTL Mem alloc Failed!!\n"));
		 return -ENOMEM;
		}
		memset_Rlsi(p_ioctl, 0, data_size);

		if (copy_from_user(p_ioctl,(char *)arg,
			(inlen + CHAR_HEADER_SIZE))) {
			AR_DEBUG(AR_DBG_NIT, ("megaide: Cpy Frm user Failed\n"));
			return -EFAULT;
		}

		scmd->request_buffer = &p_ioctl->wdata;
		scb->request_buffer = (VOID *)scmd->request_buffer;
		scb->pstScsiCmd = (VOID *)scmd;
		scmd->scsi_done = mide_char_ioctl_over;

		/*memory allocated by us, following 2 members are
		 *used for memory deallocation and sleep in queued commands
		 */
		scmd->host = (struct Scsi_Host *) pdev;
		
		scmd->state = MIDE_CHAR_IOCTL_CALLED;


		init_waitqueue_head(&adapter->wait_queue);
		spin_lock_irqsave(&io_request_lock, flags);
		switch (HandleIOCTLCall(pdev, scb))
		{
		  case IOCTL_OK:
		  {
		    AR_DEBUG(AR_DBG_NIT,("megaide:IOCTL_OK\n"));
		    megaide_complete_command(pdev, scb,(DID_OK << 16));
		    iRetVal = 0;
		    break;
		  }

		  case IOCTL_ERROR:
		  {
		    AR_DEBUG(AR_DBG_NIT,("megaide:IOCTL error\n"));
		    megaide_complete_command(pdev, scb,(DID_BAD_TARGET << 16));
		    break;
		  }

		  case IOCTL_QUEUE:
		  {
		    AR_DEBUG(AR_DBG_NIT,("megaide:IOCTL_QUEUE\n"));
		    
		    spin_unlock_irqrestore(&io_request_lock, flags);
		    flags=0;

		    for (i=0; i <(MIDE_CHAR_IOCTL_TIMEOUT*HZ); i++) {

		      interruptible_sleep_on_timeout(&adapter->wait_queue, 1);

		      if (scmd->state == MIDE_CHAR_IOCTL_FINISHED){
		        break;
		      }
		    }

		    spin_lock_irqsave(&io_request_lock, flags);

		    if (!scmd->state) {
		      AR_DEBUG(AR_DBG_NIT,("megaide:ioctlcmd Aborting\n"));
		      mide_abort_passthu_IOCTL_command(pdev, scmd);
		    } else {
		      iRetVal = 0;
		    }
		    break;
		  }

		  default:
		  {
		    megaide_complete_command(pdev, scb,(DID_BAD_TARGET << 16));
		    break;
		  }
		}/*switch*/

		if (!iRetVal) {
		  copy_to_user((char *)arg, p_ioctl, (outlen+CHAR_HEADER_SIZE));
		}

		spin_unlock_irqrestore(&io_request_lock, flags);
	}
	return iRetVal;
}/*mide_handle_ioctl()*/

UCHAR
mide_abort_passthu_IOCTL_command(HW_DEVICE_EXTENSION *pdev, Scsi_Cmnd *scmd)
{
	FAST_IDE_SCB_EXTENSION *scb = NULL;

	AR_DEBUG(AR_DBG_NIT,(" megaide: mide_abort_passthu_IOCTL_command\n"));
	scb = (FAST_IDE_SCB_EXTENSION *) scmd->host_scribble;
	megaide_abort_ioctl (pdev, scb);
	return 0;
}/*mide_abort_passthu_IOCTL_command()*/

/**
 * Description
 *  	This funtion returns the integer equvalent value of the array
 *
 * Arguments
 * 	pcArray-> Array holds the integer number
 *
 * Return Value
 * 	Signed integer equivalent value
 */

long
mide_array_to_int(const char * pcArray)
{
	int iCurrent; 		/* current char */
	long ulTotal; 	/* current total */
	int iSign; 	/* if '-', then negative, otherwise positive */

	/* skip whitespace */
	while (isspace((int)(unsigned char) *pcArray)) {
		++pcArray;
	}

	iCurrent = (int)(unsigned char) *pcArray++;
	iSign = iCurrent;
	if (iCurrent == '-' || iCurrent == '+') {
		/* skip sign */
		iCurrent = (int)(unsigned char)*pcArray++;
	}

	ulTotal = 0;

	while (isdigit(iCurrent)) {
		/*accumulate digit*/
		ulTotal = 10 * ulTotal +(iCurrent - '0');
		/*get next char*/
		iCurrent = (int)(unsigned char)*pcArray++;
	}

	if (iSign == '-') {
		return -ulTotal;
	} else {
		return ulTotal;
	}
}/*mide_array_to_int()*/

/*vi set ts=8:*/



