/*megaide_pci.c*/
/*
 * Linux MegaRAID IDEal Software RAID Stack
 * Copyright (c) 2001 LSI Logic Corporation, All Rights Reserved.
 *-----------------------------------------------------------------------------
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *-----------------------------------------------------------------------------
 * Authors:
 *      Manoj Jose       <Manoj.Jose@lsil.com>
 *      Atul Mukker      <Atul.Mukker@lsil.com>
 *
 *  This driver is supported by LSI Logic Corportation. This software is the
 *  Open Source Code of MegaRAID IDEal Software RAID Stack. Closed Part of the
 *  Source contains LSI's RAID intelligence and it will be provided as binary
 *  image
 *
 *  Support CSB6/CMD649/CMD680/SIL3112 Chipsets
 */

#include <linux/config.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/ctype.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/fcntl.h>
#include <linux/delay.h>
#include <linux/pci.h>
#include <linux/pci_ids.h>

#ifdef SUSE70_INSTALL
 #include <linux/ide.h>
#endif
#include <linux/ide.h>
#include <linux/proc_fs.h>
#include <linux/blk.h>
#include <linux/wait.h>
#include <linux/tqueue.h>
#include <linux/interrupt.h>
#include <linux/mm.h>
#include <asm/pgtable.h>
#include <linux/sched.h>
#include <linux/stat.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 7)
 #include <linux/malloc.h>
#else
 #include <linux/slab.h>
#endif
#include <linux/spinlock.h>

#include <asm/io.h>
#include <asm/irq.h>
#include <asm/dma.h>
#include <asm/uaccess.h>
#include <asm/system.h>

/*These header files are required for Shutdown Notification routines*/
#include <linux/notifier.h>
#include <linux/reboot.h>
#include <linux/init.h>
#include <linux/kernel.h>

#define SHUTDOWN_SIGS   (sigmask(SIGKILL)|sigmask(SIGINT)|sigmask(SIGTERM))
//#define ENABLE_ERROR_INJECTOR
//#define ENABLE_ERROR_VALIDATION
//#define BP_SUPPORT 0x01

#include "../sd.h"
#include "../scsi.h"
#include "../hosts.h"

#include    "./inc/megaide_datatype.h"
#include    "./inc/megaide_debug.h"
#include    "./inc/megaide_adapter.h"
#include    "./inc/megaide_ata.h"
#include    "./util/mp_hd_ioctls.h"
#include    "./util/bp_ioctl.h"
#include    "./inc/megaide_devext.h"
#include    "./inc/megaide_pci.h"
#include    "./inc/megaide_kernel_lib.h"
#include    "./inc/megaide_proc.h"
#include    "./inc/megaide_char.h"
#include    "./inc/megaide_scsi.h"
#include    "./inc/megaide_macros.h"
#ifndef MEGAIDE_LIBRARY_READY
 #include    "./lib/megaide_lib.c"
#endif

/*Global variables*/
adapter_t *gastCardInfo[MAX_CONTROLLERS] = {};

static int giNumberOfCtrlrs=0;

#ifdef BP_SUPPORT
 int 	debug_level = 0|AR_BP|AR_DEBUG_ERRORS/*|AR_DEBUG_IOCTL|AR_DEVEL_DEBUG|AR_DBG_NIT*/;
#else
 int 	debug_level = 0|AR_BP|AR_DEBUG_ERRORS|AR_HOT_PLUG|AR_DEBUG_WARNING;
#endif

#ifdef MODULE
 MODULE_PARM(debug_level, "i");
#endif

CARD_SPECIFIC_FUNCTION gaMideFunc[NUM_SUPPORTED_CONTROLLERS] =
{
 { Found64xInterrupt,  Disable64xInterrupts, Enable64xInterrupts,
   Is64x80PinCable,  ProgramDriveFor64xChip, IsPostCommandToDriveSuccess64x},

 { Found64xInterrupt,  Disable64xInterrupts, Enable64xInterrupts,
   Is64x80PinCable,  ProgramDriveFor64xChip, IsPostCommandToDriveSuccess64x},

 { Found68xInterrupt,  Disable68xInterrupts, Enable68xInterrupts,
   Is68x80PinCable,  ProgramDriveFor68xChip, IsPostCommandToDriveSuccess68x},

 { FoundCSB5Interrupt, DisableCSB5Interrupts, EnableCSB5Interrupts,
   IsCSB580PinCable, ProgramDriveForCSB5Chip, IsPostCommandToDriveSuccessCSB5},

 { Found68xInterrupt,  DisableAthenaInterrupts, EnableAthenaInterrupts,
   Is68x80PinCable,  ProgramDriveFor68xChip, IsPostCommandToDriveSuccess68x},

 { FoundCSB6Interrupt, DisableCSB6Interrupts, EnableCSB6Interrupts,
   IsCSB680PinCable, ProgramDriveForCSB6Chip, IsPostCommandToDriveSuccessCSB5},

 { FoundICH5RInterrupt, DisableICH5RInterrupts, EnableICH5RInterrupts,
   IsICH5R80PinCable, ProgramDriveForICH5RChip, IsPostCommandToDriveSuccessICH5R}
};


#ifndef  KMALLOC_MAXSIZE
 #define KMALLOC_MAXSIZE	131072UL /* Maximum allowed request size */
#endif

MODULE_AUTHOR("LSI Logic");
MODULE_DESCRIPTION("LSI Logic megaide driver");
MODULE_LICENSE("GPL");

/**
 * Description
 *	This function will Find the Megaide Controllers connected to the
 *	system and allocate device extention for each contollers, Initialize
 *  	all the device extention and global structures. This the function
 * 	executed at the time of sriver load
 * Return Value
 *	Returns Number of controllers found
 */
int
mide_find_card(Scsi_Host_Template * hostTpl,
	       unsigned short venID, unsigned short devId, long lFlag)
{
	UCHAR ucPciBus, ucPciDevFun, irq, i, j;
	unsigned short sPciIdx = 0, adpno = 0;
	unsigned short sSubsysId, sSubsysVId;
	u_int uiMegaBase;
	struct pci_dev *dev		= NULL;
	struct Scsi_Host *host	= NULL;
	HW_DEVICE_EXTENSION *pdev	= NULL;
	adapter_t *adapter = NULL;

	/*Search through PCI address space for MEGA IDE Controller*/
	while ((dev = pci_find_device(venID, devId, dev)))
	{
		ucPciBus = dev->bus->number;
		ucPciDevFun = dev->devfn;

		pci_read_config_word(dev, PCI_SUBSYSTEM_VENDOR_ID, &sSubsysVId);
		pci_read_config_word(dev, PCI_SUBSYSTEM_ID, & sSubsysId);

		if (FALSE == mide_check_for_supported_device(dev, devId,
		                        sSubsysId, sSubsysVId, lFlag)){
		  sPciIdx++;
		  continue;
		}

		AR_DEBUG(AR_DBG_INIT, (KERN_DEBUG "megaide: SSid %x SSVid %x\n",
		        sSubsysId, sSubsysVId));
		AR_DEBUG(AR_DBG_INIT, (KERN_DEBUG
		        "megaide: venID 0x%4.04x: devid 0x%4.04x: idx %d:bus %d",
		         venID, devId, sPciIdx, ucPciBus));
		AR_DEBUG(AR_DBG_INIT, (KERN_DEBUG ":slot %d:func %d\n",
		         PCI_SLOT(ucPciDevFun), PCI_FUNC(ucPciDevFun)));

		//This code moved from megaide_detect
		//Need to call only when our device detected.
		if (!adpno) {
			spin_unlock_irq (&io_request_lock);
			RemoveBuildInIDESupport(venID, devId, sSubsysId, sSubsysVId);
			spin_lock_irq (&io_request_lock);
		}

		/*Go to PCI resource start*/
		uiMegaBase  = pci_resource_start(dev, 0);
		irq   = dev->irq;
		sPciIdx++;

		dev->dma_mask = 0xFFFFFFFF;

		/* Initialize SCSI Host structure */
		host = scsi_register(hostTpl, sizeof(adapter_t));
		if (!host) {
			printk_Rlsi(KERN_WARNING "megaide: scsi_register failed");
			goto go_err_exit;
		}

		//to make sure the SCSI Mid Layer IOCTL Works
		scsi_set_pci_device(host, dev);

#if LINUX_VERSION_CODE == KERNEL_VERSION(2, 4, 9)
		host->can_dma_32 = 0;
		host->single_sg_ok = 1;
#endif
		adapter = (adapter_t *) host->hostdata;
		memset_Rlsi(adapter, 0, sizeof(adapter_t));

		pdev = (HW_DEVICE_EXTENSION *)&adapter->DeviceExtension;
		pdev->adapter = (void *) adapter;
		gastCardInfo[adpno] = adapter;

#ifdef PERFORMANCE_NUMBER
		pdev->int_count[0] = 0;
		pdev->int_count[1] = 0;
		pdev->io_count = 0;
#endif
		/*initialize all arrays/drives as being not-present*/
		for (j = 0; j < MAX_LOGICAL_DRIVES; j++)
		{
			pdev->LogicalDrive[j].ucControllerId = adpno;
		}

		for (i = 0; i < MAX_DRIVES_PER_CONTROLLER; i++)
		{
			pdev->PhysicalDrive[i].ucControllerId = adpno;
		}

		if (mide_init_cmd_memory_pool (pdev)==FALSE){
			goto go_scsi_unregister;
		}

		/*allocating memory for base dev extension :
		 * pdev->GateKeeperBuffer
		 * pdev->GateKeeper
		 * pdev->GateScbExtension
		 */
		//memory alloc failure is handled properly
		if (FALSE == mide_allocate_device_ext_mem (pdev, dev, adpno)) {
			goto go_release_pool_mem;
		}

		adapter->qSpinLock = SPIN_LOCK_UNLOCKED;
		
		/*Semaphore for IOCTL*/
		init_MUTEX (&adapter->nit_lock);

		AR_DEBUG(AR_DBG_INIT, (KERN_DEBUG
		        "scsi%d : Found MegaIDE ctrlr at 0x%x, IRQ: %d\n",
		        host->host_no, (u_int) uiMegaBase, irq));

		/*Need to initialize Device Extension resource structure*/
		adapter->pstScsiHost = host;
		host->irq = irq;
		host->io_port = uiMegaBase;
		host->n_io_port = 16;
		host->unique_id = (ucPciBus<<8) | ucPciDevFun;
		host->max_channel= M_IDE_MAX_CHANNELS;
		host->max_id = M_IDE_MAX_LOGICAL_DRIVES;
		host->max_lun = M_IDE_MAX_LOGICAL_DRIVES;
		host->cmd_per_lun= M_IDE_MAX_CMD_PER_LUN;
		host->max_sectors = MAX_SECTORS_PER_IDE_TRANSFER;
		/*Store card Information*/
		pdev->ucPCIBus = ucPciBus;
		pdev->ucPCIDev = ucPciDevFun >> 3;
		pdev->ucPCIFun = ucPciDevFun & 7;
		pdev->usSlot = PCI_SLOT(ucPciDevFun);
		pdev->ulUniqueId = (ucPciBus << 8) | ucPciDevFun;
		pdev->ulProdId = devId;
		pdev->ucControllerId = adpno;
		pdev->ulVendorId = venID;
		pdev->ulDeviceId = devId;
		pdev->usSubsysVendorId = sSubsysVId;
		pdev->usSubsysDeviceId = sSubsysId;
		adapter->pstDev = dev;

		/*set default values for our IOCTL options*/
		pdev->Options.ulRebuildRate = 10;
		pdev->Options.ulIdentical=TRUE;
		pdev->Options.ulAutoRebuild=FALSE;
		pdev->Options.ulCheckArray = 0;
		pdev->Options.ulStopRebuild = 0;
		pdev->Options.ulErrorThreshold = 0;
		pdev->Options.ulFailDrive = 0;

		init_waitqueue_head(&adapter->internal_wait);

		/*Init controller type and controller speed*/
		mide_init_cardtype_and_speed (pdev, devId);

		/*controller's base address in device ext*/
		if (FALSE == mide_init_base_address (pdev, dev)) {
			goto go_cleanup_mem;
		}

		if (AR_TRUE!=ClaimOurIOPorts(pdev)) {
			printk_Rlsi(KERN_ERR "megaide: err Claiming ioports\n");
			goto go_cleanup_mem;
		}

		DisableInterrupts(pdev);

		if ((devId == PCI_DEVICE_ID_SERVERWORKS_CSB5IDE) ||
		    (devId == PCI_DEVICE_ID_SERVERWORKS_CSB6IDE)) {

			/* To Fix CSB6 Problem, native IDE Driver sets IRQ to 0
			 * We are reading directly from Configuration space
			 */
			if (devId == PCI_DEVICE_ID_SERVERWORKS_CSB6IDE) {
			  UCHAR swks_irq = 0;
			  pci_read_config_byte(dev, 0x3C, &swks_irq);
			  if (swks_irq) {
			    irq = swks_irq;
			  }
			}

			adapter->pstScsiHost->irq = irq;
			dev->irq = irq;
		}

		if (request_irq(irq, megaide_isr, SA_SHIRQ, "megaide", adapter)) {
			printk_Rlsi(KERN_ERR
			  "megaide: Couldn't register IRQ %d\n", irq);
			goto go_release_ioports;
		}

		pdev->ulIntLine	= irq;

		mide_get_option_rom_info(pdev);

		if (AR_ERROR == HyperDiskScanForDevices(pdev)) {
			printk_Rlsi(KERN_WARNING "megaide: Scan Devices failed\n");
			goto go_err_release;
		}

		EnableInterrupts(pdev);

		if (!pdev->ucIRCDType) {
			/*
			 * This is the case when No device detected with proper
			 * IRCD Setting to the default IRCD
			 */
			pdev->ucIRCDType = IRCD_3X;
		}
		/* start polling thread as soon as the drive loads */
		if (pdev->ucCardType == INTEL_ICH5R) {
			mega_start_ICH5R_poll_thread(pdev);
		}

		/* Adjust SGL's buffer pointer for the channels */
		AssignSglPtrsForChannels(pdev);

#ifdef BP_SUPPORT
		/*backplane discovery*/
		idebp_discovery(pdev);
#endif
		AR_DEBUG(AR_DBG_INIT,(KERN_DEBUG "megaide: scsi_register ok\n"));
		/*Number of controllers found*/
		adpno++;
		/*Dont execute the go to*/
		continue;

		//Error Handling
		go_err_release:
			free_irq(host->irq, pdev);

		go_release_ioports:
			ReleaseOurIOPorts(pdev);

		go_cleanup_mem:
			CleanUpAdapter(pdev);

		go_release_pool_mem:
			mide_free_cmd_memory_pool(pdev);

		go_scsi_unregister:
			scsi_unregister (host);

		go_err_exit:
			printk_Rlsi(KERN_WARNING "megaide: leaving device !!!\n");

	}/*while () */

	return adpno;

}/*megaide find card() */

/**
 * check the device actually supported by thid driver,
 * return TRUE if device is supported else FALSE
 */
BOOLEAN
mide_check_for_supported_device(struct pci_dev *dev, unsigned short devId,
		unsigned short sSubsysId, unsigned short sSubsysVId, long lFlag)
{
	u_char class_dev;
	char ret_val = TRUE;
	pci_read_config_byte(dev, PCI_CLASS_DEVICE, &class_dev);

	/*for Athena support*/
	/*checking to make sure its class code is RAID*/
	if ((class_dev != 4) && (PCI_DEVICE_ID_CMD_3112!=devId)){
		return (FALSE);
	}

	if (!lFlag) return (ret_val);
		
	switch (devId)
	{
	  case PCI_DEVICE_ID_DELL_680:
		if (sSubsysVId == PCI_VENDOR_ID_DELL) {
			if (sSubsysId == PCI_DEVICE_ID_DELL_CERC_ATA_AS) {
				break;
			}
		}
		ret_val = FALSE;
		break;

	  case PCI_DEVICE_ID_CMD_680:
		if (sSubsysVId == PCI_VENDOR_ID_LSI_LOGIC) {
			if (sSubsysId == PCI_DEVICE_ID_LSI_524) {
				break;
			}
		}
		ret_val = FALSE;
		break;

		//to support Compaq CSB6 chipset
	  case PCI_DEVICE_ID_SERVERWORKS_CSB6IDE:
		if (sSubsysVId == PCI_VENDOR_ID_COMPAQ) {
			if (sSubsysId == PCI_DEVICE_ID_COMPAQ_CSB6) {
				break;
			}
		}

		//for IBM CSB6 support
		if (sSubsysVId == PCI_VENDOR_ID_IBM) {
			if (sSubsysId == PCI_DEVICE_ID_IBM_CSB6) {
				break;
			}
		}

		//accer csb6 support 06252003
		if (sSubsysVId == PCI_VENDOR_ID_LSI_LOGIC) {
			if (sSubsysId == PCI_DEVICE_ID_SERVERWORKS_CSB6IDE) {
				break;
			}
		}
		ret_val = FALSE;
		break;
		
	  case PCI_DEVICE_ID_CMD_3112:
		if (sSubsysVId == PCI_VENDOR_ID_LSI_LOGIC) {
			if (sSubsysId == 0x534) { /*MegaRAID SATA 150-2*/
				break;
			}
		}
		ret_val = FALSE;
		break;

	  case  PCI_DEVICE_ID_INTEL_ICH5R:
		if (sSubsysVId == PCI_VENDOR_ID_INTEL) {
			if ((sSubsysId == 0x342A) || /*winter parrk*/
				(sSubsysId == 0x3429) || /*winter parrk*/         
				(sSubsysId == 0x3432) || /*Alief*/
				(sSubsysId == 0x3434) || /*Alief*/
				(sSubsysId == 0x3436) || /*Jarrell*/
				(sSubsysId == 0x3438)) { /*Jarrell*/
				break;
			}
		}
		ret_val = FALSE;
		break;

	  case  PCI_DEVICE_ID_INTEL_HR:
		if (sSubsysVId == PCI_VENDOR_ID_INTEL) {
			if ((sSubsysId == 0x343A) || /*Brandon*/
				(sSubsysId == 0x343C) || /*Brandon*/
				(sSubsysId == 0x343E) || /*Brandon*/
				(sSubsysId == 0x3430) || /*Torrey Pines*/
				(sSubsysId == 0x3441) || /*Sun Prairie*/
				(sSubsysId == 0x3443) || /*Sun Prairie*/
				(sSubsysId == 0x3445)) { /*Grand Prairie*/
				break;
			}
		}
		if (sSubsysVId == 0x1734){
			if (sSubsysId == 0x1034) {
				/*LSI Logic Embedded SATA RAID, Hace Rapid*/
				break;
			}
		}
		ret_val = FALSE;
		break;

	  default:
		ret_val = FALSE;
		break;
	}/*switch*/
	return (ret_val);
}

/**
 * This function will allocate device memories required in device extension
 * memory allocation failure is handled properly
 */
BOOLEAN
mide_allocate_device_ext_mem (HW_DEVICE_EXTENSION *pdev,
                              struct pci_dev *dev, unsigned short adpno)
{
	ULONG length = (MAX_SECTORS_PER_IDE_TRANSFER*IDE_SECTOR_SIZE*
	                MAX_CHANNELS_PER_CONTROLLER);
	ULONG *buffer = NULL;
	ULONG reduceToAlign;
	ULONG chan, phy;
	ULONG log;
	adapter_t *adapter = (adapter_t *)pdev->adapter;

	/*allocating memory GateKeeper, GateScbExtension*/
	pdev->GateKeeper = kmalloc(get_gatekeeper_size(), GFP_KERNEL);
	if (NULL == pdev->GateKeeper) {
		printk_Rlsi(KERN_WARNING "megaide: Gate m alloc failed\n");
		goto go_err_free_mem;
	}

	pdev->GateScbExtension = (FAST_IDE_SCB_EXTENSION *)
		kmalloc(get_scb_extension_size(), GFP_KERNEL);

	if (NULL == pdev->GateScbExtension) {
		printk_Rlsi(KERN_WARNING "megaide: GateScbExt m alloc failed\n");
		goto go_err_free_mem;
	}

	length = (length > KMALLOC_MAXSIZE) ? KMALLOC_MAXSIZE:length;

	while (buffer == NULL)
	{
		buffer =
		    pci_alloc_consistent(dev, length,
		                         &adapter->h_gate_keeper_buff);

		if (buffer == NULL) {
			AR_DEBUG(AR_DEBUG_WARNING,
			        (KERN_DEBUG "megaide: gate mem alloc failed\n"));
		}

		adapter->gate_keeper_buff_length = length;

		if (length <=  IDE_SECTOR_SIZE) {
		  BUG();
		}

		if (buffer == NULL) {
		  length = length/2;
		}

	}


	if (buffer && length > IDE_SECTOR_SIZE) {
		memset_Rlsi(buffer, 0, length);

		pdev->GateKeeperBuffer = (UCHAR *) buffer;
		pdev->GateKeeperPhysicalBuffer = (ULONG)
		    adapter->h_gate_keeper_buff;
		pdev->GateKeeperBufferLength = length;

		reduceToAlign = (REGION_HW_BOUNDARY -
		    pdev->GateKeeperPhysicalBuffer%REGION_HW_BOUNDARY);
		pdev->GateKeeperBuffer += reduceToAlign;
		pdev->GateKeeperPhysicalBuffer += reduceToAlign;
		pdev->GateKeeperBufferLength -= reduceToAlign;

		if (pdev->GateKeeperBufferLength <= 2*IDE_SECTOR_SIZE) {
			BUG();
		}

		if (pdev->GateKeeperBufferLength <= 2*IDE_SECTOR_SIZE) {
		  pdev->GateKeeperPhysicalBuffer =
		  pdev->GateKeeperBufferLength = 0;
		  pdev->GateKeeperBuffer = NULL;
		}
	} else {
		printk_Rlsi(KERN_WARNING "megaide: gate keep mem alloc failed\n");
		goto go_err_free_mem;
	}

	/*allocating memory for channel :
	* pdev->Channel[].DiskConfiguration
	* pdev->Channel[].DiskEnquiry
	*/

	for (chan = 0; chan<MAX_CHANNELS_PER_CONTROLLER; chan++)
	{

	  pdev->Channel[chan].DiskConfiguration =
	      pci_alloc_consistent(dev, IDE_SECTOR_SIZE,
	                           &adapter->Channel[chan].h_disc_conf);

	  if (pdev->Channel[chan].DiskConfiguration == NULL) {
	    AR_DEBUG(AR_DEBUG_WARNING,
	            (KERN_DEBUG "megaide: mem alloc failed for DiskConf!\n"));
	    goto go_err_free_mem;
	  }

	  pdev->Channel[chan].DiskEnquiry =
	      pci_alloc_consistent(dev, IDE_SECTOR_SIZE,
	                           &adapter->Channel[chan].h_enquiry_conf);

	  if (pdev->Channel[chan].DiskEnquiry == NULL) {
	    AR_DEBUG(AR_DEBUG_WARNING,
	            (KERN_DEBUG "megaide: mem alloc failed for EnqConf!\n"));
	    goto go_err_free_mem;
	  }

	  memset_Rlsi(pdev->Channel[chan].DiskConfiguration, 0,
	              IDE_SECTOR_SIZE);
	  memset_Rlsi(pdev->Channel[chan].DiskEnquiry, 0,
	              IDE_SECTOR_SIZE);

	  pdev->Channel[chan].DiskConfigurationSgl.Physical.Address =
	        (void *) (ULONG) adapter->Channel[chan].h_disc_conf;
	  pdev->Channel[chan].DiskConfigurationSgl.Physical.Length =
	        IDE_SECTOR_SIZE;
	  pdev->Channel[chan].DiskConfigurationSgl.Physical.EndOfListFlag = 1;
	  pdev->Channel[chan].DiskEnquirySgl.Physical.Address =
	        (void *) (ULONG) adapter->Channel[chan].h_enquiry_conf;
	  pdev->Channel[chan].DiskEnquirySgl.Physical.Length = IDE_SECTOR_SIZE;
	  pdev->Channel[chan].DiskEnquirySgl.Physical.EndOfListFlag = 1;
	}

	/*allocating memory for logical drives :
	* pdev->Channel[].InternalScbExtension
	* pdev->Channel[].GateKeeper
	*/

	for (log = 0; log<MAX_LOGICAL_DRIVES; log++)
	{
		pdev->LogicalDrive[log].InternalScbExtension =
		    kmalloc(get_scb_extension_size(), GFP_KERNEL);

		if (NULL == pdev->LogicalDrive[log].InternalScbExtension) {
		  printk_Rlsi(KERN_WARNING "megaide: ScbExt1 mem alloc failed\n");
		  goto go_err_free_mem;
		}

		pdev->LogicalDrive[log].GateKeeper = (FAST_IDE_SCB_EXTENSION *)
		    kmalloc(get_gatekeeper_size(), GFP_KERNEL);

		if (NULL == pdev->LogicalDrive[log].GateKeeper) {
		  printk_Rlsi(KERN_WARNING "megaide: GateKeep mem alloc failed\n");
		  goto go_err_free_mem;
		}
	}

	//allocating memory for physical drive :ErrorLogSector
	for (phy = 0; phy < MAX_DRIVES_PER_CONTROLLER; phy++)
	{
	  UCHAR *errorLog = pci_alloc_consistent(dev, IDE_SECTOR_SIZE,
	                         &adapter->PhysicalDrive[phy].h_err_log_sec);
	  if (errorLog == NULL) {
	    printk_Rlsi(KERN_WARNING "megaide: errorLog mem alloc failed\n");
	    goto go_err_free_mem;
	  }

	  memset_Rlsi(errorLog, 0, IDE_SECTOR_SIZE);
	  pdev->PhysicalDrive[phy].ErrorLogSector = errorLog;
	  pdev->PhysicalDrive[phy].DiskErrorLogSgl.Physical.Address =
	         (void *) (ULONG)adapter->PhysicalDrive[phy].h_err_log_sec;
	  pdev->PhysicalDrive[phy].DiskErrorLogSgl.Physical.Length =
	         IDE_SECTOR_SIZE;
	  pdev->PhysicalDrive[phy].DiskErrorLogSgl.Physical.EndOfListFlag = 1;
	}

	/*IOCTL memory allocation*/
	adapter->ioctl_data = NULL;
	adapter->nit_scsi_cmd = NULL;
	adapter->host_scribble = NULL;
	adapter->nit_scsi_cmd =
	     pci_alloc_consistent(dev, sizeof(Scsi_Cmnd),
	                          &adapter->h_nit_scsi_cmd);
	adapter->host_scribble =
	     pci_alloc_consistent(dev, get_scb_extension_size(),
	                          &adapter->h_host_scribble);
	adapter->ioctl_data =
	     pci_alloc_consistent(dev,
	                          (NIT_MAX_DATA_TRANSFER + CHAR_HEADER_SIZE),
	                          &adapter->h_ioctl_data);
	if ((adapter->ioctl_data == NULL) ||
	    (adapter->nit_scsi_cmd == NULL) ||
	    (adapter->host_scribble == NULL)) {
	  printk_Rlsi(KERN_WARNING "megaide: IOCTL mem alloc failed\n");
	  goto go_err_free_mem;
	}

	return TRUE;
	/*ERROR Region*/
	go_err_free_mem:
	{
		if (pdev->GateKeeper) {
			kfree(pdev->GateKeeper);
		}

		if (pdev->GateScbExtension) {
			kfree (pdev->GateScbExtension);
		}

		if (buffer){
		  pci_free_consistent(dev, adapter->gate_keeper_buff_length,
		                      buffer, adapter->h_gate_keeper_buff);
		}

		for (chan = 0; chan<MAX_CHANNELS_PER_CONTROLLER; chan++)
		{
		  if (pdev->Channel[chan].DiskConfiguration) {
		    pci_free_consistent(adapter->pstDev, IDE_SECTOR_SIZE,
		                        pdev->Channel[chan].DiskConfiguration,
		                        adapter->Channel[chan].h_disc_conf);
		  }

		  if (pdev->Channel[chan].DiskEnquiry) {
		    pci_free_consistent(adapter->pstDev, IDE_SECTOR_SIZE,
		                        pdev->Channel[chan].DiskEnquiry,
		                        adapter->Channel[chan].h_enquiry_conf);
		  }
		}

		for (log = 0; log<MAX_LOGICAL_DRIVES; log++)
		{
		  if (pdev->LogicalDrive[log].InternalScbExtension) {
		    kfree(pdev->LogicalDrive[log].InternalScbExtension);
		  }

		  if (pdev->LogicalDrive[log].GateKeeper) {
		    kfree(pdev->LogicalDrive[log].GateKeeper);
		  }
		}

		for (phy = 0; phy < MAX_DRIVES_PER_CONTROLLER; phy++)
		{
		  if (pdev->PhysicalDrive[phy].ErrorLogSector) {
		    pci_free_consistent(adapter->pstDev, IDE_SECTOR_SIZE,
		                    pdev->PhysicalDrive[phy].ErrorLogSector,
		                    adapter->PhysicalDrive[phy].h_err_log_sec);
		  }
		}

		if (adapter->nit_scsi_cmd) {
		  pci_free_consistent(adapter->pstDev, sizeof(Scsi_Cmnd),
		                      adapter->nit_scsi_cmd,
		                      adapter->h_nit_scsi_cmd);
		}

		if (adapter->host_scribble) {
		  pci_free_consistent(adapter->pstDev, get_scb_extension_size(),
		                      adapter->host_scribble,
		                      adapter->h_host_scribble);
		}

		if (adapter->ioctl_data) {
		  pci_free_consistent(adapter->pstDev,
		                      (NIT_MAX_DATA_TRANSFER+CHAR_HEADER_SIZE),
		                      adapter->ioctl_data,
		                      adapter->h_ioctl_data);
		}
	}//go_err_free_mem
	return FALSE;
}/*mide_allocate_device_ext_mem*/

/*Initialize controller type and controller speed*/
void
mide_init_cardtype_and_speed (HW_DEVICE_EXTENSION *pdev, unsigned short deviceID)
{
	switch (deviceID)
	{
		case PCI_DEVICE_ID_CMD_648:
			pdev->ControllerSpeed = Udma66;
			pdev->ucCardType = CMD_648;
			break;

		case PCI_DEVICE_ID_CMD_649:
			pdev->ControllerSpeed = Udma100;
			pdev->ucCardType = CMD_649;
			break;

		case PCI_DEVICE_ID_CMD_680:
		case PCI_DEVICE_ID_DELL_680:
			pdev->ControllerSpeed = Udma133;
			pdev->ucCardType = CMD_680;
			break;

		case PCI_DEVICE_ID_CMD_3112:
			pdev->ControllerSpeed = Udma133;
			pdev->ucCardType = SIL_3112;
			break;

		case PCI_DEVICE_ID_SERVERWORKS_CSB5IDE:
			pdev->ControllerSpeed = Udma100;
			pdev->ucCardType = RCC_SERVER_WORKS_CSB5;
			break;

		case PCI_DEVICE_ID_SERVERWORKS_CSB6IDE:
			pdev->ControllerSpeed = Udma100;
			pdev->ucCardType = RCC_SERVER_WORKS_CSB6;
			break;

		case PCI_DEVICE_ID_INTEL_ICH5R:
		case PCI_DEVICE_ID_INTEL_HR:
			pdev->ControllerSpeed = Udma100;
			pdev->ucCardType = INTEL_ICH5R;
			break;
		
		default:
		printk_Rlsi("Unknown device!!!! deviceID %x\n", deviceID);
		break;
	}
}

/*store controller's base address in device extension*/
BOOLEAN
mide_init_base_address (HW_DEVICE_EXTENSION *pdev, struct pci_dev *dev)
{
	pdev->Channel[0].BaseIoAddress1 =
	    (IDE_REGISTERS_1 *) pci_resource_start(dev, 0);
	pdev->Channel[0].BaseIoAddress2 =
	    (IDE_REGISTERS_2 *) pci_resource_start(dev, 1);
	pdev->Channel[1].BaseIoAddress1 =
	    (IDE_REGISTERS_1 *) pci_resource_start(dev, 2);
	pdev->Channel[1].BaseIoAddress2 =
	    (IDE_REGISTERS_2 *) pci_resource_start(dev, 3);
	pdev->Channel[0].BaseBmAddress =
	    (BM_REGISTERS *) pci_resource_start(dev, 4);
	pdev->Channel[1].BaseBmAddress =
	    (BM_REGISTERS *) (pci_resource_start(dev, 4) + 8);

	if ((GET_DEVICE_ID(pdev) == PCI_DEVICE_ID_CMD_3112) ||
	    (GET_DEVICE_ID(pdev) == PCI_DEVICE_ID_CMD_680) ||
	    (GET_DEVICE_ID(pdev) == PCI_DEVICE_ID_DELL_680)) {

	  pdev->baseAddress5 = pci_resource_start(dev, 5);
	  pdev->virtBaseAddress5 = NULL;

	  if (GET_DEVICE_ID(pdev) == PCI_DEVICE_ID_CMD_3112) {
	    pdev->virtBaseAddress5 =
	        ioremap(pdev->baseAddress5, SIL_3112_BASE_REG_5_DEPTH);
	  }

	  if ((GET_DEVICE_ID(pdev) == PCI_DEVICE_ID_CMD_680) ||
	      (GET_DEVICE_ID(pdev) == PCI_DEVICE_ID_DELL_680)) {
	    pdev->virtBaseAddress5 =
	        ioremap(pdev->baseAddress5, SIL_680_BASE_REG_5_DEPTH);
	  }

	  if (NULL == pdev->virtBaseAddress5) {
	    printk_Rlsi(KERN_WARNING "megaide: virtBaseAddress5 iomap failed\n");
	    return FALSE;
	  }
	}
	return TRUE;
}

void
initialize_device_extension(HW_DEVICE_EXTENSION *pdev)
{
	u_char i, j;

	for (j = 0; j < MAX_LOGICAL_DRIVES; j++)
	{
	  pdev->LogicalDrive[j].Status = LDS_NotPresent;
	  for (i = 0; i < MAX_DRIVES_PER_CONTROLLER; i++)
	  {
	    pdev->LogicalDrive[j].PhysicalDriveTid[i] = INVALID_DRIVE_ID;
	    pdev->LogicalDrive[j].SlotPhysicalDriveTid[i] = INVALID_DRIVE_ID;
	  }
	}

	for (i = 0; i < MAX_DRIVES_PER_CONTROLLER; i++)
	{
	  if (pdev->ucIRCDType == IRCD_3X) {
	    pdev->PhysicalDrive[i].Status = PDS_Undefined;
	  } else {
	    pdev->PhysicalDrive[i].Status = PDS_OffLine;
	  }

	  pdev->PhysicalDrive[i].ucMirrorDriveId = INVALID_DRIVE_ID;
	  pdev->PhysicalDrive[i].LogicalDriveId = INVALID_DRIVE_ID;
	  pdev->PhysicalDrive[i].pArrayInfo = NULL;
	  /*to correct proc display*/
	  pdev->PhysicalDrive[i].PhysicalDriveId = i;
	  ENABLE_IRCD_UPDATE_ON_DRIVE(pdev,i);
	}

	for (i = 0; i < MAX_DRIVES_PER_CONTROLLER; i++)
	{
	  //changed this for supporting hetrogenious array
	  pdev->ArrayDrive[i].ucArrayDriveNumber = 0;
	  pdev->ArrayDrive[i].ucLogicalDriveCount = 0;
	  pdev->ArrayDrive[i].HeadLogDrive = NULL;
	  pdev->ArrayDrive[i].TailLogDrive =NULL;
	}

}/*initialize device extension*/

/* function to get information from the option rom*/
int
mide_get_option_rom_info(HW_DEVICE_EXTENSION *pdev)
{
	ULONG	ulAddr;
	UCHAR *ulVirAddr;
	int iRetVal=AR_FALSE;

	AR_DEBUG(AR_DBG_ROM,
	         (KERN_DEBUG "megaide: mide_get_option_rom_info:\n"));

	for (ulAddr = 0xC0000, iRetVal = 0;
	     (ulAddr < 0xF0000) && (iRetVal == 0); ulAddr += (PAGE_SIZE*8))
	{

	  ulVirAddr = (UCHAR *) ioremap(ulAddr, (PAGE_SIZE*8));
	  if (!ulVirAddr) {
	    break;
	  }

	  if (AR_TRUE == mide_scan_option_rom(ulVirAddr, (PAGE_SIZE*8), pdev)) {
#ifdef DEBUG_ON
	    AR_DEBUG(AR_DBG_ROM,
	        (KERN_DEBUG "megaide: CMD64X: ROM enabled at 0x%08lx\n", ulAddr));
#endif
	    iRetVal=AR_TRUE;
	  }
	  iounmap(ulVirAddr);
	}

#ifdef DEBUG_ON
	if (AR_FALSE == iRetVal) {
	  AR_DEBUG(AR_DBG_ROM, (KERN_DEBUG "megaide: ROM not enabled\n"));
	}
#endif

	return(iRetVal);
}/*mide_get_option_rom_info() */

/*This function gets information from the option rom*/
int 
mide_scan_option_rom(unsigned char * pcRom,
                     unsigned long ulRomSize, HW_DEVICE_EXTENSION *pDE)
{
  char acTempStr[80];
  u_long i, j, k;
  u_long * pulTempPtr;
  u_short * pusROMSig;
  ushort bFoundVersion = 0;
  ulong scan_depth;

  AR_DEBUG(AR_DBG_ROM, (KERN_DEBUG "megaide: mide_scan_option_rom:\n"));

  /*look every 2k*/
  for (i = 0; i<ulRomSize; i += 0x100)
  {
    pusROMSig = (u_short *) &pcRom[i];
    if (*pusROMSig != 0xAA55) {
      continue;
    }

    /* look for our signature at offset 6 bytes */
    memmove(acTempStr, &pcRom[i+6], 8);
    acTempStr[8] = (char) 0;

    if ((!strcmp(acTempStr, "HyperDis")) ||
        (!strcmp(acTempStr, "MegaRAID"))) {

      /*get firmware revision*/
      scan_depth = 
          ((pcRom[i+2]*512) >  ulRomSize) ? ulRomSize : (pcRom[i+2]*512);

      for (j=14; j < scan_depth ; j++) {

        pulTempPtr = (u_long *) &pcRom[i+j];

        /*looking for 'Hype'
         *looking for 'Mega'
         *looking for 'IDE '(this is for Compaq Bios)
         *looking for ''CERC(this is for Dell Bios)
         * looking for $XIDE$
         */
        if ((*pulTempPtr == 0x65707948) || (*pulTempPtr == 0x6167654D) ||
            (*pulTempPtr == 0x20454449) || (*pulTempPtr == 0x43524543) ||
            (*pulTempPtr == 0x44495824)) {

          memmove(acTempStr, &pcRom[i + j], 14);
          acTempStr[14] = (char) 0;

          /*heres $XIDE$*/
          if (!strncmp(acTempStr, "$XIDE$",6)) {
            j += 6;
            pDE->stFirmwareVersion.usMajorVer=pcRom[i + j] - '0';
            j ++;
            pDE->stFirmwareVersion.usMinorVer=pcRom[i + j] - '0';
            j ++;
            pDE->stFirmwareVersion.ulBuild = mide_array_to_int(&pcRom[i + j]);

            /* yes, we want this */
            return(AR_TRUE);
          }

          /* the strcmp string should be 14 bytes long.
           * the 'j' counter should increment pointer to the location of
	   * 'V' begining at Version
           *look at offset 26 for firmware version(12 bytes long),
           */
          else if (!strcmp(acTempStr, "MegaRAID IDE B")) {
            j += 26;
            bFoundVersion = 1;
          }

          /* heres old hypderdisk string*/
          /*look at offset 23 for firmware version(12 bytes long) */
          else if (!strcmp(acTempStr, "HyperDisk BIOS")) {
            j += 23;
            bFoundVersion = 1;
          }

          /* heres CPQ bios string*/
          /*look at offset 22 for firmware version(12 bytes long) */
          else if (!strcmp(acTempStr, "IDE RAID BIOS ")) {
            j += 22;
            bFoundVersion = 1;
          }

          /*heres DELL bios string*/
          /*look at offset 22 for firmware version(12 bytes long) */
          else if (!strcmp(acTempStr, "CERC ATA/2S BI")) {
            j += 25;
            bFoundVersion = 1;
          }

          if (bFoundVersion) {

            /*get usMajorVer*/
            k = 0;
            /* look for '.'*/
            while (pcRom[i + j + k] != '.') k++;
            pDE->stFirmwareVersion.usMajorVer=mide_array_to_int(&pcRom[i + j]);

            /*get usMinorVer*/
            j += (k + 1);
            k = 0;
            /* look for '.'*/
            while (pcRom[i + j + k] != '.') k++;
            pDE->stFirmwareVersion.usMinorVer=mide_array_to_int(&pcRom[i + j]);

            /*get ulBuild*/
            j += (k+1);
            pDE->stFirmwareVersion.ulBuild = mide_array_to_int(&pcRom[i + j]);

            /* yes, we want this */
            return(AR_TRUE);
          }/*if (bFoundVersion) */
        }/*if ((*pulTempPtr == 0x65707948) */
      }/*for (j=14; */

      return(AR_TRUE);   /* yes, we want this */
    }/*if ((!strcmp(acTempStr*/
  }/*for (i = 0; i*/
  return(AR_FALSE);
}/*scan option rom() */

/* scan for all the devices attached to the IDE Raid controller and initialize
 * the Device Extetion structure
 */
LONG HyperDiskScanForDevices(HW_DEVICE_EXTENSION *pdev)
{
	LONG lRetVal = AR_SUCCESSS;
	
	if (!InitIdeRaidController(pdev)) {
	  lRetVal = AR_ERROR;
	  AR_DEBUG(AR_DEBUG_WARNING,
	           (KERN_WARNING "megaide: Init IDE Raid Controllers failed\n"));
	}
	
	/*EnableInterrupt is called in this function*/
	AtapiHwInitialize(pdev);

	return lRetVal;
}/*Hyper DiskScanForDevices() */



/* This function is to gather information about the adapter's configuration.
 * In particular, it determines whether a RAID is present.
 */
BOOLEAN 
InitIdeRaidController(HW_DEVICE_EXTENSION *pdev)
{
	BOOLEAN		foundDevices;
	UCHAR		channel;
	UCHAR		log;
	UCHAR		phy;
	char		raid_type[20];
	char		raid_status[20];

	foundDevices = AR_FALSE;

	for (channel = 0; channel < MAX_CHANNELS_PER_CONTROLLER; channel++)
	{
	  /* Search for devices on this controller.*/
	  if (FindDevices(pdev, channel)) {

	    AR_DEBUG(AR_DBG_INIT,
	             (KERN_DEBUG "megaide: Found device in Channel%d\n", channel));

	    /*Remember that some devices were found.*/
	    foundDevices = AR_TRUE;
	    phy = channel << 1;
	    if ((pdev->PhysicalDrive[phy].DeviceFlags&DFLAGS_DEVICE_PRESENT)&&
	        (pdev->PhysicalDrive[phy+1].DeviceFlags&DFLAGS_DEVICE_PRESENT)){

	      /*
	       * If 2 drives are present, set SwitchDrive to 1(default is 0) .
	       * SwitchDrive used to toggle bewteen the two drives'work queues,
	       * to keep the drives fed in a fair fashion. See StartChannelIo().
	       */
	      pdev->Channel[channel].SwitchDrive = 1;
	    }
	  }/*if (find_device*/
	}/*for () */

	if (foundDevices) {
		/*Program I/O mode for each drive.*/
		for (phy = 0; phy < MAX_DRIVES_PER_CONTROLLER; phy++)
		{
		  if (IS_IDE_DRIVE(pdev, phy)) {
		    BOOLEAN success = GetIoMode(pdev, phy);
		    if (!success) {
		      /*Disable device.*/
		      pdev->PhysicalDrive[phy].DeviceFlags &=
		          ~DFLAGS_DEVICE_PRESENT;
		    }
		    pdev->PhysicalDrive[phy].ucIsPowerConnected = 1;
		  }
		}
	}

	/*initialize boot log drv mapping*/
	for (phy = 0; phy < MAX_LOGICAL_DRIVES;  phy++)
	{
		pdev->BootLogDrvMapping[phy] = phy;
	}

	initialize_device_extension(pdev);
	
	if (GetConfigInfoAndErrorLogSectorInfo(pdev)) {

		ConfigureRaidDrives(pdev, pdev->ConfigurationVirtualBuffer);

		if (pdev->ucIRCDType == IRCD_3X) {
		  EnquireProperties(pdev, TRUE);
		}
	}

	/*programming drives after Inquiry data collection*/
	for (phy = 0; phy < MAX_DRIVES_PER_CONTROLLER;  phy++)
	{
		if (pdev->PhysicalDrive[phy].ucIsPowerConnected) {
		  SetMaxTransferMode(pdev, phy);
		}
	}

	if (pdev->ulLogicalDriveCount) {

		for (log = 0; log < pdev->ulLogicalDriveCount; log ++)
		{
		  switch (pdev->LogicalDrive[log].RaidLevel) {
		    case None:
		      sprintf(raid_type, "%s", "NON-RAID");
		      break;
		    case Raid0:
		      sprintf(raid_type, "%s", "RAID0");
		      break;
		    case Raid1:
		      sprintf(raid_type, "%s", "RAID1");
		      break;
		    case Raid10:
		      sprintf(raid_type, "%s", "RAID10");
		      break;
		    case SpareDrivePool:
		      sprintf(raid_type, "%s", "SPAREPOOL");
		      break;
		    default:
		      sprintf(raid_type, "%s", "UNKNOWN");
		      break;
		  }

		  switch (pdev->LogicalDrive[log].Status) {
		    case LDS_Online:
		      sprintf(raid_status, "%s", "ONLINE");
		      break;
		    case LDS_Critical:
		      sprintf(raid_status, "%s", "CRITICAL");
		      break;
		    case LDS_Degraded:
		      sprintf(raid_status, "%s", "DEGRADED");
		      break;
		    case LDS_Deleted:
		      sprintf(raid_status, "%s", "DELETED");
		      break;
		    case LDS_OffLine:
		    default:
		      sprintf(raid_status, "%s", "OFFLINE");
		      break;
		  }

{
unsigned int sz = (pdev->LogicalDrive[log].Sectors/2) * (IDE_SECTOR_SIZE/256);
		printk_Rlsi(KERN_INFO
			"megaide: LD %d %s status=%s"
			" %ld 512-byte sectors (%d MB) drives=%ld\n",
			log, raid_type, raid_status,
			(ULONG) pdev->LogicalDrive[log].Sectors,
			(sz - sz/625 + 974)/1950,
			(ULONG) pdev->LogicalDrive[log].PhysicalDriveCount);
}

		}
		InitDriveFeatures(pdev);
	}/* else {
		return AR_FALSE;
	}*/

	return AR_TRUE;
}/*Init IdeRaidController() */



/* to identify devices attached to an IDE controller*/
BOOLEAN
FindDevices(HW_DEVICE_EXTENSION *pdev, UCHAR Channel)
{
	IDE_REGISTERS_1 *base1 =  pdev->Channel[Channel].BaseIoAddress1;
	BOOLEAN deviceResponded = AR_FALSE;
	UCHAR deviceNumber;
	UCHAR i;
	UCHAR phy;

	AR_DEBUG(AR_DEBUG_FINDDEV, (KERN_DEBUG "megaide: find_ devices Called\n"));

	/* Clear expecting interrupt flags.*/
	pdev->Channel[Channel].ExpectingInterrupt = 0;

	/*Search for devices*/
	for (deviceNumber = 0; deviceNumber < 2; deviceNumber++)
	{

		phy = (Channel << 1) + deviceNumber;
		
		/* serial ATA has only one drive per channel*/
		if (((pdev->ucCardType == SIL_3112) ||
		     (pdev->ucCardType == INTEL_ICH5R)) && (deviceNumber > 0)) {
		  break;
		}
		
		AR_DEBUG(AR_DEBUG_FINDDEV,
		         (KERN_DEBUG "megaide: Searching Channel %d device%d\n",
		         Channel, deviceNumber));
		
		if (!IsDrivePresent(pdev, phy)) {
		  continue;
		}


		/*
		 * Issue IDE Identify. If an Atapi device is actually present,
		 * the signature will be asserted, and the drive will
		 * be recognized as such.
		 */

		if (IssueIdentify(pdev, deviceNumber, Channel,
			          IDE_COMMAND_IDENTIFY)) {
		  /*IDE drive found*/
		  AR_DEBUG(AR_DEBUG_FINDDEV,
		           (KERN_DEBUG "megaide: Find TID %d is IDE\n", phy));

		  pdev->PhysicalDrive[phy].DeviceFlags |= DFLAGS_DEVICE_PRESENT;
		  deviceResponded = AR_TRUE;
		  /*Indicate IDE - not ATAPI device.*/
		  pdev->PhysicalDrive[phy].DeviceFlags &= ~DFLAGS_ATAPI_DEVICE;
		} else {
		  /*Apparently no device*/
		  continue;
		}
	}

	for (i = 0; i < MAX_CHANNELS_PER_CONTROLLER; i++)
	{
	  phy = (Channel << 1) + i;

	  if (pdev->PhysicalDrive[phy].DeviceFlags & DFLAGS_DEVICE_PRESENT) {

	   /*Initialize LastDriveFed.*/
	   pdev->Channel[Channel].LastDriveFed = i;

	     if (!(pdev->PhysicalDrive[phy].DeviceFlags & DFLAGS_ATAPI_DEVICE)&&
	           deviceResponded) {

	       /*
	       * This hideous hack is to deal with ESDI devices
	       * that return garbage geometry in the IDENTIFY data.
	       * This is ONLY for the crashdump environment as
	       * these are ESDI devices.
	       */

	       if (pdev->PhysicalDrive[phy].IdentifyData.SectorsPerTrack==0x35&&
	           pdev->PhysicalDrive[phy].IdentifyData.NumberOfHeads==0x07) {

	         /*Change these values to something reasonable.*/
	         pdev->PhysicalDrive[phy].IdentifyData.SectorsPerTrack = 0x34;
	         pdev->PhysicalDrive[phy].IdentifyData.NumberOfHeads = 0x0E;
	       }

	       if (pdev->PhysicalDrive[phy].IdentifyData.SectorsPerTrack==0x35&&
	           pdev->PhysicalDrive[phy].IdentifyData.NumberOfHeads==0x0F) {

	           /*Change these values to something reasonable.*/
	           pdev->PhysicalDrive[phy].IdentifyData.SectorsPerTrack = 0x34;
	           pdev->PhysicalDrive[phy].IdentifyData.NumberOfHeads = 0x0F;
	       }

	       if (pdev->PhysicalDrive[phy].IdentifyData.SectorsPerTrack==0x36&&
	           pdev->PhysicalDrive[phy].IdentifyData.NumberOfHeads==0x07) {

	         /*Change these values to something reasonable.*/
	          pdev->PhysicalDrive[phy].IdentifyData.SectorsPerTrack = 0x3F;
	          pdev->PhysicalDrive[phy].IdentifyData.NumberOfHeads = 0x10;
	       }
	     }
	  } /*if(pdev->PhysicalDrive[phy].DeviceFlags &)*/
	}

	for (i = 0; i < 2; i++)
	{
	  phy = (Channel << 1) + i;
	  if (pdev->PhysicalDrive[phy].DeviceFlags & DFLAGS_DEVICE_PRESENT) {
	    /*select first available drive*/
	    SELECT_DEVICE(base1, phy);
	    break;
	  }
	}

#ifdef DEBUG_ON
	if (!deviceResponded) {
	  AR_DEBUG(AR_DEBUG_FINDDEV,
	           (KERN_DEBUG "megaide: No devices Found in Chan %d\n", Channel));
	}
#endif

	return deviceResponded;
}/*Find Devices() */



BOOLEAN 
GetIoMode(HW_DEVICE_EXTENSION *pdev, UCHAR TargetId)
{
	IDENTIFY_DATA *idenData;
	ULONG sectors;
	BOOLEAN success = AR_TRUE;

	idenData = &(pdev->PhysicalDrive[TargetId].IdentifyData);

	/*Calculate the total addressable sectors using CHS convention.*/
	sectors = idenData->NumberOfCylinders *
	          idenData->NumberOfHeads *
	          idenData->SectorsPerTrack;
	
	AR_DEBUG(AR_DBG_INIT, (KERN_DEBUG "megaide: sectors%ld \n", sectors));

	if (idenData->TranslationFieldsValid &
	    IDENTIFY_FAST_TRANSFERS_SUPPORTED) {
		
		if ((idenData->Capabilities &
		     IDENTIFY_CAPABILITIES_LBA_SUPPORTED)) {
		  pdev->PhysicalDrive[TargetId].DeviceFlags |= DFLAGS_LBA;
		  /*Get the total number of LBA-mode addressable sectors.*/
		  sectors = idenData->UserAddressableSectors;

		  AR_DEBUG(AR_DBG_INIT,
		           (KERN_DEBUG "megaide: TID %dsupports LBA\n", TargetId));
		}
	}
   	
	AR_DEBUG(AR_DBG_INIT, (KERN_DEBUG "megaide: ##sectors%ld \n", sectors));

	pdev->PhysicalDrive[TargetId].NumberOfBlocks = sectors;

	if (IS_48_BIT_ENABLED(pdev, TargetId) &&
	    idenData->AddressableSectors48bit[0]) {
	  pdev->PhysicalDrive[TargetId].NumberOfBlocks48bit =
	      idenData->AddressableSectors48bit[0];
	}
	GetTransferMode(pdev, TargetId);

	return(success);
}/*Get IoMode() */


BOOLEAN 
GetTransferMode(HW_DEVICE_EXTENSION *pdev, UCHAR TargetId)
{
	IDENTIFY_DATA *idenData;
	idenData = &(pdev->PhysicalDrive[TargetId].IdentifyData);

	/*By default assume this drive is PioMode 0*/
	pdev->PhysicalDrive[TargetId].TransferMode = MP_PioMode0;

	if (idenData->AdvancedPioModes != 0) {

		if (idenData->AdvancedPioModes & IDENTIFY_PIO_MODE4) {
		  pdev->PhysicalDrive[TargetId].TransferMode = MP_PioMode4;
		} else {
		  if (idenData->AdvancedPioModes & IDENTIFY_PIO_MODE3) {
		    pdev->PhysicalDrive[TargetId].TransferMode = MP_PioMode3;
		  } else {
		    pdev->PhysicalDrive[TargetId].TransferMode = MP_PioMode0;
		  }
		}
	}

	if (idenData->MultiWordDmaActive) {
	  switch (idenData->MultiWordDmaActive)
	  {
	    case 1:
	      pdev->PhysicalDrive[TargetId].TransferMode = MP_DmaMode0;
	      pdev->PhysicalDrive[TargetId].DeviceFlags |= DFLAGS_USE_DMA;
	      break;

	    case 2:
	      pdev->PhysicalDrive[TargetId].TransferMode = MP_DmaMode1;
	      pdev->PhysicalDrive[TargetId].DeviceFlags |= DFLAGS_USE_DMA;
	      break;

	    case 4:
	      pdev->PhysicalDrive[TargetId].TransferMode = MP_DmaMode2;
	      pdev->PhysicalDrive[TargetId].DeviceFlags |= DFLAGS_USE_DMA;
	      break;
	  }
	}

	if (idenData->UltraDmaActive) {
	  switch (idenData->UltraDmaActive)
	  {
	    case 1:
	      pdev->PhysicalDrive[TargetId].TransferMode = MP_UdmaMode0;
	      pdev->PhysicalDrive[TargetId].DeviceFlags |= DFLAGS_USE_UDMA;
	      break;

	    case 2:
	      pdev->PhysicalDrive[TargetId].TransferMode = MP_UdmaMode1;
	      pdev->PhysicalDrive[TargetId].DeviceFlags |= DFLAGS_USE_UDMA;
	      break;

	    case 4:
	      pdev->PhysicalDrive[TargetId].TransferMode = MP_UdmaMode2;
	      pdev->PhysicalDrive[TargetId].DeviceFlags |= DFLAGS_USE_UDMA;
	      break;

	    case 8:
	      pdev->PhysicalDrive[TargetId].TransferMode = MP_UdmaMode3;
	      pdev->PhysicalDrive[TargetId].DeviceFlags |= DFLAGS_USE_UDMA;
	      break;

	    case 0x10:
	      pdev->PhysicalDrive[TargetId].TransferMode = MP_UdmaMode4;
	      pdev->PhysicalDrive[TargetId].DeviceFlags |= DFLAGS_USE_UDMA;
	      break;

	    case 0x20:
	      pdev->PhysicalDrive[TargetId].DeviceFlags |= DFLAGS_USE_UDMA;

	      /*to take care of higher Udma speed*/
	      if (Udma100 >= pdev->ControllerSpeed) {
	        pdev->PhysicalDrive[TargetId].TransferMode = MP_UdmaMode5;
	      } else {
	        pdev->PhysicalDrive[TargetId].TransferMode = MP_UdmaMode4;
	      }
	      break;

	    /* Ultra DMA 6 support*/
	    case 0x40:
	      pdev->PhysicalDrive[TargetId].DeviceFlags |= DFLAGS_USE_UDMA;
	      if (Udma133 == pdev->ControllerSpeed) {
	        pdev->PhysicalDrive[TargetId].TransferMode = MP_UdmaMode6;
	      } else if (Udma100 == pdev->ControllerSpeed) {
	        pdev->PhysicalDrive[TargetId].TransferMode = MP_UdmaMode5;
	      } else {
	        pdev->PhysicalDrive[TargetId].TransferMode = MP_UdmaMode4;
	      }
	      break;
	  }
	}

	return  AR_TRUE;
}/*GetTransferMode() */


BOOLEAN
SetTransferMode(HW_DEVICE_EXTENSION *pdev, UCHAR TargetId,
                        MP_TRANSFER_MODE TransferMode)
{
	IDE_REGISTERS_1 *base1;
	IDE_REGISTERS_2 *base2;
	BM_REGISTERS    *BaseBmAddress;
	UCHAR status = 0;
	UCHAR transfer_mode;

	//this change is applicable only to 3x ircd
	if (pdev->ucIRCDType == IRCD_3X) {
		/*to program drive as in enquiry Data*/
		UCHAR tMode = GetInquiryTransferMode(pdev, TargetId);

		if (tMode >= MP_UdmaMode0) {
		  pdev->PhysicalDrive[TargetId].TransferMode =
		    MEGA_FIND_MIN(tMode, pdev->PhysicalDrive[TargetId].TransferMode);
		  TransferMode = tMode;
		}
	}

	//This is the Fix for Maxtor drives not working
	//if DMA write is more than 62 secotrs.. This is a drive issue
	// and it is present with UDMA 6 mode. we are programming back to
	//UDMA5
	if (PCI_DEVICE_ID_CMD_3112 == GET_DEVICE_ID(pdev)) {
		//this change is applicable only Maxtor drives
		if (!strncmp(pdev->PhysicalDrive[TargetId].IdentifyData.ModelNumber,
		           "aMtxro", 6))
		{
			printk ("megaide: Drv[%d] is Maxtor SATA drive\n", TargetId);
			printk ("megaide: UDMA6 is not working for Maxtor\n");
			printk ("megaide: setting to UDMA5 mode\n");
			if (MP_UdmaMode6 == TransferMode) {
				TransferMode = MP_UdmaMode5;
			}
		}
	}

	switch (TransferMode)
	{
		case MP_DmaMode0:
			transfer_mode = STM_MULTIWORD_DMA(0);
			break;

		case MP_DmaMode1:
			transfer_mode = STM_MULTIWORD_DMA(1);
			break;

		case MP_DmaMode2:
			transfer_mode = STM_MULTIWORD_DMA(2);
			break;

		case MP_UdmaMode0:
			transfer_mode = STM_UDMA(0);
			break;

		case MP_UdmaMode1:
			transfer_mode = STM_UDMA(1);
			break;

		case MP_UdmaMode2:
			transfer_mode = STM_UDMA(2);
			break;

		case MP_UdmaMode3:
			transfer_mode = STM_UDMA(3);
			break;

		case MP_UdmaMode4:
			transfer_mode = STM_UDMA(4);
			break;

		case MP_UdmaMode5:
			transfer_mode = STM_UDMA(5);
			break;

		case MP_UdmaMode6:
			transfer_mode = STM_UDMA(6);
			break;

		default:
			/*CURRENTLY we are not programming drives to PIO mode*/
			return FALSE;
			break;
	}

	base1 = (IDE_REGISTERS_1 *) pdev->Channel[TargetId >> 1].BaseIoAddress1;
	base2 = (IDE_REGISTERS_2 *) pdev->Channel[TargetId >> 1].BaseIoAddress2;
	BaseBmAddress  = (BM_REGISTERS *) pdev->Channel[TargetId >> 1].BaseBmAddress;
	
	SELECT_DEVICE(base1, TargetId);

	/*Program Drives*/

	ScsiPortWritePortUchar((unsigned)((UCHAR *) base1 + 1),
			FEATURE_SET_TRANSFER_MODE);
	ScsiPortWritePortUchar((unsigned)&(base1->SectorCount),
			transfer_mode);
	ScsiPortWritePortUchar((unsigned)&(base1->Command),
			IDE_COMMAND_SET_FEATURES);
	WAIT_ON_BASE_BUSY(base1, status);

	if ((status & IDE_STATUS_ERROR) || (status & IDE_STATUS_BUSY)) {
		printk_Rlsi(KERN_WARNING "megaide: setfeatures TID %d failed.\n",
		TargetId);
		return FALSE;
	}
	
	status = ScsiPortReadPortUchar((unsigned)&(BaseBmAddress->Status.AsUchar));
	status |= 0x60;
	ScsiPortWritePortUchar((unsigned)&(BaseBmAddress->Status.AsUchar), status);
	status = ScsiPortReadPortUchar((unsigned)&(BaseBmAddress->Status.AsUchar));

	/*Program PCI space*/

	gaMideFunc[pdev->ucCardType].pfnProgramChip(pdev, TargetId,
	                                            (UCHAR) TransferMode);

	pdev->PhysicalDrive[TargetId].TransferMode = TransferMode;

	return TRUE;
}

BOOLEAN
AssignSglPtrsForChannels(HW_DEVICE_EXTENSION *pdev)
{

    CHANNEL *pChannel;
    SGL_ENTRY *sgl;
    LONG lengthInNextBoundary;
    LONG lengthLeftInBoundary;
    ULONG maxSglEntries, chan;
    ULONG sglBasePhysicalAddress, length;
    ULONG ulStartAddress, ulEndAddress, ulStartPage, ulEndPage;
    ULONG ulLengthReduced = 0;
    UCHAR ucControllerId;
    adapter_t * adapter = (adapter_t *)pdev->adapter;

	ucControllerId = pdev->ucControllerId;

	pChannel = &(pdev->Channel[0]);

	for (chan = 0; chan<MAX_CHANNELS_PER_CONTROLLER; chan++)
	{
	  /*Dynamic DMA Implementation*/
	  pChannel[chan].pSglEntry =
	      pci_alloc_consistent(adapter->pstDev,
	              (MAX_SGL_ENTRIES_PER_PHYSICAL_DRIVE * sizeof(SGL_ENTRY)),
	              &adapter->Channel[chan].h_sgl_entry);

	  if (pChannel[chan].pSglEntry == NULL) {
	    AR_DEBUG(AR_DEBUG_WARNING,
	             (KERN_DEBUG "megaide: pci_alloc failed SglEntry\n"));
	    return FALSE;
	  }

	  memset_Rlsi(pChannel[chan].pSglEntry, 0,
	      MAX_SGL_ENTRIES_PER_PHYSICAL_DRIVE * sizeof(SGL_ENTRY));

	  sglBasePhysicalAddress = (ULONG) adapter->Channel[chan].h_sgl_entry;

	  sgl = pChannel[chan].pSglEntry;

	  if (sglBasePhysicalAddress == 0) {
	    printk_Rlsi(KERN_ERR "megaide: Physical Address Came as Zero.....\n");
	    return(FALSE);
	  }

	  if (sglBasePhysicalAddress %  SGL_HW_BOUNDARY) {
	    length = (((sglBasePhysicalAddress / SGL_HW_BOUNDARY) + 1) *
	              SGL_HW_BOUNDARY) - sglBasePhysicalAddress;
	  } else {
	    length = SGL_HW_BOUNDARY;
	  }

	  if (length >=
	      (MAX_SGL_ENTRIES_PER_PHYSICAL_DRIVE * sizeof(SGL_ENTRY))) {
	    length = MAX_SGL_ENTRIES_PER_PHYSICAL_DRIVE * sizeof(SGL_ENTRY);
	  }

	  ulStartAddress = (ULONG) sglBasePhysicalAddress;
	  ulEndAddress = ulStartAddress + length;


	  /* Check to see if SGL Pointer is DWORD Aligned and make it so.
	   * Remember we allocated an extra SGL entry for this purpose.
	   */

	  if ((ULONG) sglBasePhysicalAddress & 3) {

	    sglBasePhysicalAddress =
	        (((ULONG) sglBasePhysicalAddress & 0xfffffffcL) + 4L);
	    ulLengthReduced = sglBasePhysicalAddress - ulStartAddress;
	    ulStartAddress = (ULONG) sglBasePhysicalAddress;
	    length -= ulLengthReduced;
	    sgl += ulLengthReduced;
	  }

	  ulStartPage = (ULONG) (ulStartAddress / SGL_HW_BOUNDARY);
	  ulEndPage = (ULONG) (ulEndAddress / SGL_HW_BOUNDARY);

	  if (ulStartPage == ulEndPage) {
	    /*all entries are in the same page*/
	    maxSglEntries = (length / sizeof(SGL_ENTRY));
	  } else {
	    /* Make sure the SGL does NOT cross a 4K boundary.*/
	    lengthLeftInBoundary = SGL_HW_BOUNDARY -
	          ((ULONG) sglBasePhysicalAddress &(SGL_HW_BOUNDARY - 1));
	    lengthInNextBoundary = length - lengthLeftInBoundary;

	    if (lengthInNextBoundary > lengthLeftInBoundary) {
	      /* SGL crosses 4KB boundary and top portion is larger
	       * than bottom. Use top portion.
	       */
	      sglBasePhysicalAddress =
	          sglBasePhysicalAddress + lengthLeftInBoundary;
	      sgl = (SGL_ENTRY *) ((ULONG) sgl + lengthLeftInBoundary);
	      maxSglEntries =  lengthInNextBoundary / sizeof(SGL_ENTRY);
	    } else {
	      maxSglEntries =  lengthLeftInBoundary / sizeof(SGL_ENTRY);
	    }
	  }

	  /*Save SGL info for next call.*/
	  pChannel[chan].SglBaseVirtualAddress = (SGL_ENTRY *) sgl;
	  pChannel[chan].SglBasePhysicalAddress = sglBasePhysicalAddress;
	  pChannel[chan].MaxSglEntries = maxSglEntries;
	}

	return TRUE;
}



/* Removes the build in IDE driver if present*/
void
RemoveBuildInIDESupport(unsigned short ReqVenID, unsigned short ReqDevId,
                        unsigned short ReqSubsysId, unsigned short ReqSubsysVId)
{
	struct pci_dev	*pstPciDev;
	int iCount;
	int subVid, subDid;

	AR_DEBUG(AR_DBG_INIT, (KERN_DEBUG "megaide: Remove CMDSupport\n"));

	for (iCount = 0; (iCount<MAX_HWIFS); ++iCount)
	{
		pstPciDev = ide_hwifs[iCount].pci_dev;

		if (!pstPciDev || ide_hwifs[iCount].present == 0) {
			continue;
		}

		subVid = ide_hwifs[iCount].pci_dev->subsystem_vendor;
		subDid = ide_hwifs[iCount].pci_dev->subsystem_device;
		//Remove Only requested Device
		if ((ReqDevId == ide_hwifs[iCount].pci_dev->device) &&
		    (ReqVenID == ide_hwifs[iCount].pci_dev->vendor)  &&
		    (ReqSubsysId == subDid) &&
		    (ReqSubsysVId == subVid)) {
			AR_DEBUG(AR_DBG_INIT,
			        (KERN_DEBUG "megaide: ide_hwifs[%d].present=%d\n",
			        iCount, ide_hwifs[iCount].present));
			ide_unregister(iCount);
		}
	}/*for*/
}/*Remove Build InIDESupport() */

/*This function check for the io region availability and claim
 *those io regions for megaide driver
 */
BOOLEAN
ClaimOurIOPorts(HW_DEVICE_EXTENSION *pdev)
{
	if (check_region((ULONG) pdev->Channel[0].BaseIoAddress1, (ULONG) 8)) {
	  AR_DEBUG(AR_DEBUG_FINDDEV,
	           (KERN_DEBUG "megaide: check_region ioport[0] not claimed\n"));
	  release_region((ULONG) pdev->Channel[0].BaseIoAddress1, (ULONG) 8);
	}
	request_region((ULONG) pdev->Channel[0].BaseIoAddress1,
	               (ULONG)8, "megaide");

	if (check_region((ULONG) pdev->Channel[0].BaseIoAddress2, (ULONG) 4)) {
	  AR_DEBUG(AR_DEBUG_FINDDEV,
	           (KERN_DEBUG "megaide: check_region ioport[1] not claimed\n"));
	  release_region((ULONG) pdev->Channel[0].BaseIoAddress2+2, (ULONG) 1);
	}
	request_region((ULONG) pdev->Channel[0].BaseIoAddress2,
	               (ULONG) 4, "megaide");

	if (check_region((ULONG) pdev->Channel[1].BaseIoAddress1, (ULONG) 8)) {
	  AR_DEBUG(AR_DEBUG_FINDDEV,
	           (KERN_DEBUG "megaide: check_region ioport[2] not claimed\n"));
	  release_region((ULONG) pdev->Channel[1].BaseIoAddress1, (ULONG) 8);
	}
	request_region((ULONG) pdev->Channel[1].BaseIoAddress1,
	               (ULONG) 8, "megaide");

	if (check_region((ULONG) pdev->Channel[1].BaseIoAddress2, (ULONG) 4)) {
	  AR_DEBUG(AR_DEBUG_FINDDEV,
	           (KERN_DEBUG "megaide: check_region ioport[3] not claimed\n"));
	  release_region((ULONG) pdev->Channel[1].BaseIoAddress2+2, (ULONG) 1);
	}
	request_region((ULONG)pdev->Channel[1].BaseIoAddress2,
	               (ULONG)4, "megaide");

	if (check_region((ULONG) pdev->Channel[0].BaseBmAddress, (ULONG) 8)) {
	  AR_DEBUG(AR_DEBUG_FINDDEV,
	    (KERN_DEBUG "megaide: check_region ioport[4] not claimed\n"));
	  release_region((ULONG) pdev->Channel[0].BaseBmAddress, (ULONG) 8);
	}

	if (check_region((ULONG) pdev->Channel[1].BaseBmAddress, (ULONG) 8)) {
	  AR_DEBUG(AR_DEBUG_FINDDEV,
	           (KERN_DEBUG "megaide: check_region ioport[5] not claimed\n"));
	  release_region((ULONG) pdev->Channel[1].BaseBmAddress, (ULONG) 8);
	}

	request_region((ULONG) pdev->Channel[0].BaseBmAddress,
	               (ULONG) 16, "megaide");

	return AR_TRUE;
}/*Request Our IOPorts() */



BOOLEAN
ReleaseOurIOPorts(HW_DEVICE_EXTENSION *pdev)
{

	AR_DEBUG(AR_DEBUG_RELEASE, (KERN_DEBUG "megaide: ReleaseOur IOPorts\n"));

#ifdef BP_SUPPORT
	idebp_release(pdev);
#endif

	release_region((ULONG) pdev->Channel[0].BaseIoAddress1, (ULONG) 8);
	release_region((ULONG) pdev->Channel[0].BaseIoAddress2, (ULONG) 4);
	release_region((ULONG) pdev->Channel[1].BaseIoAddress1, (ULONG) 8);
	release_region((ULONG) pdev->Channel[1].BaseIoAddress2, (ULONG) 4);
	release_region((ULONG) pdev->Channel[0].BaseBmAddress, (ULONG) 16);
	return AR_TRUE;
}/*Release Our IOPorts() */

/*This function will check for the presence of the device with the given target ID*/
BOOLEAN 
IsDrivePresent(HW_DEVICE_EXTENSION *pdev, UCHAR tid)
{
	IDE_REGISTERS_1 *base1 = pdev->Channel[tid>>1].BaseIoAddress1;
	UCHAR ucStatus;

	AR_DEBUG(AR_DEBUG_FINDDEV, (KERN_DEBUG 
		"megaide: is_drive _present chan %d \n", (tid>>1)));
	
	if(GET_DEVICE_ID(pdev)==PCI_DEVICE_ID_CMD_3112) {
	
		/*One drive per channel*/
		if (tid & 0x01) return AR_TRUE;
		
		if (0x03/*DET_DEVICE_DETECTED_PHY_GOOD*/ != 
			AthenaDetectPhysicalChange(pdev, (tid>>1))) {
			AR_DEBUG(AR_DEBUG_FINDDEV, (KERN_DEBUG 
				"megaide: Drive NOT responding at %d \n", tid));
			return AR_FALSE;
		} else {
			AR_DEBUG(AR_DEBUG_FINDDEV, (KERN_DEBUG 
				"megaide: Drive responding at %d \n", tid));
			return AR_TRUE;
		}
	}
	
	SELECT_DEVICE(base1, tid);
	GET_STATUS(base1, ucStatus);
	if (0xff == ucStatus || 0x7f == ucStatus) {
		AR_DEBUG(AR_DEBUG_FINDDEV, (KERN_DEBUG 
			"megaide: No divice Status returned as 0xff\n"));
		return AR_FALSE;
	}
	
	SELECT_DEVICE(base1, tid);
	ScsiPortWritePortUchar((unsigned)&base1->SectorCount, 0x5a);
	ScsiPortStallExecution(60);
	
	if (0x5a == ScsiPortReadPortUchar((unsigned)&base1->SectorCount)) {
		AR_DEBUG(AR_DEBUG_FINDDEV, (KERN_DEBUG 
			"megaide: Drv %d is present\n", tid));
		return AR_TRUE;
	} else {
		AR_DEBUG(AR_DEBUG_FINDDEV, (KERN_DEBUG 
			"megaide: Drv %d is not present\n", tid));
		return AR_FALSE;
	}
}/*Is Drive Present() */



/*Issue IDENTIFY command to a device*/
BOOLEAN 
IssueIdentify(HW_DEVICE_EXTENSION *pdev, UCHAR DeviceNo,
                      UCHAR Channel, UCHAR Command)
{
	IDE_REGISTERS_1 *base1 = pdev->Channel[Channel].BaseIoAddress1;
	ULONG i;
	UCHAR status;
	UCHAR phy;
	PUSHORT puIdentifyData;
	BM_REGISTERS *baseBm = pdev->Channel[Channel].BaseBmAddress;

	AR_DEBUG(AR_DEBUG_FINDDEV, (KERN_DEBUG "megaide: Issue Identify:\n"));

	/* Select device 0 or 1*/
	SELECT_DEVICE(base1, DeviceNo);

   	/*
	 * Check that the status register makes sense.
    	 * The call came here since there is a drive.
	 * so let us not worry about whether there
	 * is any drive at this place
	 */
	GET_BASE_STATUS(base1, status);

	/* Load CylinderHigh and CylinderLow with number bytes to transfer*/
	ScsiPortWritePortUchar((unsigned)&base1->CylinderHigh, (0x200 >> 8));
	ScsiPortWritePortUchar((unsigned)&base1->CylinderLow, (0x200 & 0xFF));

	/*Send IDENTIFY command.*/
	ScsiPortWritePortUchar((unsigned)&base1->Command, Command);

	WAIT_ON_BUSY(base1, status);

	if ((!(status & IDE_STATUS_BUSY)) && (!(status & IDE_STATUS_DRQ))) {
	  AR_DEBUG(AR_DEBUG_FINDDEV, (KERN_DEBUG "megaide: Identtify Failed\n"));
	  /* this is an error... so let us not try any more.*/
		if (pdev->ucCardType == INTEL_ICH5R) {
			UCHAR status1;
			CLEAR_BM_INT(baseBm, status1);
		}
	  return AR_FALSE;
	}

	WAIT_ON_BUSY(base1, status);

	/*Wait for DRQ.*/
	WAIT_FOR_DRQ(base1, status);


	/*Read status to acknowledge any interrupts generated.*/
	GET_BASE_STATUS(base1, status);

	/**Check for error on really stupid master devices
	 * that assert random patterns of bits in the status
	 * register at the slave address.
	 */

	if ((Command == IDE_COMMAND_IDENTIFY) && (status & IDE_STATUS_ERROR)) {
	  AR_DEBUG(AR_DEBUG_FINDDEV, (KERN_DEBUG "megaide: Identify Failed 3\n"));
		if (pdev->ucCardType == INTEL_ICH5R) {
			UCHAR status1;
			CLEAR_BM_INT(baseBm, status1);
		}
	  return AR_FALSE;
	}

	/** Suck out 256 words. After waiting for one model
	 * that asserts busy after receiving the Packet Identify command.
	 */

	WAIT_ON_BUSY(base1, status);

	if (!(status & IDE_STATUS_DRQ)) {
	  AR_DEBUG(AR_DEBUG_FINDDEV, (KERN_DEBUG "megaide: Failed to access\n"));
		if (pdev->ucCardType == INTEL_ICH5R) {
			UCHAR status1;
			CLEAR_BM_INT(baseBm, status1);
		}
	  return AR_FALSE;
	}

	phy = (Channel << 1) + DeviceNo;

	memset_Rlsi(&pdev->PhysicalDrive[phy].IdentifyData,
	  0, sizeof(IDENTIFY_DATA));

	AR_READ_BUFFER(base1,
	               (PUSHORT) &pdev->PhysicalDrive[phy].IdentifyData, 256);

	/* Check out a few capabilities / limitations of the device.*/
	if (pdev->PhysicalDrive[phy].IdentifyData.MediaStatusNotification & 1) {
	  /* Determine if this drive supports the MSN functions.*/

	  AR_DEBUG(AR_DEBUG_WARNING,
	           (KERN_DEBUG "megaide:Identify:Marking drive %d as removable\n",
	           (Channel * 2 + DeviceNo)));

	    pdev->PhysicalDrive[(Channel * 2) + DeviceNo].DeviceFlags |=
	          DFLAGS_REMOVABLE_DRIVE;
	}

	if (pdev->PhysicalDrive[phy].IdentifyData.MaximumBlockTransfer) {
	  /* Determine max. block transfer for this device*/
	  pdev->PhysicalDrive[phy].MaximumBlockXfer =
	       (UCHAR) (pdev->PhysicalDrive[phy].IdentifyData.\
	  MaximumBlockTransfer & 0xFF);
	}

	puIdentifyData = (PUSHORT) &(pdev->PhysicalDrive[phy].IdentifyData);

	if (pdev->PhysicalDrive[phy].IdentifyData.GeneralConfiguration & 0x20 &&
	    Command != IDE_COMMAND_IDENTIFY) {

	  /* This device interrupts with the assertion of DRQ after
	   * receiving Atapi Packet Command
	   */
	  pdev->PhysicalDrive[phy].DeviceFlags |= DFLAGS_INT_DRQ;

	  AR_DEBUG(AR_DEBUG_FINDDEV,
	           (KERN_DEBUG "megaide: dev will interrupts on DRQassertion\n"));

	}
#ifdef AR_DEBUG
	 else {
	  AR_DEBUG(AR_DEBUG_FINDDEV,
	         (KERN_DEBUG "megaide: dev will not interrupt on DRQassertion\n"));
	}
#endif
	if (((pdev->PhysicalDrive[phy].IdentifyData.GeneralConfiguration &
	      0xF00) == 0x100) && Command != IDE_COMMAND_IDENTIFY) {
	  /*This is a tape.*/

	  pdev->PhysicalDrive[phy].DeviceFlags |= DFLAGS_TAPE_DEVICE;

	  AR_DEBUG(AR_DEBUG_FINDDEV,
	           (KERN_DEBUG "megaide: Identify: Device is a tape drive.\n"));

	} else {
	  AR_DEBUG(AR_DEBUG_FINDDEV,
	           (KERN_DEBUG "megaide: Identify:Device is not a tape drive.\n"));
	}

	/*
	 * Work around for some IDE and one model Atapi that
	 * will present more than 256 bytes for the Identify data.
	 */

	WAIT_ON_BUSY(base1, status);

	for (i = 0; i < 0x10000; i++)
	{
	  GET_STATUS(base1, status);

	  if (status & IDE_STATUS_DRQ) {
	    /*Suck out any remaining bytes and throw away.*/

	  } else {
	    break;
	  }
	}

	AR_DEBUG(AR_DEBUG_FINDDEV,
	         (KERN_DEBUG "megaide: Status after read words(%x) \n", status));

	if (pdev->ucCardType == INTEL_ICH5R) {
		UCHAR status1;
		CLEAR_BM_INT(baseBm, status1);
	}
	return AR_TRUE;

} /*Issue Identify() */


BOOLEAN InitDriveFeatures(HW_DEVICE_EXTENSION *pdev)
{
	UCHAR ucDrvInd;

	for (ucDrvInd = 0; ucDrvInd<MAX_DRIVES_PER_CONTROLLER; ucDrvInd++)
	{
		InitSingleDriveFeatures(pdev, ucDrvInd);
	}
	return AR_TRUE;

}/*Init DriveFeatures*/


BOOLEAN InitSingleDriveFeatures(HW_DEVICE_EXTENSION *pdev, UCHAR ucDrvInd)
{
	IDE_REGISTERS_1 *base1;
	UCHAR statusByte;

	if (!IS_IDE_DRIVE(pdev, ucDrvInd))
		return AR_FALSE;

	base1 = pdev->Channel[ucDrvInd >> 1].BaseIoAddress1;

	SELECT_DEVICE(base1, ucDrvInd);
	
	if (pdev->PhysicalDrive[ucDrvInd].bEnableRwCache) {
	  AR_DEBUG(AR_DBG_INIT, (KERN_DEBUG "megaide: Write Cache Enabled \n"));
	  ScsiPortWritePortUchar((unsigned)((UCHAR *) base1 + 1),
	                          FEATURE_ENABLE_WRITE_CACHE);
	} else {

	  AR_DEBUG(AR_DBG_INIT, (KERN_DEBUG "megaide: Write Cache Disabled\n"));
	  ScsiPortWritePortUchar((unsigned)((UCHAR *) base1 + 1),
	                          FEATURE_DISABLE_WRITE_CACHE);
	}

	ScsiPortWritePortUchar((unsigned)&(base1->Command),
	                        IDE_COMMAND_SET_FEATURES);
	WAIT_ON_BASE_BUSY(base1, statusByte);

	if (pdev->PhysicalDrive[ucDrvInd].bEnableReadAheadCache) {
	  AR_DEBUG(AR_DBG_INIT, (KERN_DEBUG "megaide: Read Ahead Enabled \n"));
	  ScsiPortWritePortUchar((unsigned)((UCHAR *) base1 + 1),
	                          FEATURE_ENABLE_READ_CACHE);
	} else {
	  AR_DEBUG(AR_DBG_INIT, (KERN_DEBUG "megaide: Read Ahead Disabled \n"));
	  ScsiPortWritePortUchar((unsigned)((UCHAR *) base1 + 1),
	                         FEATURE_DISABLE_READ_CACHE);
	}

	ScsiPortWritePortUchar((unsigned)&(base1->Command),
	                        IDE_COMMAND_SET_FEATURES);
	WAIT_ON_BASE_BUSY(base1, statusByte);

	set_multi_block_xfers(pdev, ucDrvInd);

	return AR_TRUE;
}/*Init DriveFeatures*/


BOOLEAN
set_multi_block_xfers(HW_DEVICE_EXTENSION *pdev, UCHAR tid)
{
	IDE_REGISTERS_1 *base1;
	UCHAR statusByte = 0, err;

	base1 = pdev->Channel[tid>>1].BaseIoAddress1;

	AR_DEBUG(AR_DEBUG_WARNING,
	  (KERN_DEBUG "megaide: set_multi_block_xfers TID=%d \n", tid));

	/*If supported, setup Multi-block transfers.*/
	if (pdev->PhysicalDrive[tid].MaximumBlockXfer != 0)
	{
		/*Select the device.*/
		SELECT_DEVICE(base1, tid);

		/* Setup sector count to reflect the # of blocks.*/
		ScsiPortWritePortUchar((unsigned)&base1->SectorCount,
	                    pdev->PhysicalDrive[tid].MaximumBlockXfer);

		/* Issue the command.*/
		ScsiPortWritePortUchar((unsigned)&base1->Command,
		            IDE_COMMAND_SET_MULTIPLE);

		/*Wait for busy to drop.*/
		WAIT_ON_BASE_BUSY(base1, statusByte);

		/*
		 * Check for errors. Reset the value to 0(disable MultiBlock)
		 * if the command was aborted.
		 */

		if (statusByte & IDE_STATUS_ERROR) {

		  /*Read the error register.*/
		  err = ScsiPortReadPortUchar((unsigned)((UCHAR *) base1 + 1));

		  AR_DEBUG(AR_DBG_INIT,
		    (KERN_DEBUG "megaide: Err setting mult mode.Stat %x,err %x\n",
		     statusByte, err));

		  /* Adjust the devExt. value, if necessary.*/
		  pdev->PhysicalDrive[tid].MaximumBlockXfer = 0;

		} else {
		  AR_DEBUG(AR_DBG_INIT,
		     (KERN_DEBUG "megaide: TID %d. MaximumBlockXfer - %ld\n",
		     tid, pdev->PhysicalDrive[tid].MaximumBlockXfer));
		}
	}
	
	if (pdev->ucCardType == INTEL_ICH5R) {
		UCHAR status1;
		BM_REGISTERS *baseBm = pdev->Channel[tid>>1].BaseBmAddress;
		CLEAR_BM_INT(baseBm, status1);
	}
	
	return AR_TRUE;

}/*set multi block xfers() */


void
SetDriveParameters(HW_DEVICE_EXTENSION *pdev)
{
	ULONG i;
	UCHAR statusByte = 0;
	UCHAR phy, Channel;
	IDE_REGISTERS_1 *base1;
	IDE_REGISTERS_2 *base2;
	IDENTIFY_DATA	 *idenData = NULL;
	BM_REGISTERS *baseBm;

	for (phy = 0; phy<MAX_DRIVES_PER_CONTROLLER; phy++)
	{
		if (!IS_IDE_DRIVE(pdev, phy)) {
			continue;
		}

		Channel = phy >> 1;
		base1 = (IDE_REGISTERS_1*)pdev->Channel[Channel].BaseIoAddress1;
		base2 = (IDE_REGISTERS_2*)pdev->Channel[Channel].BaseIoAddress2;
		idenData = &pdev->PhysicalDrive[phy].IdentifyData;

		/*Set up registers for SET PARAMETER command*/
		ScsiPortWritePortUchar((unsigned)&base1->DriveSelect,
			(UCHAR) (((phy<<4)|0xA0)|(idenData->NumberOfHeads-1)));

		ScsiPortWritePortUchar((unsigned)&base1->SectorCount,
			(UCHAR) idenData->SectorsPerTrack);

		/*Send SET PARAMETER command*/
		ScsiPortWritePortUchar((unsigned)&base1->Command,
			IDE_COMMAND_SET_DRIVE_PARAMETERS);

		ScsiPortStallExecution(1000);

		/*Wait for up to 30 milliseconds for ERROR or command complete*/
		for (i = 0; i < 30 * 1000; i++)
		{
		  UCHAR err;

		  statusByte = 
		     ScsiPortReadPortUchar((unsigned)&base2->AlternateStatus);

		  if (statusByte & IDE_STATUS_ERROR) {
		    
		    err = ScsiPortReadPortUchar((unsigned)((UCHAR *) base1 + 1));
		    AR_DEBUG(AR_DBG_INIT, (KERN_DEBUG 
			     "megaide:  SetDrv Err bit set. Status %x, err%x\n",
		             err, statusByte));
		    break;

		  } else if ((statusByte& ~IDE_STATUS_INDEX)==IDE_STATUS_IDLE) {
		    break;
		  } else {
		    ScsiPortStallExecution(100);
		  }
		}

		AR_DEBUG(AR_DBG_INIT,
		  (KERN_DEBUG "megaide: Status : %X\n", statusByte));
		if (pdev->ucCardType == INTEL_ICH5R) {
			baseBm = pdev->Channel[phy>>1].BaseBmAddress;
			CLEAR_BM_INT(baseBm, statusByte);
		}
	}

} /*Set Drive Parameters()*/



void
SetDriveParams(HW_DEVICE_EXTENSION *pdev,  UCHAR phy)
{
	ULONG i;
	UCHAR statusByte = 0;
	UCHAR Channel;
	IDE_REGISTERS_1 *base1;
	IDE_REGISTERS_2 *base2;
	IDENTIFY_DATA	 *idenData = NULL;


	if (!IS_IDE_DRIVE(pdev, phy)) {
		return;
	}

	Channel = phy >> 1;
	base1 = (IDE_REGISTERS_1 *) pdev->Channel[Channel].BaseIoAddress1;
	base2 = (IDE_REGISTERS_2 *) pdev->Channel[Channel].BaseIoAddress2;
	idenData = &pdev->PhysicalDrive[phy].IdentifyData;


	/*Set up registers for SET PARAMETER command.*/
	ScsiPortWritePortUchar(
		(unsigned)&base1->DriveSelect,
		(UCHAR) (((phy << 4) | 0xA0) |(idenData->NumberOfHeads - 1)));

	ScsiPortWritePortUchar((unsigned)&base1->SectorCount,
		(UCHAR) idenData->SectorsPerTrack);

	/* Send SET PARAMETER command*/
	ScsiPortWritePortUchar((unsigned)&base1->Command,
		IDE_COMMAND_SET_DRIVE_PARAMETERS);

	ScsiPortStallExecution(1000);

	/*Wait for up to 30 milliseconds for ERROR or command complete*/
	for (i = 0; i < 30 * 1000; i++)
	{
		UCHAR err;

	        statusByte =
		  ScsiPortReadPortUchar((unsigned)&base2->AlternateStatus);

		if (statusByte & IDE_STATUS_ERROR) {
		  err = ScsiPortReadPortUchar((unsigned)((UCHAR *) base1 + 1));

		  AR_DEBUG(AR_DBG_INIT, (KERN_DEBUG
		           "megaide: Error bit set1 Status %x, error %x\n",
		           err, statusByte));
			break;

		} else if ((statusByte & ~IDE_STATUS_INDEX) == IDE_STATUS_IDLE) {

			break;

		} else {

			ScsiPortStallExecution(100);
		}
	}
} /*Set Drive Params()*/



VOID 
CleanUpAdapter(HW_DEVICE_EXTENSION *pdev)
{
	UCHAR	ucControllerId;
	ULONG chan;
	ULONG log;
	ULONG phy;
	adapter_t * adapter = (adapter_t *)pdev->adapter;

	AR_DEBUG(AR_DEBUG_RELEASE, (KERN_DEBUG "megaide: CleanAdapter Called\n"));

	if (!pdev) {
		return;
	}

	ucControllerId = pdev->ucControllerId;

	/*terminate background tasks*/
	megaide_kill_backgroundtask_and_wait(pdev);

	/* Free resources */

	pdev->GateKeeperBuffer -=  (REGION_HW_BOUNDARY -
	    (pdev->GateKeeperPhysicalBuffer % REGION_HW_BOUNDARY));

	/*Dynamic DMA Implementation*/
	pci_free_consistent(adapter->pstDev, adapter->gate_keeper_buff_length,
	    pdev->GateKeeperBuffer, adapter->h_gate_keeper_buff);

	kfree(pdev->GateKeeper);
	kfree (pdev->GateScbExtension);

	for (chan = 0; chan<MAX_CHANNELS_PER_CONTROLLER; chan++)
	{
		pci_free_consistent(adapter->pstDev,
		                    (MAX_SGL_ENTRIES_PER_PHYSICAL_DRIVE *
		                                sizeof(SGL_ENTRY)),
		                    pdev->Channel[chan].pSglEntry,
		                    adapter->Channel[chan].h_sgl_entry);

		pci_free_consistent(adapter->pstDev, IDE_SECTOR_SIZE,
		                    pdev->Channel[chan].DiskConfiguration,
		                    adapter->Channel[chan].h_disc_conf);

		pci_free_consistent(adapter->pstDev, IDE_SECTOR_SIZE,
		                    pdev->Channel[chan].DiskEnquiry,
		                    adapter->Channel[chan].h_enquiry_conf);

	}

	for (log = 0; log<MAX_LOGICAL_DRIVES; log++)
	{
		kfree(pdev->LogicalDrive[log].InternalScbExtension);
		kfree(pdev->LogicalDrive[log].GateKeeper);
	}

	for (phy = 0; phy < MAX_DRIVES_PER_CONTROLLER; phy++)
	{
		pci_free_consistent(adapter->pstDev, IDE_SECTOR_SIZE,
		                    pdev->PhysicalDrive[phy].ErrorLogSector,
		                    adapter->PhysicalDrive[phy].h_err_log_sec);
	}

	pci_free_consistent(adapter->pstDev, sizeof(Scsi_Cmnd),
		            adapter->nit_scsi_cmd , adapter->h_nit_scsi_cmd);
	pci_free_consistent(adapter->pstDev, get_scb_extension_size(),
		            adapter->host_scribble , adapter->h_host_scribble);
	pci_free_consistent(adapter->pstDev,
		            (NIT_MAX_DATA_TRANSFER + CHAR_HEADER_SIZE),
		            adapter->ioctl_data, adapter->h_ioctl_data);

	if (pdev->virtBaseAddress5) {
		iounmap ((void *)pdev->virtBaseAddress5);
	}
}/*Clean UpAdapter() */



BOOLEAN
PostCommandToDrive(HW_DEVICE_EXTENSION *pdev,
                           ULONG ulTargetId, ULONG ulStartBlock,
                           ULONG ulBlockCount, UCHAR ucCmd,
                           ULONG ulSglPhysicalAddress)

{
	ULONG chan = ulTargetId >> 1;
	CHANNEL *pChannel = &(pdev->Channel[chan]);
	UCHAR statusByte, ucSectorNumber,
	      ucCylinderLow, ucCylinderHigh, bmStatus;
	UCHAR bmCommand = 0;
	IDE_REGISTERS_1 *base1 = pChannel->BaseIoAddress1;
	BM_REGISTERS *bmBase = pChannel->BaseBmAddress;
	UCHAR ucSectorNumberExt, b48bitFlag=FALSE;


#ifdef TEST_RESET_PATH
    if (ulTestResetPath)
    {
        if (ulCommandCount > 100)
        {
            ulTestResetPath = FALSE;
	    /*just fake it out to test the reset path*/
            return SRB_STATUS_PENDING;
        }
    }
#endif

	ucSectorNumber = (UCHAR) (ulStartBlock & 0x000000ff);
	ucCylinderLow = (UCHAR) ((ulStartBlock & 0x0000ff00) >> 8);
	ucCylinderHigh = (UCHAR) ((ulStartBlock & 0x00ff0000) >> 16);

	if (IS_48_BIT_ADDESSABLE(pdev, ulTargetId) &&
		(ulStartBlock > 0xFFFFFFF)) {

		b48bitFlag = TRUE;
		ucSectorNumberExt = (ulStartBlock & 0xff000000) >> 24;

		SELECT_LBA_DEVICE_48BIT(base1, ulTargetId);

		/* Previous contents */
		ScsiPortWritePortUchar((unsigned)&base1->SectorCount, ((ulBlockCount & 0xFF00)>>8));
		ScsiPortWritePortUchar((unsigned)&base1->SectorNumber,
		                        ucSectorNumberExt);
		ScsiPortWritePortUchar((unsigned)&base1->CylinderLow, 0);
		ScsiPortWritePortUchar((unsigned)&base1->CylinderHigh, 0);

		/* New contents */
		ScsiPortWritePortUchar((unsigned)&base1->SectorCount,
		                       ulBlockCount & 0xFF);
		ScsiPortWritePortUchar((unsigned)&base1->SectorNumber,
		                        ucSectorNumber);
		ScsiPortWritePortUchar((unsigned)&base1->CylinderLow,
		                        ucCylinderLow);
		ScsiPortWritePortUchar((unsigned)&base1->CylinderHigh,
		                        ucCylinderHigh);
	}
	else
	{
		SELECT_LBA_DEVICE(base1, ulTargetId, ulStartBlock);

		ScsiPortWritePortUchar((unsigned)&base1->SectorCount,
		                       (UCHAR) ulBlockCount);
		ScsiPortWritePortUchar((unsigned)&base1->SectorNumber,
		                       ucSectorNumber);
		ScsiPortWritePortUchar((unsigned)&base1->CylinderLow,
		                       ucCylinderLow);
		ScsiPortWritePortUchar((unsigned)&base1->CylinderHigh,
		                       ucCylinderHigh);
	}

	switch (ucCmd)
	{
		case READ:
		case READ_6:
		case READ_10:
		{
			/* Here no need to handle WriteBack commands cause
			 * this function is only used for BBW
			 */
			if (b48bitFlag==TRUE) {
			  ScsiPortWritePortUchar((unsigned)&base1->Command,
			                         IDE_COMMAND_READ_DMA_EXT);
			} else {
			  ScsiPortWritePortUchar((unsigned)&base1->Command,
			                         IDE_COMMAND_READ_DMA);
			}
			bmCommand = READ_TRANSFER;
			break;
		}
		case WRITE:
		case WRITE_6:
		case WRITE_10:
		{
			if (b48bitFlag==TRUE) {
			  ScsiPortWritePortUchar((unsigned)&base1->Command,
			                         IDE_COMMAND_WRITE_DMA_EXT);
			} else {
			  ScsiPortWritePortUchar((unsigned)&base1->Command,
			                         IDE_COMMAND_WRITE_DMA);
			}
			bmCommand = WRITE_TRANSFER;
			break;
		}

		case VERIFY:
		{

			/*verify command doesn't have any buffer transfers*/
			ScsiPortWritePortUchar((unsigned)&base1->Command,
			                       IDE_COMMAND_VERIFY);
			pChannel->ExpectingInterrupt = IDE_PIO_INTERRUPT;
			return TRUE;
			break;
		}
	}

	bmStatus = ScsiPortReadPortUchar((unsigned)&(bmBase->Status.AsUchar));

	/*Clear interrupt and error bits(if set)*/
	CLEAR_BM_INT(bmBase, statusByte);

	ScsiPortWritePortUlong((unsigned)&(bmBase->SglAddress),
	                       ulSglPhysicalAddress);

	/*Start transfer*/
	ScsiPortWritePortUchar((unsigned)&(bmBase->Command.AsUchar),
	                      (UCHAR) (bmCommand | START_TRANSFER));

	return TRUE;
}

HW_DEVICE_EXTENSION *
GetDeviceExtension(u_char ucIndex)
{
	if (ucIndex < giNumberOfCtrlrs) {
		adapter_t * adapter = (adapter_t *)gastCardInfo[ucIndex];
		return (&adapter->DeviceExtension);
	} else {
		return NULL;
	}
}

#ifdef BP_SUPPORT
ULONG 
megaide_get_bp_opcode(ULONG opcode)
{
	return bp_values[opcode];
}
#endif

/**
 * mega_start_ICH5R_poll_thread: Start polling thread for ICH5R
 * This function starts the polling thread that is used for detecting the 
 * drive insertion in a ICH5R controller. If the thread is already running 
 * function will not try to create the thread.
 * 
 * Parameters
 * @pdev	: Driver data structure pointer
 * @return	: void
 **/
void mega_start_ICH5R_poll_thread(HW_DEVICE_EXTENSION *pdev)
{

	adapter_t	* adapter = (adapter_t *)pdev->adapter;
	
	/* check if the thread is already running*/
	if (pdev->ICH5R_thread_flag) {
		return;
	}
	
	/*Initializing completion handler. to detect the thread completion*/
	init_completion(&adapter->ICH5R_completion);
	
	pdev->ICH5R_thread_pid = 
		kernel_thread((int (*)(void *))mega_ICH5R_poll_thread, pdev, 0);
}

/**
 * mega_kill_ICH5R_poll_thread: Terminate ICH5R polling thread.
 * This function terminate the polling thread that is used for detecting the 
 * drive insertion in a ICH5R controller. Function will actually wait till the 
 * thread stop
 * Parameters
 * @pdev	: Driver data structure pointer
 * @return	: void
 **/
void mega_kill_ICH5R_poll_thread(HW_DEVICE_EXTENSION *pdev)
{
	adapter_t	* adapter = (adapter_t *)pdev->adapter;
	
	if (pdev->ICH5R_thread_flag) {
		kill_proc(pdev->ICH5R_thread_pid, SIGKILL, 0);
		wait_for_completion(&adapter->ICH5R_completion);
	}
}

/**
 * mega_ICH5R_poll_thread: ICH5R polling thread
 * This is a kernel thread that detects drive insertion in ICH5R. Thread starts
 * as soon as the driver loads and it will stop only when driver unloads or 
 * when system shut down
 * 
 * Polling time interval of this thread is 2 Sec
 *
 * Thread will detect only the drive insertion on a channel where drive was 
 * not connected previously and it sets the respective flags for the same.
 * MegaIDE driver's drive insertion removal module will process this event
 * later.
 *
 * Access of the generic data structure in this thread should be SMP proof to
 * make sure a smooth operation in an SMP system.
 *
 * Parameters
 * @pdev	: Driver data structure pointer
 * @return	: void
 **/
void mega_ICH5R_poll_thread(HW_DEVICE_EXTENSION *pdev)
{
	UCHAR			mask;
	UCHAR			status;
	unsigned long		flags;
	struct Scsi_Host	*host;
	wait_queue_head_t	wait_queue_ICH5R;
	UCHAR			dissable_flag = 0;	
	host = ((adapter_t *)pdev->adapter)->pstScsiHost;

	/* Check if thread already active/running?*/
	if (pdev->ICH5R_thread_flag)
		return;
	
      
	/*
         * We only listen to signals if the HA was loaded as a module.
         * If the HA was compiled into the kernel, then we don't listen
         * to any signals.
         */
        if( host->loaded_as_module ) {
		siginitsetinv(&current->blocked, SHUTDOWN_SIGS);
	} else {
		siginitsetinv(&current->blocked, 0);
        }
	
	/* Flush resources */
	daemonize();
	reparent_to_init();

	/* Set the name of this process. */
	sprintf(current->comm, "megaide");
	
	pdev->ICH5R_thread_flag = 1;
	
	init_waitqueue_head(&wait_queue_ICH5R);

	while(1)
	{
		
		schedule();

		/* waiting for 2 sec, polling time is 2 Sec */
		interruptible_sleep_on_timeout(&wait_queue_ICH5R, (HZ*2));

		/* check if thread termination received? */
		if( host->loaded_as_module ) {
			if(signal_pending(current))
				break;
		}
		
		/*
		 * Check if both drives are present?. If yes terminate thread
		 * Disable/enable channel with out drive.
		 */
 
		mask = 0;
		
		if (!pdev->PhysicalDrive[0].ucIsPowerConnected) 
			mask |= 0x01;
		
		if (!pdev->PhysicalDrive[2].ucIsPowerConnected)
			mask |= 0x02;
			
		if (!mask || pdev->drive_inserted.u.Drives) {
			continue;
		}
		spin_lock_irqsave(&io_request_lock, flags);
		
		/* reading port control and status register */
		pci_read_config_byte_Rlsi(pdev, 0x92, &status);
		if (dissable_flag == 0) {
			status ^= mask;
			pci_write_config_byte_Rlsi(pdev, 0x92, status);
			dissable_flag = 1;
			AR_DEBUG(AR_HOT_PLUG, (KERN_DEBUG 
				"megaide: dissabled chan mask= 0x%x\n", mask));
			spin_unlock_irqrestore(&io_request_lock, flags);
			continue;
		} else if (dissable_flag == 1) {
			status |= mask;
			pci_write_config_byte_Rlsi(pdev, 0x92, status);
			dissable_flag = 2;
			AR_DEBUG(AR_HOT_PLUG, (KERN_DEBUG 
				"megaide: enabled chan mask= 0x%x\n", mask));
			spin_unlock_irqrestore(&io_request_lock, flags);
			continue;
		} else {
			dissable_flag = 0;
			AR_DEBUG(AR_HOT_PLUG, (KERN_DEBUG 
				"megaide: going to detect status= 0x%x\n", status));
		}
		
		/* Check only for the drive which is not present. */	 
		if (status & 0x10) {
			if (!pdev->PhysicalDrive[0].ucIsPowerConnected) {
				pdev->drive_inserted.u.Drives |= 1;
				AR_DEBUG(AR_HOT_PLUG, (KERN_DEBUG 
					"megaide: drv0 inserted! stat=0x%x\n",
					status));
			}
		}

		if (status & 0x20) {
			if (!pdev->PhysicalDrive[2].ucIsPowerConnected) {
				pdev->drive_inserted.u.Drives |= (1 <<2);
				AR_DEBUG(AR_HOT_PLUG, (KERN_DEBUG 
					"megaide: drv2 inserted! stat=0x%x\n",
					status));
			}
		} 
				
		spin_unlock_irqrestore(&io_request_lock, flags);
	}
	
	/*
	 * If anyone is waiting for me to let them know I am on way out!
	 */
	pdev->ICH5R_thread_flag = 0;
	complete_and_exit(&((adapter_t *)pdev->adapter)->ICH5R_completion, 0);
}

#include    "megaide_scsi.c"
#include    "megaide_ata.c"
#include    "megaide_char.c"
#include    "megaide_proc.c"
#include    "megaide_kernel_lib.c"

/* To make driver a SCSI module */
#ifdef MODULE
	Scsi_Host_Template driver_template = MEGAIDE;
	#include "../scsi_module.c"
#endif/*MODULE*/

/*vi set ts=8:*/
