/*megaide_proc.c*/
/*
 * Linux MegaRAID IDEal Software RAID Stack
 * Copyright (c) 2001 LSI Logic Corporation, All Rights Reserved.
 *-----------------------------------------------------------------------------
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *-----------------------------------------------------------------------------
 */

#define AR_PDRV_SET_FAILED "set status failed"
#define AR_PDRV_SET_HOTSPARE "set status hotspare"
#define AR_PDRV_INIT_ERRLOG "init errorlog"
#define AR_PDRV_GEN_ERROR "generate error"

int 
megaide_proc_info(char *pcBuffer, char **ppcStart, off_t offset,
                      int iLength, int iHostNo, int iInOut)
{
	AR_DEBUG(AR_DBG_ENTRY, (KERN_DEBUG "megaide_proc _info Called\n"));
	*ppcStart = pcBuffer;
	return 0;
}/*megaide_proc_info()*/

#ifdef CONFIG_PROC_FS
extern struct proc_dir_entry proc_root;
static struct proc_dir_entry *pstMegaProcDirEntry;

void
megaideproc_attach( void )
{
	int iCount;

	pstMegaProcDirEntry = proc_mkdir("megaide", &proc_root);

	pstMegaProcDirEntry = create_proc_entry("megaide",
	                                        S_IFDIR | S_IRUGO |
	                                        S_IXUGO, &proc_root);

	if (!pstMegaProcDirEntry) {
		printk_Rlsi(KERN_WARNING "megaide: failed to create root\n");
	} else {
		for (iCount = 0; iCount < giNumberOfCtrlrs; iCount++)
		{
			megaide_create_proc_entry(iCount, pstMegaProcDirEntry);
		}
	}
}

void
megaideproc_detach( HW_DEVICE_EXTENSION *pdev )
{
	int iCount;
	adapter_t *adapter;

	for (iCount = 0; iCount < giNumberOfCtrlrs; iCount++)
	{
		char acBuf[12] = { 0 };
		sprintf(acBuf, "%d", iCount);
		adapter = (adapter_t *)gastCardInfo[iCount];
		if( adapter->pstControllerProcDirEntry ) {
			remove_proc_entry(acBuf, pstMegaProcDirEntry);
			adapter->pstControllerProcDirEntry=NULL;
		}
	}
	remove_proc_entry("megaide", &proc_root);
}

static char *Ldrv_Raid_Str[5] = {
        "NONE   ",
        "RAID 0 ",
        "RAID 1 ",
        "RAID 10",
        "NOT PRESENT" };

static char *Ldrv_Status_Str[5] = {
       "ONLINE  ",
       "CRITICAL",
       "DEGRADED",
       "OFFLINE ",
       "NOT PRESENT" };

static char *Yes_No_Str[2] = { "NO ", "YES" };

static char *Pdrv_Id_Str[5] = {
       "PRIMARY MASTER  ",
       "PRIMARY SLAVE   ",
       "SECONDARY MASTER",
       "SECONDARY SLAVE ",
       "NOT PRESENT     " };

/*This will write the debug informations in to the proc buffer
 *in device extention structure
 */
int
mega_proc_printf(HW_DEVICE_EXTENSION *pdev, const char *pcFmt, ...)
{
	va_list args;
	int 	iCount;

	if (pdev->iProcIdx > PROCBUFSIZE){
		return 0;
	}
	va_start(args, pcFmt);
	iCount = vsprintf((pdev->achProcBuf + pdev->iProcIdx), pcFmt, args);
	va_end(args);
	pdev->iProcIdx += iCount;

	return iCount;
}/*mega_proc_printf()*/


int
mega_proc_read_config(char *page, char **start, off_t offset,
                      int iCount, int *eof, void *data)
{

	int i, count = 0;
	HW_DEVICE_EXTENSION *pdev = (HW_DEVICE_EXTENSION *)data;
	
	*start = page;

	mega_proc_printf(pdev, "Driver Version         : %d.%d.%d \t\n",
		HYPERDISK_MAJOR_VERSION,
		HYPERDISK_MINOR_VERSION,
		HYPERDISK_BUILD_VERSION + 'a' - 1);
	
	if ((pdev->stFirmwareVersion.usMajorVer == 0) &&
	    (pdev->stFirmwareVersion.usMinorVer == 0) &&
	    (pdev->stFirmwareVersion.ulBuild == 0)) {
		mega_proc_printf(pdev, "Firmware               : Disabled\n");
	} else {
		mega_proc_printf(pdev, "Firmware Revision      : %d.%d.%d\n",
			pdev->stFirmwareVersion.usMajorVer,
			pdev->stFirmwareVersion.usMinorVer,
			pdev->stFirmwareVersion.ulBuild);
	}
	mega_proc_printf(pdev, "IRCD Version           : %c.%c \t\n",
		pdev->stIRCDVersion.usMajorVer,
		pdev->stIRCDVersion.usMinorVer);
	mega_proc_printf(pdev,
	   "Primary Base Address   : %8xH\tPrimary Control Address   : %8xH\n",
	   pdev->Channel[0].BaseIoAddress1,
	   pdev->Channel[0].BaseIoAddress2);
	mega_proc_printf(pdev,
	   "Secondary Base Address : %8xH\tSecondary Control Address : %8xH\n",
	   pdev->Channel[1].BaseIoAddress1,
	   pdev->Channel[1].BaseIoAddress2);
	mega_proc_printf(pdev,
	   "BusMaster Base Address : %8xH\tIRQ                       : %8d\n",
	   pdev->Channel[0].BaseBmAddress, pdev->ulIntLine );
	mega_proc_printf(pdev,
	   "PCI VendorId           : %8xH\tPCI DeviceId              : %8xH\n",
	   pdev->ulVendorId, pdev->ulDeviceId);
	mega_proc_printf(pdev,
	   "PCI SubSytem VendorId  : %8xH\tPCI SubSytem DeviceId     : %8xH\n",
	   pdev->usSubsysVendorId,
	   pdev->usSubsysDeviceId);
        mega_proc_printf(pdev,
	   "PCI Bus Number         : %8d \tPCI Device Number         : %8d\n",
	   pdev->ucPCIBus, pdev->ucPCIDev );
	mega_proc_printf(pdev,
	   "PCI Function           : %8d \n",
	   pdev->ucPCIFun );
	mega_proc_printf(pdev, "# Logical Drives       : %8d \t",
	   pdev->ulLogicalDriveCount);

	/*Changed the logic to find Number of drives*/
	for(i = 0; i < MAX_DRIVES_PER_CONTROLLER; i++)
	{
		if (pdev->PhysicalDrive[i].DeviceFlags&DFLAGS_DEVICE_PRESENT) {
			count++;
		}
	}

	mega_proc_printf(pdev,"# Physical Drives         : %8d\n\n",
	        count);

	mega_proc_printf(pdev, "Driver Options:\n");
	mega_proc_printf(pdev,
	  "Polling Interval       : %8u \tError Threshold          : %8u\n",
	  pdev->Options.ulErrorPolling, pdev->Options.ulErrorThreshold);
	mega_proc_printf(pdev,
	  "Fail Drive             : %8d \tAuto Rebuild             : %8d\n",
	  pdev->Options.ulFailDrive, pdev->Options.ulAutoRebuild);
	mega_proc_printf(pdev,
	  "Check Array            : %8d \tStop Rebuild             : %8d\n",
	  pdev->Options.ulCheckArray, pdev->Options.ulStopRebuild);
	mega_proc_printf(pdev,
	  "Identical              : %8d \tRebuild Rate             : %8d\n",
	  pdev->Options.ulIdentical, pdev->Options.ulRebuildRate*10);

	COPY_BACK(pdev, offset, iCount);
	return iCount;
}/*mega_proc_read_config*/

int 
string_to_integers(const char *str, ULONG *pulValues, int max_count)
{
	ULONG i = 0, ulValCount = 0, k = 0, len = 0, count = 0;
	char tmp[20];

	len = strlen(str);
	memset_Rlsi(tmp, 0, sizeof(tmp));
	for(i = 0; i < len; i++)
	{
		if ((str[i] == ' ') ||(str[i] == '\n')) {
			for(count = 0; count < (strlen(tmp)); count++)
			{
				if (!(isdigit(tmp[count]))) {
					return -1;
				}
			}
			*(pulValues + ulValCount) = mide_array_to_int(tmp);
			ulValCount++;

			while(str[i] == ' ')
			{
			   i++;
			}

			memset_Rlsi(tmp, 0, sizeof(tmp));
			k = 0;
		}
		tmp[k++] = str[i];

		if (ulValCount >= max_count) {
			break;
		}
	}
	return ulValCount;
}/*string_to_integers*/

void 
pdrv_status_to_string(MP_PHYSICAL_DRIVE_STATUS eStatus, char *cString)
{
 switch (eStatus)
 {
	case PDS_Online :
		sprintf(cString, "ONLINE    ");
		break;
	case PDS_Critical :
		sprintf(cString, "CRITICAL  ");
		break;
	case PDS_Failed :
		sprintf(cString, "FAILED    ");
		break;
	case PDS_HotSpare :
		sprintf(cString, "HOT SPARE ");
		break;
	case PDS_Rebuilding :
		sprintf(cString, "REBUILDING");
		break;
	case PDS_OffLine :
		sprintf(cString, "OFFLINE   ");
		break;
	default :
		sprintf(cString, "UNKNOWN   ");
		break;
 }
}/*pdrv_status_to_string*/

int
mega_proc_read_pdrv_info(char *page, char **start, off_t offset,
                         int iCount, int *eof, void *data)
{
	PHYSICAL_DRIVE_INFO *pPdrv = (PHYSICAL_DRIVE_INFO *) data;
	HW_DEVICE_EXTENSION *pdev;
	char tmode[8], status[12];
	char cModelNo[PHYSICAL_DRIVE_MODEL_LENGTH + 1];
	char cSerialNo[PHYSICAL_DRIVE_SERIAL_NO_LENGTH + 1];
        char cFirmwareRev[PHYSICAL_DRIVE_FIRMWARE_REV_LENGTH + 1];
	UCHAR ucIs80PinCable, i;
	adapter_t *adapter;

	*start = page;

	adapter = (adapter_t *)gastCardInfo[pPdrv->ucControllerId];
	pdev = &adapter->DeviceExtension;

	if (!(pPdrv->DeviceFlags & DFLAGS_DEVICE_PRESENT)) {
		mega_proc_printf(pdev, "Drive is Not Present\n");
		COPY_BACK(pdev, offset, iCount);
		return iCount;
	}

	ucIs80PinCable = Is80PinCable(pdev, pPdrv->PhysicalDriveId>>1);
	pdrv_status_to_string(pPdrv->Status, status);

	switch (pPdrv->TransferMode)
	{
	   case MP_PioMode0 :
	   case MP_PioMode1 :
	   case MP_PioMode2 :
	   case MP_PioMode3 :
	   case MP_PioMode4 :
		sprintf(tmode, "PIO(%d)", pPdrv->TransferMode);
		break;

	   case MP_DmaMode0 :
	   case MP_DmaMode1 :
	   case MP_DmaMode2 :
		sprintf(tmode, "DMA(%d)", pPdrv->TransferMode - 5);
		break;

	   case MP_UdmaMode0 :
	   case MP_UdmaMode1 :
	   case MP_UdmaMode2 :
	   case MP_UdmaMode3 :
	   case MP_UdmaMode4 :
	   case MP_UdmaMode5 :
		sprintf(tmode, "UDMA(%d)", pPdrv->TransferMode - 8);
		break;

	   default :
		sprintf(tmode, "UNKNOWN");
		break;

	}

	for(i = 0; i < 40; i += 2)
	{
	   cModelNo[i] = pPdrv->IdentifyData.ModelNumber[i + 1];
	   cModelNo[i + 1] = pPdrv->IdentifyData.ModelNumber[i];
	}
	cModelNo[40] = '\0';

	for(i = 0; i < 20; i += 2)
	{
	   cSerialNo[i] = pPdrv->IdentifyData.SerialNumber[i + 1];
	   cSerialNo[i + 1] = pPdrv->IdentifyData.SerialNumber[i];
	}
	cSerialNo[20] = '\0';

	for(i = 0; i < 8; i += 2)
	{
	   cFirmwareRev[i] = pPdrv->IdentifyData.FirmwareRevision[i + 1];
	   cFirmwareRev[i + 1] = pPdrv->IdentifyData.FirmwareRevision[i];
	}
	cFirmwareRev[8] = '\0';

	mega_proc_printf(pdev, "Model No : %s\n", cModelNo);
	mega_proc_printf(pdev, "Serial No : %s\n", cSerialNo);
	mega_proc_printf(pdev, "Firmware Revision : %s\n", cFirmwareRev);
	mega_proc_printf(pdev, "Drive Id : %s\tTransfer Mode : %s\n",
	                 Pdrv_Id_Str[pPdrv->PhysicalDriveId], tmode);
	mega_proc_printf(pdev, "Status : %s\t\tNo Of Drive Errors : %d\n",
	                 status, pPdrv->ErrorFound);
	mega_proc_printf(pdev, "Capacity : %d\t\t80 pin Cable : %s\n",
	                 pPdrv->BiosExposedSectors, Yes_No_Str[ucIs80PinCable]);
	mega_proc_printf(pdev, "Smart Compliant : %s\t\tSmart Enabled : %s\n",
	                 Yes_No_Str[pPdrv->IdentifyData.CmdSupported1 & 1],
	                 Yes_No_Str[pPdrv->IdentifyData.CmdEnabled1 & 1]);

	COPY_BACK(pdev, offset, iCount);
	return iCount;
}/*mega_proc_read_pdrv_info*/

int 
mega_proc_read_registers(char *page, char **start, off_t offset,
                         int iCount, int *eof, void *data)
{
	HW_DEVICE_EXTENSION *pdev = (HW_DEVICE_EXTENSION *) data;
	UCHAR ucCfr, ucArttim0, ucArttim1, ucDrwtim0, ucDrwtim1;
	UCHAR ucArttim2, ucDrwtim2, ucDrwtim3, ucMrdmode;
	UCHAR ucBmidesr0, ucBmidesr1, ucUdidetcr0, ucUdidetcr1;

	*start = page;

	pci_read_config_byte_Rlsi(pdev, 0x50, &ucCfr);
	pci_read_config_byte_Rlsi(pdev, 0x53, &ucArttim0);
	pci_read_config_byte_Rlsi(pdev, 0x54, &ucDrwtim0);
	pci_read_config_byte_Rlsi(pdev, 0x55, &ucArttim1);
	pci_read_config_byte_Rlsi(pdev, 0x56, &ucDrwtim1);
	pci_read_config_byte_Rlsi(pdev, 0x57, &ucArttim2);
	pci_read_config_byte_Rlsi(pdev, 0x58, &ucDrwtim2);
	pci_read_config_byte_Rlsi(pdev, 0x5b, &ucDrwtim3);
	pci_read_config_byte_Rlsi(pdev, 0x71, &ucMrdmode);
	pci_read_config_byte_Rlsi(pdev, 0x72, &ucBmidesr0);
	pci_read_config_byte_Rlsi(pdev, 0x73, &ucUdidetcr0);
	pci_read_config_byte_Rlsi(pdev, 0x7a, &ucBmidesr1);
	pci_read_config_byte_Rlsi(pdev, 0x7b, &ucUdidetcr1);

	mega_proc_printf(pdev, "PCI Register Information\n");

	mega_proc_printf(pdev,
	                 "CFR      : %x\tARTTIM0   : %x\tDRWTIM0  : %x\n",
	                  ucCfr, ucArttim0, ucDrwtim0);
	mega_proc_printf(pdev,
	                 "ARTTIM1  : %x\tDRWTIM1   : %x\tARTTIM2  : %x\n",
	                 ucArttim1, ucDrwtim1, ucArttim2);
	mega_proc_printf(pdev,
	                 "DRWTIM2  : %x\tDRWTIM3   : %x\tMRDMODE  : %x\n",
	                 ucDrwtim2, ucDrwtim3, ucMrdmode);
	mega_proc_printf(pdev,
	                 "BMIDESR0 : %x\tUDIDETCR0 : %x\tBMIDESR1 : %x\n",
	                 ucBmidesr0, ucUdidetcr0, ucBmidesr1);
	mega_proc_printf(pdev, "UDIDETCR1 : %x\n", ucUdidetcr1);

	COPY_BACK(pdev, offset, iCount);
	return iCount;
}/*mega_proc_read_registers()*/

int 
mega_proc_read_status(char *page, char **start, off_t offset,
                      int iCount, int *eof, void *data)
{
	HW_DEVICE_EXTENSION *pdev = (HW_DEVICE_EXTENSION *) data;
	LOGICAL_DRIVE_INFO *pLdrv = NULL;
	int i;
	char cStatus1[12];
	char cStatus2[12], cStatus3[12], cStatus4[12];

	*start = page;

	mega_proc_printf(pdev, "No Of Pending SCB's in Controller : %d\n",
	                 pdev->NumberOfPendingScb);
	mega_proc_printf(pdev, "No Of Pending Commands in Controller : %d\n\n",
	                 pdev->NumberOfPendingCmd);

	mega_proc_printf(pdev, "Logical Drive Information\n");

	for(i = 0; i < pdev->ulLogicalDriveCount; i++)
	{
		pLdrv = &pdev->LogicalDrive[i];
		mega_proc_printf(pdev, "\tStatus : %s\t\t",
		Ldrv_Status_Str [pLdrv->Status]);
		mega_proc_printf(pdev, "Logical Drive No: %d\n", i);
	}

	mega_proc_printf(pdev, "\nPhysical Drive Information\n");
	mega_proc_printf(pdev,
	       "\t\t\t------  PRIMARY  ------\t\t-----  SECONDARY  -----\n");
	mega_proc_printf(pdev, "\t\t\tMASTER\t\tSLAVE\t\tMASTER\t\tSLAVE\n");
	pdrv_status_to_string(pdev->PhysicalDrive[0].Status, cStatus1);
	pdrv_status_to_string(pdev->PhysicalDrive[1].Status, cStatus2);
	pdrv_status_to_string(pdev->PhysicalDrive[2].Status, cStatus3);
	pdrv_status_to_string(pdev->PhysicalDrive[3].Status, cStatus4);
	mega_proc_printf(pdev, "STATUS : \t\t%s\t%s\t%s\t%s\n",
	                 cStatus1, cStatus2, cStatus3, cStatus4);
	mega_proc_printf(pdev, "No Of Errors : \t\t %8d\t%8d\t%8d\t%8d\n",
	                 pdev->PhysicalDrive[0].ErrorFound,
	                 pdev->PhysicalDrive[1].ErrorFound,
	                 pdev->PhysicalDrive[2].ErrorFound,
	                 pdev->PhysicalDrive[3].ErrorFound);
	mega_proc_printf(pdev, "No Of Pending Commands : %8d\t%8d\t%8d\t%8d\n",
	                pdev->PhysicalDrive[0].CurrentQueueLength,
	                pdev->PhysicalDrive[1].CurrentQueueLength,
	                pdev->PhysicalDrive[2].CurrentQueueLength,
	                pdev->PhysicalDrive[3].CurrentQueueLength);

#ifdef PERFORMANCE_NUMBER
{
	int i, len=DBG_TIME_INTERVAL, len1 =1, len2= DBG_TIME_INTERVAL;
	ULONG io_2_cmd0=0, io_2_cmd1=0, cmd_2_isr0=0, cmd_2_isr1=0, io_2_isr=0;

	mega_proc_printf(pdev, "Total Commands   = %8d\n", pdev->io_count);
	mega_proc_printf(pdev, "Drive1 Total[%5d]                 Drive2 Total[%5d]\n", pdev->int_count[0], pdev->int_count[1]);
	mega_proc_printf(pdev, "io-2-cmd          cmd-2-isr           io-2-cmd          cmd-2-isr         IO [queue to ISR]\n");

	for (i=0; i<26; i++)
	{
		mega_proc_printf(pdev, "[%3d-%3d] =%5d  [%3d-%3d] =%5d    [%3d-%3d] =%5d  [%3d-%3d] =%5d  [%3d-%3d] =%5d\n",
			(i*len), ((i+1)*len), pdev->io_2_pcmd[0][i], (i*len1), ((i+1)*len1), pdev->pcmd_2_isr[0][i],
			(i*len), ((i+1)*len), pdev->io_2_pcmd[1][i], (i*len1), ((i+1)*len1), pdev->pcmd_2_isr[1][i],
			(i*len2), ((i+1)*len2), pdev->scbduration[i]);
			io_2_cmd0 += (pdev->io_2_pcmd[0][i] * ((i*len)+((i+1)*len))) / 2;
			io_2_cmd1 += (pdev->io_2_pcmd[1][i] * ((i*len)+((i+1)*len))) / 2;
			cmd_2_isr0 += (pdev->pcmd_2_isr[0][i] * ((i*len1)+((i+1)*len1))) /2;
			cmd_2_isr1 += (pdev->pcmd_2_isr[1][i] * ((i*len1)+((i+1)*len1))) /2;
			io_2_isr += (pdev->scbduration[i]* ((i*len2)+((i+1)*len2)))/2;
	}

	io_2_cmd0 = (io_2_cmd0/(pdev->int_count[0]+1));
	cmd_2_isr0 = (cmd_2_isr0 /(pdev->int_count[0]+1));
	io_2_cmd1 = (io_2_cmd1 /(pdev->int_count[1]+1));
	cmd_2_isr1 = (cmd_2_isr1 /(pdev->int_count[1]+1));
	io_2_isr = (io_2_isr / (pdev->io_count+1));

	mega_proc_printf(pdev, "Average-->   %3d               %3d                 %3d               %3d               %3d\n",
		io_2_cmd0, cmd_2_isr0, io_2_cmd1, cmd_2_isr1, io_2_isr);
}
#endif //PERFORMANCE_NUMBER

	COPY_BACK(pdev, offset, iCount);
	return iCount;
}/*mega_proc_read_status*/

int
mega_proc_read_ldrv_info(char *page, char **start, off_t offset,
                         int iCount, int *eof, void *data)
{
	LOGICAL_DRIVE_INFO *pLdrv = (LOGICAL_DRIVE_INFO *) data;
	HW_DEVICE_EXTENSION *pdev;
	adapter_t *adapter;

	*start = page;

	adapter = (adapter_t *)gastCardInfo[pLdrv->ucControllerId];
	pdev = &adapter->DeviceExtension;

	mega_proc_printf(pdev, "RAID Level : %s\t\tStatus : %s\n",
	                 Ldrv_Raid_Str[pLdrv->RaidLevel],
	                 Ldrv_Status_Str[pLdrv->Status]);
	if (pLdrv->RaidLevel == Raid1) {
		mega_proc_printf(pdev, "Sectors : %d\t\tStripe Size : N/A\n",
		                 pLdrv->Sectors);
	} else {
		mega_proc_printf(pdev, "Sectors : %d\t\tStripe Size : %dKB\n",
		                 pLdrv->Sectors, pLdrv->StripeSize/2);
	}
	mega_proc_printf(pdev, 
	                 "Stripes Per Row : %d\t\tPhysical Drive Count : %d\n",
	                 pLdrv->StripesPerRow, pLdrv->PhysicalDriveCount);

	COPY_BACK(pdev, offset, iCount);
	return iCount;
}/*mega_proc_read_ldrv_info*/

void
create_logical_drive_entries(struct proc_dir_entry *pstParent,
                             HW_DEVICE_EXTENSION *pdev)
{
	UCHAR achString[12] = { 0 }, i, j;
	struct proc_dir_entry *pstLogicalDrives = NULL;
	struct proc_dir_entry *pstSymLink = NULL;
	struct proc_dir_entry *pstLdrvNo = NULL, *pstLdrvInfo = NULL;
	char cPath[30], cDrvNo[18];
	LOGICAL_DRIVE_INFO *pLdrv = NULL;

        pstLogicalDrives = proc_mkdir("logicaldrives", pstParent);
	if (!pstLogicalDrives){
		printk_Rlsi(KERN_WARNING"megaide: proc_mkdir failed\n");
	} else {
	for(i = 0; i < pdev->ulLogicalDriveCount; i++)
	{
		sprintf(achString, "%d", i);
		pstLdrvNo =  proc_mkdir(achString, pstLogicalDrives);
		if (!pstLdrvNo){
		  printk_Rlsi(KERN_WARNING "megaide: proc_mkdir failed\n");
		} else {
		  pLdrv = &pdev->LogicalDrive[i];

		  for(j = 0; j < pLdrv->PhysicalDriveCount; j++)
		  {
		    if (INVALID_DRIVE_ID == pLdrv->PhysicalDriveTid[j]) {
		      continue;
		    }

    if (!(pdev->PhysicalDrive[pLdrv->PhysicalDriveTid[j]].DeviceFlags &
          DFLAGS_DEVICE_PRESENT)) {
      continue;
    }
		    memset_Rlsi(cDrvNo, 0, sizeof(cDrvNo));
		    sprintf(cDrvNo, "drive-%d", j);
		    sprintf(cPath, "/proc/megaide/%d/physicaldrives/%d",
		            pdev->ucControllerId,
		            pLdrv->PhysicalDriveTid[j]);

		    pstSymLink = proc_symlink(cDrvNo, pstLdrvNo, cPath);
		    if (!pstSymLink) {
		      printk_Rlsi(KERN_WARNING "megaide:proc_symlink failed\n");
		    }
		  }

		  pstLdrvInfo = create_proc_entry("info", S_IRUSR|S_IFREG,
		                                  pstLdrvNo);

		  if (!pstLdrvInfo) {
		    printk_Rlsi(KERN_WARNING "megaide:create_proc_entry failed\n");
		  } else {
		    pstLdrvInfo->nlink = 1;
		    pstLdrvInfo->read_proc = mega_proc_read_ldrv_info;
		    pstLdrvInfo->write_proc = NULL;
		    pstLdrvInfo->data = &pdev->LogicalDrive[i];
		  }
		}
	} /*for loop*/
	} /*else*/
}/* create_logical_drive_entries */

void
create_physical_drive_entries(struct proc_dir_entry *pstParent,
                              HW_DEVICE_EXTENSION *pdev)
{
	UCHAR achString[12] = { 0 }, i;
	struct proc_dir_entry *pstPhysicalDrives = NULL;
	struct proc_dir_entry *pstPdrvNo = NULL, *pstPdrvInfo = NULL;

        pstPhysicalDrives =  proc_mkdir("physicaldrives", pstParent);
	if (!pstPhysicalDrives) {
		printk_Rlsi(KERN_WARNING "megaide: proc_mkdir failed\n");
	}else {
	for(i = 0; i < MAX_DRIVES_PER_CONTROLLER; i++)
	{
		sprintf(achString, "%d", i);
		pstPdrvNo =  proc_mkdir(achString, pstPhysicalDrives);
		if (!pstPdrvNo) {
			printk_Rlsi(KERN_WARNING "megaide: proc_mkdir failed\n");
		} else {
			pstPdrvInfo = create_proc_entry("info", S_IRUSR|S_IFREG,
			                                pstPdrvNo);

			if (!pstPdrvInfo){
			  printk_Rlsi(KERN_WARNING
			             "megaide: create_proc_entry failed\n");
			} else {
			  pstPdrvInfo->nlink = 1;
			  pstPdrvInfo->read_proc = mega_proc_read_pdrv_info;
			  pstPdrvInfo->write_proc = NULL;
			  pstPdrvInfo->data = &pdev->PhysicalDrive[i];
			}
		}
	}
	}
}/* create_physical_drive_entries */

typedef struct
{
        mode_t mode;
        char *entry_name;
        read_proc_t *proc_reader;
        write_proc_t *proc_write;
        struct proc_dir_entry *entry_parent;
        struct proc_dir_entry *entry;
} std_procentry_t;

static std_procentry_t proc_subcard_entries[] = {
        {S_IRUSR|S_IFREG, "config", mega_proc_read_config,
        NULL, NULL, NULL},
        {S_IRUSR|S_IFREG, "status", mega_proc_read_status,
        NULL, NULL, NULL},
        {S_IRUSR|S_IFREG, "registers", mega_proc_read_registers,
        NULL, NULL, NULL},
        {(mode_t)0, NULL, NULL, NULL, NULL, NULL}
};

void megaide_create_proc_entry(int iIndex, struct proc_dir_entry *pstParent)
{
	adapter_t *adapter;
	UCHAR achString[64] = { 0 };
	HW_DEVICE_EXTENSION *pdev;
	struct proc_dir_entry *pstContrlrProcDirEntryTemp = NULL;
	std_procentry_t *p;

	adapter = (adapter_t *)gastCardInfo[iIndex];
	pdev = &adapter->DeviceExtension;

	sprintf(achString, "%d", iIndex);

	pstContrlrProcDirEntryTemp =
	    adapter->pstControllerProcDirEntry =
	        proc_mkdir(achString, pstParent);

	if (!pstContrlrProcDirEntryTemp) {
	  printk_Rlsi(KERN_WARNING "megaide: proc_mkdir failed\n");
	} else {
	  for(p = proc_subcard_entries; p->entry_name; p++)
	  {
	    p->entry_parent = adapter->pstControllerProcDirEntry;
	    p->entry =
	       create_proc_entry(p->entry_name, p->mode,
	                         adapter->pstControllerProcDirEntry);
	    if (p->entry) {
	      p->entry->nlink = 1;
	      p->entry->read_proc = p->proc_reader;
	      p->entry->write_proc = p->proc_write;
	      p->entry->data = pdev;
	    }
	  }
	  create_physical_drive_entries(adapter->pstControllerProcDirEntry,
	                                pdev);
	  create_logical_drive_entries(adapter->pstControllerProcDirEntry,
	                               pdev);
	}
}/*megaide_create_proc_entry()*/

#endif	/* CONFIG_PROC_FS */
/*vi set ts=8:*/

