/*megaide_scsi.c*/
/*
 * Linux MegaRAID IDEal Software RAID Stack
 * Copyright (c) 2001 LSI Logic Corporation, All Rights Reserved.
 *-----------------------------------------------------------------------------
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *-----------------------------------------------------------------------------
 */

char OemProductId[16] = " LD XY  IDERAID ";
char OemVendorId[8]  = "LSILOGIC";
volatile static int giInternalDoneFlag = 0;
volatile static int giInternalDoneErrcode = 0;

static struct notifier_block megaide_reboot_notifier = {
	notifier_call : megaide_reboot_notify
};

/**
 * Description
 *	Driver detect entry point for the mid-layer,
 *	We sacn the PCI bus for our controller and start them
 * Return Value
 *	Number of adapter found
 */
int
megaide_detect(Scsi_Host_Template *hostTpl)
{
	int adp = 0;

	AR_DEBUG(AR_DBG_ENTRY, (KERN_DEBUG "megaide_detect called\n"));

	hostTpl->proc_name = "megaide";

	/*Check if driver support PCI*/
	if (!pci_present()) {
		printk_Rlsi(KERN_WARNING "megaide: PCI bios not present.");
		return 0;
	}

	printk_Rlsi(KERN_INFO "megaide: " MEGAIDE_VERSION M_IDE_CRLFSTR);

	adp += mide_find_card(hostTpl, PCI_VENDOR_ID_CMD,
	                      PCI_DEVICE_ID_CMD_648, 0);
	adp += mide_find_card(hostTpl, PCI_VENDOR_ID_CMD,
	                      PCI_DEVICE_ID_CMD_649, 0);
	adp += mide_find_card(hostTpl, PCI_VENDOR_ID_CMD,
	                      PCI_DEVICE_ID_DELL_680, 1);
	adp += mide_find_card(hostTpl, PCI_VENDOR_ID_CMD,
	                      PCI_DEVICE_ID_CMD_680, 1);
	adp += mide_find_card(hostTpl, PCI_VENDOR_ID_CMD,
	                      PCI_DEVICE_ID_CMD_3112, 1);
	adp += mide_find_card(hostTpl, PCI_VENDOR_ID_SERVERWORKS,
	                      PCI_DEVICE_ID_SERVERWORKS_CSB6IDE, 1);
	
	adp += mide_find_card(hostTpl, PCI_VENDOR_ID_INTEL,
	                      PCI_DEVICE_ID_INTEL_ICH5R, 1);
	
	adp += mide_find_card(hostTpl, PCI_VENDOR_ID_INTEL,
	                      PCI_DEVICE_ID_INTEL_HR, 1);

	giNumberOfCtrlrs = adp;
	if (adp) {
#ifdef CONFIG_PROC_FS
		megaideproc_attach();
#endif
		megaidedev_attach();
		/*
		 * Register the Shutdown Notification hook in kernel
		 */
		if(register_reboot_notifier(&megaide_reboot_notifier)) {
			printk (KERN_INFO "megaide: shutdown routine not registered!!\n");
		}
	}

	return(adp);
}/*megaide_detect()*/


/** Description
 *	Driver entry point to the scsi mid-layer,
 *	Get information about the card/driver
 * Return Value
 *	Pointer to a static character array containing a
 *	brief description of the driver
 */

const char *
megaide_info(struct Scsi_Host * host)
{
	static char acBuffer[512];
	HW_DEVICE_EXTENSION *pdev;
	adapter_t *adapter;

	AR_DEBUG(AR_DBG_ENTRY, (KERN_DEBUG "megaide_info Called\n"));

	adapter = (adapter_t *) host->hostdata;
	pdev = (HW_DEVICE_EXTENSION *) &adapter->DeviceExtension;
	if ((pdev->stFirmwareVersion.usMajorVer == 0) &&
	    (pdev->stFirmwareVersion.usMinorVer == 0) &&
	    (pdev->stFirmwareVersion.ulBuild == 0)) {
		
		//Bios disabled
		sprintf(acBuffer,
"LSI Logic MegaIDE RAID BIOS Disabled, %d targs %d chans %d luns",
		        host->max_id,
		        host->max_channel,
		        host->max_lun);
	} else {
	
		sprintf(acBuffer,
"LSI Logic MegaIDE RAID BIOS Version %d.%d.%8.8ld, %d targs %d chans %d luns",
		        pdev->stFirmwareVersion.usMajorVer,
		        pdev->stFirmwareVersion.usMinorVer,
		        pdev->stFirmwareVersion.ulBuild,
		        host->max_id,
		        host->max_channel,
		        host->max_lun);
	}
	
	return acBuffer;
}/*megaide_info()*/


/** Description
 *	This function Release the controller's resources
 * Return Value
 *	Returns Zero if command succeds
 */
int
megaide_release(struct Scsi_Host *host)
{
	HW_DEVICE_EXTENSION *pdev;
	adapter_t *adapter;

	AR_DEBUG(AR_DBG_ENTRY, (KERN_DEBUG "megaide _release Called\n"));

	adapter = (adapter_t *) host->hostdata;
	pdev = (HW_DEVICE_EXTENSION *) &adapter->DeviceExtension;


	free_irq(host->irq, adapter);

	DisableInterrupts(pdev);

	/*kill ICH5R polling thread*/
	if (pdev->ucCardType == INTEL_ICH5R) {
		mega_kill_ICH5R_poll_thread(pdev);
	}
	
	FlushCache(pdev);

	EnableInterrupts(pdev);

	ReleaseOurIOPorts(pdev);

	/*deallocate the momory allocated in each adapter*/
	CleanUpAdapter(pdev);

	mide_free_cmd_memory_pool (pdev);

	giNumberOfCtrlrs --;

	if (!giNumberOfCtrlrs)
		unregister_reboot_notifier(&megaide_reboot_notifier);
	
	/*Unregister the driver*/
	scsi_unregister(host);

	/*Remove /proc directories for driver debugging*/
#ifdef CONFIG_PROC_FS
	megaideproc_detach(pdev);
#endif

	/* Un Register character Driver interface */
	megaidedev_detach();

	return 0;
}/*megaide_release()*/


/** Description
 *	This function issues a command and does not wait for it to finish.
 *	When the command is finished, the done() function is called with the
 *	Scsi_Cmnd structure pointer as a parameter. This function is called
 *	only if the can_queue variable is non-zero.
 *	This function is called by SCSI midlayer only
 *
 * Arguments - Input
 *	Pointer to the Scsi_Cmnd structure,
 *	Pointer to the callback function
 *
 * Arguments - Output
 *
 * Return Value
 *	Returns zero if command succeeds
 */
int
megaide_queue(Scsi_Cmnd * scmd, void(*scsi_done)(Scsi_Cmnd *))
{
	int busy = 0;
	adapter_t *adapter;
	HW_DEVICE_EXTENSION *pdev;

	adapter = (adapter_t *) scmd->host->hostdata;
	pdev = (HW_DEVICE_EXTENSION *) &adapter->DeviceExtension;

	scmd->scsi_done = scsi_done;
	/*Going to Process the command*/
	busy = FastIDEProcessAndQueueRequest(pdev, scmd);

	return busy;
}/*megaide_queue()*/


ULONG
FastIDEProcessAndQueueRequest(HW_DEVICE_EXTENSION *pdev, Scsi_Cmnd *scmd)
{
	LOGICAL_DRIVE_INFO *ldInfo;
	FAST_IDE_SCB_EXTENSION *scb;

	ldInfo = GetLDInfo(pdev, scmd->channel, scmd->target, scmd->lun);

	if (!ldInfo) {
		scmd->result = (DID_BAD_TARGET << 16); /* No such device*/
		scmd->scsi_done(scmd);
		return 0;
	}

	if ((ldInfo->Status == LDS_OffLine)) {
		UCHAR scbStatus = 0;
		UCHAR scsiStatus = 0;

		scsiStatus = COMMAND_TERMINATED << 1;
		scbStatus =  DID_BAD_TARGET;
		scmd->result = (scbStatus) <<16 |scsiStatus;
		scmd->device->online = FALSE;
		printk_Rlsi(KERN_ERR "megaide: LD Offline!!\n");
		scmd->scsi_done(scmd);
		return 0;
	}

	check_for_config_update(pdev);
	scmd->host_scribble = 0;

	switch (scmd->cmnd[0]) {
		case VERIFY :
		case READ_6:
		case READ_10:
		case WRITE_6:
		case WRITE_10:
		case WRITE_VERIFY:
		{

		  if (ldInfo->Status == LDS_Deleted) {
		    scmd->result = (DID_BAD_TARGET << 16); /*No such device*/
		    scmd->scsi_done(scmd);
		    return 0;
		  }

		  /*alloc host_scribble*/
		  scmd->host_scribble =
		      mide_scsi_malloc(pdev,
		                       get_scb_extension_size());
		  scb = (FAST_IDE_SCB_EXTENSION *) scmd->host_scribble;

		  if (scb == NULL) {
		    printk_Rlsi(KERN_WARNING
		                "megaide: host_scribble mem alloc failed \n");
		    scmd->result = (DID_ERROR << 16);
		    scmd->scsi_done (scmd);
		    /*non-zero status, could not find memory*/
		    return 1;
		  }

		    memset_Rlsi(scb, 0, get_scb_extension_size());

		    /*os specific scsi packet stuff copied to srb extension*/
		    scb->pstScsiCmd = (VOID *)scmd;
		    scb->target = scmd->target;
		    scb->lun = scmd->lun;
		    scb->channel = scmd->channel;
#ifdef PERFORMANCE_NUMBER
		    pdev->io_count++;
		    scb->time = jiffies;
#endif
		    memcpy_Rlsi(scb->scsi_cdb, scmd->cmnd, MAX_COMMAND_SIZE);
		    scb->cdb_len = scmd->cmd_len;
		    scb->request_buffer = (VOID *)scmd->buffer;
		    scb->sglem = scmd->use_sg;

		   if(! megaide_atapi_startio(pdev,scb)) {
		      mide_scsi_free(pdev, scmd->host_scribble,
		                     get_scb_extension_size());
		      scmd->result = (DID_BAD_TARGET << 16);
		      scmd->host_scribble = 0;
		      scmd->scsi_done(scmd);
		   }
		   break;
		}

		case READ_CAPACITY:
		{
		  ULONG bytesPerBlock = IDE_SECTOR_SIZE;
		  PREAD_CAPACITY_DATA readCapacity;
		  ULONG lsize;

		  memset_Rlsi(scmd->buffer, 0, scmd->request_bufflen);
		  readCapacity = (PREAD_CAPACITY_DATA) scmd->buffer;
		  //reporting logical block size
		  lsize = ldInfo->Sectors - 1;
		  readCapacity->BytesPerBlock = BIG_ENDIAN_ULONG(bytesPerBlock);
		  readCapacity->LogicalBlockAddress = BIG_ENDIAN_ULONG(lsize);
		  scmd->result = (DID_OK << 16);
		  scmd->scsi_done(scmd);
		  break;
		}

		case TEST_UNIT_READY:
		  scmd->result = (DID_OK << 16);
		  scmd->scsi_done(scmd);
		  break;

		case MODE_SENSE :
		  scmd->result = (DID_BAD_TARGET << 16);
		  scmd->scsi_done(scmd);
		  break;

		case INQUIRY:
		{
		  INQUIRYDATA *pInq;

		  memset_Rlsi(scmd->buffer, 0, scmd->request_bufflen);
		  pInq = (INQUIRYDATA *) scmd->buffer;
		  pInq->Versions = 2;
		  pInq->ResponseDataFormat = 2;
		  ((UCHAR *) pInq) [7] = 0x32;
		  pInq->AdditionalLength = 0xFA;

		  /*Fill Actual Vendor ID depending on  SubsystemVendorID*/
		  memmove_Rlsi(pInq->ProductId, OemProductId, 16);

		  /*Fill Actual Product ID depending on  SubsystemVendorID*/
		  memmove_Rlsi(pInq->VendorId, OemVendorId, 8);
		  pInq->ProductId[4] = ' ';/*higher digit */
		  /*lower digit*/
		  pInq->ProductId[5] = (UCHAR)ldInfo->LogicalDriveNumber + '0';
 		  scmd->result = (DID_OK << 16);
		  scmd->scsi_done(scmd);
		  break;
		}

		case REQUEST_SENSE :
		{
		  memset_Rlsi(scmd->buffer, 0, scmd->request_bufflen);
		  ((UCHAR *) scmd->buffer) [0] = 0x70;
		  ((UCHAR *) scmd->buffer) [7] = 0x18;
		  ((UCHAR *) scmd->buffer) [8] = 0xFF;
		  ((UCHAR *) scmd->buffer) [9] = 0xFF;
		  ((UCHAR *) scmd->buffer) [10] = 0xFF;
		  ((UCHAR *) scmd->buffer) [11] = 0xFF;
		  ((UCHAR *) scmd->buffer) [19] = 0xFF;
		  ((UCHAR *) scmd->buffer) [20] = 0xFF;
		  ((UCHAR *) scmd->buffer) [21] = 0xFF;
		  ((UCHAR *) scmd->buffer) [22] = 0xFF;
		  ((UCHAR *) scmd->buffer) [23] = 0xFF;
		  scmd->result = (DID_OK << 16);
		  scmd->scsi_done(scmd);
		  break;
		}

		case FORMAT_UNIT :
		  scmd->result = (DID_BAD_TARGET << 16);
		  scmd->scsi_done(scmd);
		  break;

		case MEGAIDE_IOCTL_NEW:
		case MEGAIDE_IOCTL :
		{
		  /*alloc host_scribble */
		  scmd->host_scribble =
		      mide_scsi_malloc(pdev,
		                       get_scb_extension_size());
		  scb = (FAST_IDE_SCB_EXTENSION *) scmd->host_scribble;

		  if (scb == NULL) {
		    printk_Rlsi(KERN_WARNING
		                "megaide: unable to allocate host_scribble\n");
		    scmd->result = (DID_ERROR << 16);
		    scmd->scsi_done (scmd);
		   /*non-zero status, could not find memory*/
		    return 1;
		  }

		  memset_Rlsi(scb, 0, get_scb_extension_size());
		  /*os specific scsi packet stuff copied to srb extension*/
		  scb->pstScsiCmd = (VOID *)scmd;
		  scb->target = scmd->target;
		  scb->lun = scmd->lun;
		  scb->channel = scmd->channel;
		  memcpy_Rlsi(scb->scsi_cdb, scmd->cmnd, MAX_COMMAND_SIZE);
		  scb->cdb_len = scmd->cmd_len;
		  scb->request_buffer = (VOID *)scmd->buffer;
		  scb->sglem = scmd->use_sg;

		  switch (HandleIOCTLCall(pdev, scb))
		  {
		    case IOCTL_OK:
		      megaide_complete_command(pdev, scb, (DID_OK << 16));
		      break;

		    case IOCTL_ERROR:
		      megaide_complete_command(pdev, scb,
		                               (DID_BAD_TARGET << 16));
		      AR_DEBUG(AR_DEBUG_IOCTL, (KERN_DEBUG
		               "megaide: IOCTL err\n"));
		      break;

		    case IOCTL_QUEUE:
		      AR_DEBUG(AR_DEBUG_IOCTL,
		               (KERN_DEBUG "megaide: IOCTL queued\n"));
		      break;

		    default:
		      break;
		  }
		  break;
		}

		default:
		   scmd->result = (DID_BAD_TARGET << 16);
		   scmd->scsi_done(scmd);
		   break;
	}
	return 0;
} /*FastIDEProcessAndQueueRequest*/


void
megaide_complete_command(HW_DEVICE_EXTENSION *pdev,
                         FAST_IDE_SCB_EXTENSION *scb, int result)
{
	Scsi_Cmnd *scmd;
	scmd = (Scsi_Cmnd *) scb->pstScsiCmd;

	if(! scmd) { return; }

	if(scmd->host_scribble) {
		/* IOCTL memory is allocated by driver,
		 * not by scsi, so handled separately
		 */
		if (scmd->state != MIDE_CHAR_IOCTL_CALLED) {
			mide_scsi_free(pdev, scmd->host_scribble,
			               get_scb_extension_size());
		}
		scmd->host_scribble = 0;
	}

	scmd->result = result;
	scmd->scsi_done(scmd);
}

/* Description
 *	The command function issues a command synchronously and waits for
 *	it to complete. In our driver we should put these commands also
 *	in our own queue command function.
 *	This function is called by SCSI midlayer only
 *
 * Arguments - Input
 *	Pointer to the Scsi_Cmnd structure
 *
 * Arguments - Output
 *
 * Return Value
 *	Returns the result of scsi command
 */

int
megaide_command(Scsi_Cmnd * scmd)
{
	HW_DEVICE_EXTENSION *pdev;
	adapter_t *adapter;

	AR_DEBUG(AR_DBG_ENTRY, (KERN_DEBUG "megaide _command Called\n"));

	giInternalDoneFlag = 0;
	adapter = (adapter_t *) scmd->host->hostdata;
	pdev = (HW_DEVICE_EXTENSION *) &adapter->DeviceExtension;

	//ucControllerId = pdev->ucControllerId;

	/*this command should go to the head of the queue*/
	pdev->uiFlag |= M_IDE_BLOCK_CMD;

	/* Queue command, and wait until it has completed */
	megaide_queue(scmd, mide_internal_done);

	/*Wait till command get over*/
	while (!giInternalDoneFlag) {
		interruptible_sleep_on(&adapter->internal_wait);
	}
	return giInternalDoneErrcode;
}/*megaide_command()*/


void
mide_internal_done(Scsi_Cmnd * scmd)
{
	HW_DEVICE_EXTENSION *pdev;
	adapter_t *adapter;

	adapter = (adapter_t *) scmd->host->hostdata;
	pdev = (HW_DEVICE_EXTENSION *) &adapter->DeviceExtension;
	giInternalDoneErrcode = scmd->result;
	giInternalDoneFlag++;
	wake_up(&adapter->internal_wait);
}/*mide_internal_done()*/


/* Description
 *	This is used to request that the currently outstanding SCSI command,
 *	indicated by the Scsi_Cmnd pointer, be aborted.
 *	The SCSI layer will first attempt to abort the command then if need
 *	be start using a larger hammer on the problem until it gets to the
 *	point of trying to reset the entire controller.
 *	This function is called by SCSI midlayer only
 *
 * Arguments - Input
 *	Pointer to the Scsi_Cmnd structure
 *
 * Arguments - Output
 *
 * Return Value
 *	SCSI_ABORT_SNOOZE		-	We are not doing anything
 *	SCSI_ABORT_SUCCESS		-	Abort command success
 *	SCSI_ABORT_PENDING		-	Called abort,
 *						waiting for interrupt
 *	SCSI_ABORT_BUSY			-	Unable to Abort
 *	SCSI_ABORT_ERROR		-	Something went wrong!!
 */

int megaide_abort(Scsi_Cmnd * scmd)
{
	HW_DEVICE_EXTENSION *pdev;
	adapter_t *adapter;
	PLOGICAL_DRIVE_INFO ldInfo;
	FAST_IDE_SCB_EXTENSION *scb = NULL;

	adapter = (adapter_t *) scmd->host->hostdata;
	pdev = (HW_DEVICE_EXTENSION *) &adapter->DeviceExtension;

	AR_DEBUG(AR_DBG_ENTRY, (KERN_DEBUG "megaide _abort  Called\n"));

	if ((scmd->cmnd[0] == MEGAIDE_IOCTL_NEW) ||
		(scmd->cmnd[0] == MEGAIDE_IOCTL)) {
		scb = (FAST_IDE_SCB_EXTENSION *) scmd->host_scribble;
		megaide_abort_ioctl(pdev, scb);
		return SCSI_ABORT_SUCCESS;
	}

	ldInfo = GetLDInfo(pdev, scmd->channel, scmd->target, scmd->lun);

	if (ldInfo) {
		if (ldInfo->Status == LDS_Deleted) {
			/*No such device*/
			scmd->result = (DID_BAD_TARGET << 16);
			scmd->scsi_done(scmd);
			return SCSI_ABORT_SUCCESS;
		}

		if (ldInfo->Status == LDS_OffLine) {
			UCHAR scbStatus = 0;
			UCHAR scsiStatus = 0;

			scsiStatus = COMMAND_TERMINATED << 1;
			scbStatus = DID_BAD_TARGET;
			scmd->result = (scbStatus) << 16 | scsiStatus;
			printk_Rlsi(KERN_ERR "megaide: Array is Offline \n");
			scmd->device->online = FALSE;
			scmd->scsi_done(scmd);
			return SCSI_ABORT_SUCCESS;
		}
	}

	printk_Rlsi(KERN_INFO "megaide: snooze and wait for reset \n");
	return(SCSI_ABORT_SNOOZE);
}/*megaide_abort()*/


/** Description
 *	Reset a previous SCSI request.
 *	This function is called by SCSI midlayer only
 *
 * Arguments - Input
 *	Pointer to the Scsi_Cmnd structure
 *	int i_ResetFlags
 *
 * Arguments - Output
 *
 * Return Value
 *	SCSI_RESET_SNOOZE	-	We are not doing anything,
 *					Command may finish after some time
 *	SCSI_RESET_PUNT		-	We are not doing anything,
 *					Given up, mid_level can proceed
 *	SCSI_RESET_SUCCESS	-	Reset command success
 *	SCSI_RESET_PENDING	-	Called reset, waiting for interrupt
 *	SCSI_RESET_WAKEUP	-	Did reset, but do not expect
 *					an interrupt to signal DID_RESET
 *	SCSI_RESET_NOT_RUNNING	-	Command is not active in Low level code
 *					Probably finished
 *	SCSI_RESET_ERROR	-	Something went wrong!!
 */
int 
megaide_reset(Scsi_Cmnd * scmd, unsigned int i_ResetFlags)
{
	int iRetVal;
	PLOGICAL_DRIVE_INFO ldInfo;
	adapter_t *adapter;
	HW_DEVICE_EXTENSION *pdev ;
	
	adapter = (adapter_t *) scmd->host->hostdata;
	pdev = (HW_DEVICE_EXTENSION *) &adapter->DeviceExtension;

	ldInfo = GetLDInfo(pdev, scmd->channel, scmd->target, scmd->lun);

	if (!ldInfo || ldInfo->Status == LDS_OffLine) {
		return SCSI_RESET_SUCCESS;
	}

	if (FastIDEResetBus(pdev) == TRUE) {
		iRetVal = SCSI_RESET_SUCCESS;
	} else {
		iRetVal = SCSI_RESET_ERROR;
	}
	return(iRetVal);
}/*megaide_reset()*/



/** Description
 *	Finally the mapping of SCSI to PC disk geometry has never been
 *	exactly a standard. The bios_param function is called by the SCSI
 *	layer to ask the controller to either query its BIOS for the faked
 *	disk geometry or to compute a geometry(hopefully using the same
 *	algorithm as the controller BIOS itself).
 *
 * Arguments - Input
 *   Disk *disk		- Disk geometry
 *   kdev_t dev		- Device node
 *   int *pi_geom	- Returns geometry fields
 *     geom[0] = heads
 *     geom[1] = sectors
 *     geom[2] = cylinders
 * Arguments - Output
 *   int *pi_geom	- geometry fields
 *
 * Return Value
 *	Returns zero if command succeeds
 */
int 
megaide_biosparam(Disk * disk, kdev_t dev, int * piGeom)
{
	int heads, sectors, cylinders;
	HW_DEVICE_EXTENSION *pdev;
	adapter_t *adapter;

	AR_DEBUG(AR_DBG_ENTRY, (KERN_DEBUG "megaide: biosparam Called\n"));
	/* Get pointer to host config structure */
	adapter = (adapter_t *) disk->device->host->hostdata;
	pdev = (HW_DEVICE_EXTENSION *) &adapter->DeviceExtension;

	if (disk->capacity < 1032192) { /* size less than 504MB */
		heads = 16;
		sectors = 63;
		cylinders = (int) disk->capacity / (heads * sectors);
	} else if (disk->capacity < 2064384) { /* size less than 1008MB */
		heads = 32;
		sectors = 63;
		cylinders = (int) disk->capacity / (heads * sectors);
	} else if (disk->capacity < 4128768) { /* size less than 2016MB */
		heads = 64;
		sectors = 63;
		cylinders = (int) disk->capacity / (heads * sectors);
	} else if (disk->capacity < 8257536) { /* size less than 4032MB */
		heads = 128;
		sectors = 63;
		cylinders = (int) disk->capacity / (heads * sectors);
	} else { /* size greater than 4032MB */
		heads = 255;
		sectors = 63;
		cylinders = (int) disk->capacity / (heads * sectors);
	}

	/* return result */
	piGeom[0] = heads;
	piGeom[1] = sectors;
	piGeom[2] = cylinders;

	return 0;
}/*megaide_biosparam()*/

/** Description
 *	Interrupt service routine. This routine should register at the
 *	time of requesting IRQ.
 *
 * Arguments - Input
 *
 *	int iIrq	-	IRQ number
 *	void *vpDdevp	-	Device specific field passed to the ISR routine
 *				by the kernel. Here it is megaide_host_config*
 *	struct pt_regs * pstRegs
 * 			-	Stucture defines the way the registers
 *				are stored on the stack during system call
 * Arguments - Output
 *	None
 * Return Value
 *	Returns nothing
 */

void
megaide_isr(int iIrq, void *vpDevp, struct pt_regs * pstRegs)
{
	unsigned long 		flags;
	adapter_t *adapter;

	AR_DEBUG(AR_DEBUG_INTERUPT, (KERN_DEBUG "megaide _isr Called\n"));
	spin_lock_irqsave(&io_request_lock, flags);
	adapter = (adapter_t *) vpDevp;
	FastIDEInterrupt((HW_DEVICE_EXTENSION *) &adapter->DeviceExtension);
	spin_unlock_irqrestore(&io_request_lock, flags);
}/*megaide_isr()*/

/*new error handler entry points*/
int
eh_megaide_abort(Scsi_Cmnd *scmd)
{
	ulong ret_val = 0;

	AR_DEBUG (AR_DBG_ENTRY,(KERN_DEBUG "eh_megaide_abort() Called\n"));
	ret_val = megaide_abort(scmd);

	return ((SCSI_ABORT_SUCCESS == ret_val)? SUCCESS: FAILED);
}

int 
eh_megaide_reset_bus(Scsi_Cmnd *scmd)
{
	ulong ret_val = 0;

	AR_DEBUG (AR_DBG_ENTRY,(KERN_DEBUG "eh_megaide_reset_bus() Called\n"));
	ret_val= megaide_reset(scmd, 0);

	return ((SCSI_RESET_SUCCESS == ret_val)? SUCCESS: FAILED);
}

int 
eh_megaide_reset_host(Scsi_Cmnd *scmd)
{
	AR_DEBUG(AR_DBG_ENTRY,(KERN_DEBUG "eh_megaide_reset_host() Called\n"));
	return eh_megaide_reset_bus(scmd);
}

int eh_megaide_reset_dev(Scsi_Cmnd *scmd)
{
	AR_DEBUG(AR_DBG_ENTRY,(KERN_DEBUG "eh_megaide_reset_dev() Called\n"));
	return eh_megaide_reset_bus(scmd);
}
/*new error handler entry points*/

int
mide_scsi_free (HW_DEVICE_EXTENSION *pdev, void *data, unsigned int length)
{
	mide_release_scb_pool_memory (pdev, (FAST_IDE_SCB_EXTENSION *) data);
	return TRUE;
}

/**
 * megaide_reboot_notify()
 * @this - unused
 * @code - shutdown code
 * @unused - unused
 *
 * This routine will be called when the use has done a forced shutdown on the
 * system. Flush the Adapter and disks cache.
 */
static int
megaide_reboot_notify (struct notifier_block *this, unsigned long code,
		void *unused)
{
	PHW_DEVICE_EXTENSION pdev;
	adapter_t * adapter;
	int i;

	printk (KERN_INFO "megaide: reboot notify called!\n");
	for (i=0; i<giNumberOfCtrlrs; i++)
	{
		adapter = (adapter_t *)gastCardInfo[0];
		pdev = &adapter->DeviceExtension;

		free_irq (pdev->ulIntLine, pdev);

		/*kill ICH5R polling thread*/
		if (pdev->ucCardType == INTEL_ICH5R) {
			mega_kill_ICH5R_poll_thread(pdev);
		}
		
		DisableInterrupts(pdev);
		FlushCache(pdev);
		EnableInterrupts(pdev);
	}
	return NOTIFY_DONE;
}


void *
mide_scsi_malloc (HW_DEVICE_EXTENSION *pdev, unsigned int length)
{
	return ((void *)mide_get_scb_from_pool (pdev));
}

int
mide_init_cmd_memory_pool(HW_DEVICE_EXTENSION *pdev)
{
	int i=0;
	FAST_IDE_SCB_EXTENSION *pScbExt;
	adapter_t * adapter = (adapter_t *) pdev->adapter;
	INIT_LIST_HEAD (&adapter->scb_ext_free_list);

	for (i=0; i<M_IDE_TOTAL_COMMANDS; i++)
	{
		pScbExt = kmalloc (get_scb_extension_size(), GFP_KERNEL);
		list_add((struct list_head *)&pScbExt->list,
		         &adapter->scb_ext_free_list);
	}
	return TRUE;
}/*init scb extention pool*/


FAST_IDE_SCB_EXTENSION *
mide_get_scb_from_pool(HW_DEVICE_EXTENSION *pdev)
{
	adapter_t * adapter = (adapter_t *) pdev->adapter;
	struct list_head *head = &adapter->scb_ext_free_list;
	FAST_IDE_SCB_EXTENSION *pScbExt = NULL;

	if (!list_empty(head)) {
		pScbExt = list_entry(head->next, FAST_IDE_SCB_EXTENSION, list);
		list_del_init(head->next);
	}

	return (pScbExt);
}

int
mide_release_scb_pool_memory (HW_DEVICE_EXTENSION *pdev, 
                              FAST_IDE_SCB_EXTENSION *pScbExt)
{
//	memset (pScbExt, 0, sizeof(FAST_IDE_SCB_EXTENSION));
	adapter_t * adapter = (adapter_t *) pdev->adapter;
	list_add ((struct list_head *)&pScbExt->list,
	          &adapter->scb_ext_free_list);
	return TRUE;
}

int
mide_free_cmd_memory_pool(HW_DEVICE_EXTENSION *pdev)
{
	struct list_head *pos, *next;
	adapter_t * adapter = (adapter_t *) pdev->adapter;
	FAST_IDE_SCB_EXTENSION *pScbExt = NULL;

	list_for_each_safe(pos, next, &adapter->scb_ext_free_list) {
		pScbExt = list_entry(pos, FAST_IDE_SCB_EXTENSION, list);
		list_del_init((struct list_head *)&pScbExt->list);
		kfree (pScbExt);
	}
	return TRUE;
}

void
fill_sense_buffer(FAST_IDE_SCB_EXTENSION *scb , UCHAR errByte,
                   BOOLEAN rwError , ULONG errSector)
{
	Scsi_Cmnd * scmd;

	scmd = (Scsi_Cmnd *) scb->pstScsiCmd;

	if (errByte &(ABRT_ERR | ID_ERR |  MCR_ERR)) {
		if (scmd->sense_buffer) {
			scmd->sense_buffer[0] = 0x70;
			scmd->sense_buffer[2] = ABORTED_COMMAND;
		}
	}

	if (errByte &(ECC_ERR | ICRC_ERR)) {
		if (scmd->sense_buffer) {
			scmd->sense_buffer[0] = rwError ? 0x70:0xF0;
			scmd->sense_buffer[2] = rwError ? 0:MEDIUM_ERROR;
			/*set up LBA address*/
			scmd->sense_buffer[3] = (errSector >> 24) & 0xff;
			scmd->sense_buffer[4] = (errSector >> 16) & 0xff;
			scmd->sense_buffer[5] = (errSector >> 8) & 0xff;
			scmd->sense_buffer[6] =  errSector & 0xff;
			scmd->sense_buffer[7] =  6;

			if (rwError == 0) { /*read error*/
				scmd->sense_buffer[12] = 0x11;
				scmd->sense_buffer[13] = 0x0;
			}

			if (rwError == 1) { /*write error */
				scmd->sense_buffer[12] = 0x0C;
				scmd->sense_buffer[13] = 0x0;
			}
		}
	}
} /*fill sense buffer*/


ULONG 
GetPhysicalAddressAndLength(void *buffer, unsigned short sglem,
                            ULONG offset, ULONG *pulContiguousLength)
{
	ULONG phyAddr;
	struct scatterlist *sgl;
	ULONG curLen, sglInd;

	sgl = (struct scatterlist *) buffer;
	*pulContiguousLength = 0;
	phyAddr = 0;

	for (sglInd=0, curLen = 0; sglInd < sglem; sglInd++)
	{
		if ((curLen + sgl[sglInd].length) > offset) {
		  /*This is the SGL Element */
		  *pulContiguousLength =
		       sg_dma_len(& sgl[sglInd])-(offset - curLen);
		   phyAddr =
		       (ULONG)sg_dma_address (&sgl[sglInd])+(offset - curLen);
		   break;
		}
		curLen += sg_dma_len(& sgl[sglInd]);
	}
	return phyAddr;
}/*GetPhysicalAddressAndLength*/
/*vi set ts=8:*/

